/* 
 Copyright (c) 2014-2024 Marvell International Ltd., All Rights Reserved
 Redistribution and use in source and binary forms, with or without modification, 
 are permitted provided that the following conditions are met:

 1. Redistributions of source code must retain the above copyright notice, this list 
    of conditions and the following disclaimer.
 2. Redistributions in binary form must reproduce the above copyright notice, this list 
    of conditions and the following disclaimer in the documentation and/or other materials 
    provided with the distribution.
    
 THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY EXPRESS 
 OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY 
 AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR 
 CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR 
 CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR 
 SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY 
 THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR 
 OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE 
 POSSIBILITY OF SUCH DAMAGE.
*/

#include "openSourceMtdAPITypes.h"
#include "openSourceMtdUser.h"
#include "openSourceMtdAPI.h"
#include "openSourceMtdAPIDevices.h"

/*******************************************************************
 Local Function Prototypes
 *******************************************************************/
MTD_STATIC MTD_STATUS mtdAutonegIsSpeedDuplexResolutionDone
(
    IN MTD_DEV_PTR devPtr,
    IN MTD_U16 port,
    OUT MTD_BOOL *anSpeedResolutionDone
);

MTD_STATIC MTD_STATUS mtdAutonegIsCompleted
(
    IN MTD_DEV_PTR devPtr,
    IN MTD_U16 port,
    OUT MTD_BOOL *anStatusReady
);

MTD_STATIC MTD_STATUS mtdReadMasterOrSlaveBit
(
    IN MTD_DEV_PTR devPtr,
    IN MTD_U16 port,
    OUT MTD_BOOL *msBitFlag
);

MTD_STATIC MTD_STATUS mtdGetMasterSlaveConfigFault
(
    IN MTD_DEV_PTR devPtr,
    IN MTD_U16 port,
    OUT MTD_BOOL *msConfigFaultFlag
);



/*******************************************************************/
/*******************************************************************
 API Version
 Resets
 Download Mode Selection/Query
 Low Power Mode
 MDIO Download Mode
 Firmware Download
 Firmware version
 *******************************************************************/

MTD_VOID mtdGetAPIVersion
(
    OUT MTD_U8 *major,
    OUT MTD_U8 *minor,
    OUT MTD_U8 *buildID
)
{
    *major = MTD_API_MAJOR_VERSION;
    *minor = MTD_API_MINOR_VERSION;
    *buildID = MTD_API_BUILD_ID;
}

MTD_STATUS mtdSoftwareReset
(
    IN MTD_DEV_PTR devPtr,
    IN MTD_U16 port,
    IN MTD_U16 timeoutMs
)
{
    MTD_U16 counter;
    MTD_BOOL phyReady;
    
    MTD_ATTEMPT(mtdHwSetPhyRegField(devPtr,port,MTD_TUNIT_IEEE_PMA_CTRL1,15,1,1)); /* bit self clears when done */

    if (timeoutMs)
    {
        counter=0;
        MTD_ATTEMPT(mtdIsPhyReadyAfterReset(devPtr,port,&phyReady));
        while (phyReady == MTD_FALSE && counter <= timeoutMs)
        {
            MTD_ATTEMPT(mtdWait(devPtr,1));
            MTD_ATTEMPT(mtdIsPhyReadyAfterReset(devPtr,port,&phyReady));
            counter++;
        }

        if (counter < timeoutMs)
        {
            return MTD_OK;
        }
        else
        {
            return MTD_FAIL; /* timed out without becoming ready */
        }
    }
    else
    {
        return MTD_OK;
    }
}

MTD_STATUS mtdChipHardwareReset
(
    IN MTD_DEV_PTR devPtr,
    IN MTD_U16 port
)
{
    MTD_U16 lastPortMdioAddr, basePort, upperHalfFirstPort;

    if (MTD_IS_X33X0_BASE(devPtr->deviceId))
    {
        /* 88X33X0 for chip reset must use last port for full reset */
        /* redirect to last port*/
        if (devPtr->numPortsPerDevice == 0) /* check for divide by 0 */
        {
            return MTD_FAIL;
        }
        basePort = (port/(devPtr->numPortsPerDevice)) * (devPtr->numPortsPerDevice);
        lastPortMdioAddr = (basePort + (devPtr->numPortsPerDevice)) - 1;
        return (mtdHwSetPhyRegField(devPtr,lastPortMdioAddr,MTD_CUNIT_PORT_CTRL,14,1,1));
    }
    else
    {
        if (MTD_IS_X3580_E2580_DEVICE(devPtr))
        {
            /* Need to reset both sides of device */
            if (devPtr->numPortsPerDevice == 0) /* check for divide by 0 */
            {
                return MTD_FAIL;
            }        
            basePort = (port/(devPtr->numPortsPerDevice)) * (devPtr->numPortsPerDevice);
            upperHalfFirstPort = basePort + 4;
          
            MTD_ATTEMPT(mtdHwXmdioWrite(devPtr,upperHalfFirstPort,MTD_CUNIT_PORT_CTRL,(1<<14)));             
            MTD_ATTEMPT(mtdWait(devPtr, 225)); /* need to wait 225ms to reset other 4 ports */
            MTD_ATTEMPT(mtdHwXmdioWrite(devPtr,basePort,MTD_CUNIT_PORT_CTRL,(1<<14)));
            MTD_ATTEMPT(mtdWait(devPtr, 10)); /* need to wait 10ms before any MDIO activity after reset */            
            return MTD_OK;
        }
        /* else/all others */
       
        return (mtdHwSetPhyRegField(devPtr,port,MTD_CUNIT_PORT_CTRL,14,1,1));
    }

    return MTD_FAIL; /* should never come to here */
}


#define MTD_BOOT_PROGRESS_POS    (1)
#define MTD_CODE_STARTED_POS     (4)
#define MTD_BOOT_PROGRESS_MASK    (0x3<<MTD_BOOT_PROGRESS_POS)
#define MTD_CODE_STARTED_MASK     (0x1<<MTD_CODE_STARTED_POS)

#define MTD_WAIT_DL          0x01

MTD_STATUS mtdIsPhyInMdioDownloadMode
(
    IN MTD_DEV_PTR devPtr,
    IN MTD_U16 port,
    OUT MTD_BOOL *downloadMode
)
{    
    MTD_U16 val;

    *downloadMode = MTD_FALSE;

    MTD_ATTEMPT(mtdHwXmdioRead(devPtr,port,MTD_BOOT_STATUS_REG,&val));
    
    if (((val & MTD_BOOT_PROGRESS_MASK)>>MTD_BOOT_PROGRESS_POS) == MTD_WAIT_DL)
    {
        *downloadMode =  MTD_TRUE;
    }
    else
    {
        *downloadMode = MTD_FALSE;
    }
    return MTD_OK;
}


MTD_STATUS mtdDidPhyAppCodeStart
(
    IN MTD_DEV_PTR devPtr,
    IN MTD_U16 port,
    OUT MTD_BOOL *appStarted
)
{
    MTD_U16 val;

    *appStarted = MTD_FALSE;    

    MTD_ATTEMPT(mtdHwXmdioRead(devPtr,port,MTD_BOOT_STATUS_REG,&val));

    if ((val & MTD_CODE_STARTED_MASK)>>MTD_CODE_STARTED_POS)
    {
        *appStarted = MTD_TRUE;
    }
    return MTD_OK;
}


MTD_STATUS mtdIsTunitResponsive
(
    IN MTD_DEV_PTR devPtr,
    IN MTD_U16 port,
    OUT MTD_BOOL *tunitReady
)
{
    MTD_BOOL lowpowermode;
    MTD_U8 major, minor, inc, test; /* for firmware version */
    MTD_U16 bistStatus, tunitMode, temp;
    
    *tunitReady = MTD_TRUE;

    MTD_ATTEMPT(mtdIsPhyReadyAfterReset(devPtr,port,tunitReady));

    if (*tunitReady == MTD_FALSE)
    {
        return MTD_OK;
    }

    MTD_ATTEMPT(mtdIsTunitInLowPowerMode(devPtr,port,&lowpowermode));

    if (lowpowermode == MTD_TRUE)
    {
        *tunitReady = MTD_FALSE;
        return MTD_OK;
    }

    MTD_ATTEMPT(mtdGetFirmwareVersion(devPtr,port,&major,&minor,&inc,&test));
    MTD_ATTEMPT(mtdHwXmdioRead(devPtr,port,1,0xC00C,&bistStatus))

    if ((test != 0) && 
        (bistStatus & (0xAB7D)))
    {
        *tunitReady = MTD_FALSE;
        return MTD_OK;        
    }

    MTD_ATTEMPT(mtdHwXmdioRead(devPtr,port,MTD_TUNIT_XG_EXT_STATUS,&temp));
    MTD_ATTEMPT(mtdHwGetRegFieldFromWord(temp,8,4,&tunitMode));

    if (tunitMode == 3 || tunitMode == 8)
    {
        *tunitReady = MTD_FALSE;
        return MTD_OK;
    }
        
    return MTD_OK;
}


MTD_STATUS mtdIsPhyReadyAfterReset
(
    IN MTD_DEV_PTR devPtr,
    IN MTD_U16 port,
    OUT MTD_BOOL *phyReady
)              
{
    MTD_U16 val;

    *phyReady = MTD_FALSE;
    
    MTD_ATTEMPT(mtdHwGetPhyRegField(devPtr,port,MTD_TUNIT_IEEE_PMA_CTRL1,15,1,&val));

    if(val)
    {
        *phyReady = MTD_FALSE;   /* if still in reset return '0' (could be coming up, or disabled by download mode) */
    }
    else
    {
        *phyReady = MTD_TRUE;   /* if Phy is in normal operation */
    }
    return MTD_OK;
}

#define MTD_LOW_POWER_MODE_MAX_TIME 500 /* in ms, worst case time to respond to low power mode */
#define MTD_LOW_POWER_MODE  (2) 

MTD_STATUS mtdPutPhyInLowPowerMode
(
    IN MTD_DEV_PTR devPtr,
    IN MTD_U16 port
)
{
    MTD_BOOL xfiloopback, sgmiiloopback, sgmiian_enable,dummy;
    MTD_U16 sgmiispeed, horxunit;
    MTD_U16 timeoutCntr, tmp1, tmp2;
    

    for (horxunit=MTD_X_UNIT; horxunit<=MTD_H_UNIT; horxunit++)
    {
        if ((horxunit == MTD_X_UNIT && MTD_HAS_X_UNIT(devPtr)) ||
            (horxunit == MTD_H_UNIT))
        {
            MTD_ATTEMPT(mtdGet1000BXSGMIIControl(devPtr,port,horxunit,&sgmiiloopback,&sgmiispeed,&sgmiian_enable,&dummy));
            MTD_ATTEMPT(mtdGet10GBRControl(devPtr,port,horxunit,&xfiloopback,&dummy));

            MTD_ATTEMPT(mtdSet1000BXSGMIIControl(devPtr,port,horxunit,sgmiiloopback,sgmiispeed,sgmiian_enable,MTD_TRUE,MTD_FALSE,MTD_FALSE));
            MTD_ATTEMPT(mtdSet10GBRControl(devPtr,port,horxunit,xfiloopback,MTD_TRUE,MTD_FALSE));
        }
    }

    MTD_ATTEMPT(mtdPutTunitInLowPowerMode(devPtr,port));

    /* Wait for T unit in low power mode, then run workaround for 1G LED Activity */
    MTD_ATTEMPT(mtdHwXmdioRead(devPtr,port,MTD_TUNIT_XG_EXT_STATUS,&tmp1));
    MTD_ATTEMPT(mtdHwGetRegFieldFromWord(tmp1,8,4,&tmp2));
    timeoutCntr = 0;
    while(tmp2 != MTD_LOW_POWER_MODE && (timeoutCntr++ < MTD_LOW_POWER_MODE_MAX_TIME))
    {
        MTD_ATTEMPT(mtdWait(devPtr,1));
        MTD_ATTEMPT(mtdHwXmdioRead(devPtr,port,MTD_TUNIT_XG_EXT_STATUS,&tmp1));
        MTD_ATTEMPT(mtdHwGetRegFieldFromWord(tmp1,8,4,&tmp2));
    }

    if (timeoutCntr >= MTD_LOW_POWER_MODE_MAX_TIME)
    {
        return MTD_FAIL;
    }
    else
    {
        /* run workaround for 1G LED Activity, this is needed on X32X0, X33X0, E20X0 and E21X0 PHYs */
        MTD_ATTEMPT(mtdHwXmdioRead(devPtr,port,3,0x8FC6,&tmp1));
        MTD_ATTEMPT(mtdHwXmdioRead(devPtr,port,3,0x8FC3,&tmp2));

        MTD_ATTEMPT(mtdHwSetPhyRegField(devPtr,port,3,0x8FC6,15,1,1));
        MTD_ATTEMPT(mtdHwSetPhyRegField(devPtr,port,3,0x8FC6,12,2,1));

        MTD_ATTEMPT(mtdHwSetPhyRegField(devPtr,port,3,0x8FC3,15,1,1));
        MTD_ATTEMPT(mtdHwSetPhyRegField(devPtr,port,3,0x8FC3,12,1,1));
        
        /* delay only needs to be 1 us, guaranteed by serial interface speed and rmw access */
        MTD_ATTEMPT(mtdHwXmdioWrite(devPtr,port,3,0x8FC6,tmp1));
        MTD_ATTEMPT(mtdHwXmdioWrite(devPtr,port,3,0x8FC3,tmp2));
    }    

    return MTD_OK;
}


MTD_STATUS mtdRemovePhyLowPowerMode
(
    IN MTD_DEV_PTR devPtr,
    IN MTD_U16 port
)
{
    MTD_BOOL xfiloopback, sgmiiloopback, sgmiian_enable,dummy;
    MTD_U16 sgmiispeed, horxunit;

    for (horxunit=MTD_X_UNIT; horxunit<=MTD_H_UNIT; horxunit++)
    {
        if ((horxunit == MTD_X_UNIT && MTD_HAS_X_UNIT(devPtr)) ||
            (horxunit == MTD_H_UNIT))
        {        
            MTD_ATTEMPT(mtdGet1000BXSGMIIControl(devPtr,port,horxunit,&sgmiiloopback,&sgmiispeed,&sgmiian_enable,&dummy));
            MTD_ATTEMPT(mtdGet10GBRControl(devPtr,port,horxunit,&xfiloopback,&dummy));

            MTD_ATTEMPT(mtdSet1000BXSGMIIControl(devPtr,port,horxunit,sgmiiloopback,sgmiispeed,sgmiian_enable,MTD_FALSE,MTD_FALSE,MTD_FALSE));
            MTD_ATTEMPT(mtdSet10GBRControl(devPtr,port,horxunit,xfiloopback,MTD_FALSE,MTD_FALSE));
        }
    }

    MTD_ATTEMPT(mtdRemoveTunitLowPowerMode(devPtr,port));
    
    return MTD_OK;
}

MTD_STATUS mtdPutTunitInLowPowerMode
(
    IN MTD_DEV_PTR devPtr,
    IN MTD_U16 port
)
{
    return (mtdHwSetPhyRegField(devPtr,port,MTD_TUNIT_IEEE_PMA_CTRL1,11,1,1));
}

MTD_STATUS mtdRemoveTunitLowPowerMode
(
    IN MTD_DEV_PTR devPtr,
    IN MTD_U16 port
)
{
    if (mtdHwSetPhyRegField(devPtr,port,MTD_TUNIT_IEEE_PMA_CTRL1,11,1,0) == MTD_FAIL ||
        mtdHwSetPhyRegField(devPtr,port,MTD_TUNIT_IEEE_PCS_CTRL1,11,1,0) == MTD_FAIL)
    {
        return MTD_FAIL;
    }
    else
    {    
        return MTD_OK;
    }
}

MTD_STATUS mtdIsTunitInLowPowerMode
(
    IN MTD_DEV_PTR devPtr,
    IN MTD_U16 port,
    OUT MTD_BOOL *inLowPowerMode
)
{
    MTD_U16 tmp1,tmp2;
    
    MTD_ATTEMPT(mtdHwXmdioRead(devPtr,port,MTD_TUNIT_XG_EXT_STATUS,&tmp1));
    MTD_ATTEMPT(mtdHwGetRegFieldFromWord(tmp1,8,4,&tmp2));

    if (tmp2 == MTD_LOW_POWER_MODE)
    {
        *inLowPowerMode = MTD_TRUE;
    }
    else
    {
        *inLowPowerMode = MTD_FALSE;
    }

    return MTD_OK;
}

MTD_STATUS mtdPutPhyInParallelWriteMode
(
    IN MTD_DEV_PTR devPtr,
    IN MTD_U16 port
)
{
    /* Do not do a rmw on this register. Other bits in this register should not be written to non-zero */
    /* or it can cause bad MDIO behavior which will affect all devices on the bus. */
    return(mtdHwXmdioWrite(devPtr, port, 3, 0xD101, (1<<14)));   
}

MTD_STATUS mtdRemovePhyFromParallelWriteMode
(
    IN MTD_DEV_PTR devPtr,
    IN MTD_U16 port
)
{       
    /* Do not do a rmw on this register. Other bits in this register should not be written to non-zero */
    /* or it can cause bad MDIO behavior which will affect all devices on the bus. */    
    return(mtdHwXmdioWrite(devPtr, port, 3, 0xD101, 0));
}


MTD_STATUS mtdPutPhyInMdioDownloadMode
(
    IN MTD_DEV_PTR devPtr,
    IN MTD_U16 port
)
{       
    if (mtdHwSetPhyRegField(devPtr,port,31,0xF008,5,1,1) == MTD_OK)
    {
        return (mtdHwSetPhyRegField(devPtr,port,MTD_CUNIT_PORT_CTRL,12,1,1)); /* T unit hardware reset */
    }
    else
    {
        return MTD_FAIL;
    }
}

MTD_STATUS mtdRemovePhyMdioDownloadMode
(
    IN MTD_DEV_PTR devPtr,
    IN MTD_U16 port
)
{
    if (mtdHwSetPhyRegField(devPtr,port,31,0xF008,5,1,0) == MTD_OK)
    {
        return (mtdHwSetPhyRegField(devPtr,port,MTD_CUNIT_PORT_CTRL,12,1,1)); /* T unit hardware reset */
    }
    else
    {
        return MTD_FAIL;
    }
}

MTD_STATUS mtdGetFirmwareVersion
(
    IN MTD_DEV_PTR devPtr,
    IN MTD_U16 port, 
    OUT MTD_U8 *major, 
    OUT MTD_U8 *minor, 
    OUT MTD_U8 *inc, 
    OUT MTD_U8 *test)
{
    MTD_U16 reg_49169, reg_49170;
    
    MTD_ATTEMPT(mtdHwXmdioRead(devPtr,port,1,49169,&reg_49169));
    
    *major = (reg_49169 & 0xFF00) >> 8;
    *minor = (reg_49169 & 0x00FF);

    MTD_ATTEMPT(mtdHwXmdioRead(devPtr,port,1,49170,&reg_49170));
    
    *inc = (reg_49170 &0xFF00) >> 8;
    *test = (reg_49170 &0x00FF);

    if (!(*major || *minor || *inc || *test)) /* firmware is not running if all 0's */
    {
        return MTD_FAIL;
    }
    return MTD_OK;
}

/****************************************************************************/
/*******************************************************************
   802.3 Clause 28 and Clause 45
   Autoneg Related Control & Status for Copper
 *******************************************************************/
/*******************************************************************
  Find out speeds available
  Enabling speeds for autonegotiation
  Reading speeds enabled for autonegotiation
  Forcing speed (AN disabled)
  Set/get pause advertisement for autonegotiation
  Checking if link is up and at what speed
  Checking if AN is completed and what the result was
  Other Autoneg-related Control and Status (restart,disable/enable,
  force master/slave/auto, etc.)
 *******************************************************************/

MTD_STATUS mtdGetSpeedsAvailable
(
    IN MTD_DEV_PTR devPtr,
    IN MTD_U16 port,
    OUT MTD_U16 *speed_bits     
)
{
    MTD_UNUSED_PARAMETER(port);

    *speed_bits = MTD_ADV_NONE;

    if (MTD_IS_5G_DEVICE(devPtr->deviceId))
    {
       *speed_bits = MTD_SPEED_ALL_2XX0; /* this needs to be first, because it's based on 33X0 and 35X0 device type */
    }    
    else if (MTD_IS_X32X0_BASE(devPtr->deviceId))
    {
        *speed_bits = MTD_SPEED_ALL_32X0;
    }
    else if (MTD_IS_X33X0_BASE(devPtr->deviceId))
    {
        *speed_bits = MTD_SPEED_ALL_33X0_35X0;
    }
    else if (MTD_IS_E21X0_BASE(devPtr->deviceId))
    {
        if (MTD_IS_2P5G_DEVICE(devPtr->deviceId))
        {
            *speed_bits = MTD_SPEED_ALL_21X1;
        }
        else
        {
            *speed_bits = MTD_SPEED_ALL_2XX0;
        }
    }
    else if (MTD_IS_X3540_FAMILY(devPtr->deviceId) || MTD_IS_X3510_FAMILY(devPtr->deviceId))
    {
        *speed_bits = MTD_SPEED_ALL_33X0_35X0;
    }
    else if (MTD_IS_X36X0_BASE(devPtr->deviceId))
    {
        *speed_bits = MTD_SPEED_ALL_36X0;    
    }
    else if (MTD_IS_E2780_FAMILY(devPtr->deviceId))
    {
        *speed_bits = MTD_SPEED_ALL_E2780;    
    }
    else
    {
        MTD_DBG_ERROR("mtdGetSpeedsAvailable: Unknown device type\n");
        return MTD_FAIL;
    }

    return MTD_OK;

}


#define MTD_7_0010_SPEED_BIT_LENGTH 4
#define MTD_7_0010_SPEED_BIT_POS    5
#define MTD_7_8000_SPEED_BIT_LENGTH 2
#define MTD_7_8000_SPEED_BIT_POS    8
#define MTD_7_0020_SPEED_BIT_LENGTH 1   /* for 88X32X0 family and 88X33X0 family */
#define MTD_7_0020_SPEED_BIT_POS    12
#define MTD_7_0020_SPEED_BIT_LENGTH2 2   /* for 88X33X0/E21X0 family 2.5/5G */
#define MTD_7_0020_SPEED_BIT_POS2    7


/* Bit defines for speed bits */
#define MTD_FORCED_SPEEDS_BIT_MASK  (MTD_SPEED_10M_HD_AN_DIS | MTD_SPEED_10M_FD_AN_DIS | \
                                     MTD_SPEED_100M_HD_AN_DIS | MTD_SPEED_100M_FD_AN_DIS)
#define MTD_LOWER_BITS_MASK         0x000F /* bits in base page */
#define MTD_GIG_SPEED_POS           4
#define MTD_XGIG_SPEED_POS          6
#define MTD_2P5G_SPEED_POS          11
#define MTD_5G_SPEED_POS            12
#define MTD_GET_1000BT_BITS(_speedBits) ((_speedBits & (MTD_SPEED_1GIG_HD | MTD_SPEED_1GIG_FD))>>MTD_GIG_SPEED_POS) /* 1000BT bits */
#define MTD_GET_10GBT_BIT(_speedBits) ((_speedBits & MTD_SPEED_10GIG_FD)>>MTD_XGIG_SPEED_POS) /* 10GBT bit setting */
#define MTD_GET_2P5GBT_BIT(_speedBits) ((_speedBits & MTD_SPEED_2P5GIG_FD)>>MTD_2P5G_SPEED_POS) /* 2.5GBT bit setting */
#define MTD_GET_5GBT_BIT(_speedBits) ((_speedBits & MTD_SPEED_5GIG_FD)>>MTD_5G_SPEED_POS) /* 5GBT bit setting */

MTD_STATUS mtdEnableSpeeds
(
    IN MTD_DEV_PTR devPtr,
    IN MTD_U16 port,
    IN MTD_U16 speed_bits, 
    IN MTD_BOOL anRestart
)
{
    MTD_BOOL speedForced;
    MTD_U16 dummy;
    MTD_U16 tempRegValue;
    MTD_U16 speeds_not_available, speeds_available;

    if (MTD_IS_X33X0_BASE(devPtr->deviceId))
    {
        const MTD_U16 chipRev = (devPtr->deviceId & 0xf); /* get the chip revision */

        if (chipRev == 9 || chipRev == 5 || chipRev == 1 || /* Z2 chip revisions */
            chipRev == 8 || chipRev == 4 || chipRev == 0)   /* Z1 chip revisions */
        {
            /* this is an X33X0 or E20X0 Z2/Z1 device and not supported (not compatible with A0) */
            return MTD_FAIL;
        }
    }

    MTD_ATTEMPT(mtdGetSpeedsAvailable(devPtr,port,&speeds_available));

    if (speed_bits == MTD_ALL_SPEEDS_AVAILABLE)
    {
        speed_bits = speeds_available; /* change to all available speeds on this device */
    }
    else if (speed_bits & MTD_FORCED_SPEEDS_BIT_MASK)
    {
        return MTD_FAIL; /* tried to force the speed, this function is for autonegotiation control */
    }
    
    speeds_not_available = ~(speeds_available); /* speeds not available now */

    if (speeds_not_available & speed_bits)
    {
        return MTD_FAIL; /* tried to advertise a speed not available on this device*/
    }

    /* Enable AN and set speed back to power-on default in case previously forced
       Only do it if forced, to avoid an extra/unnecessary soft reset */
    MTD_ATTEMPT(mtdGetForcedSpeed(devPtr,port,&speedForced,&dummy));
    if (speedForced)
    {
        MTD_ATTEMPT(mtdUndoForcedSpeed(devPtr,port,MTD_FALSE));
    }
     
    if (speed_bits == MTD_ADV_NONE)
    {
        /* Set all speeds to be disabled
         Take care of bits in 7.0010 (advertisement register, 10BT and 100BT bits) */
        MTD_ATTEMPT(mtdHwSetPhyRegField(devPtr,port,7,0x0010,\
                MTD_7_0010_SPEED_BIT_POS, MTD_7_0010_SPEED_BIT_LENGTH, \
                0));
                
        /* Take care of speed bits in 7.8000 (1000BASE-T speed bits) */
        MTD_ATTEMPT(mtdHwSetPhyRegField(devPtr,port,7,0x8000,\
                MTD_7_8000_SPEED_BIT_POS, MTD_7_8000_SPEED_BIT_LENGTH, \
                0));

        /* Now take care of bit in 7.0020 (10GBASE-T) */
        MTD_ATTEMPT(mtdHwSetPhyRegField(devPtr,port,7,0x0020,\
                MTD_7_0020_SPEED_BIT_POS, MTD_7_0020_SPEED_BIT_LENGTH, 0));

        if (speeds_available & (MTD_SPEED_2P5GIG_FD | MTD_SPEED_5GIG_FD))
        {
            /* Now take care of bits in 7.0020 (2.5G, 5G speed bits) */
            MTD_ATTEMPT(mtdHwSetPhyRegField(devPtr,port,7,0x0020,\
                    MTD_7_0020_SPEED_BIT_POS2, MTD_7_0020_SPEED_BIT_LENGTH2, 0));
        }
    }
    else
    {
        /* Take care of bits in 7.0010 (advertisement register, 10BT and 100BT bits) */
        MTD_ATTEMPT(mtdHwSetPhyRegField(devPtr,port,7,0x0010,\
                MTD_7_0010_SPEED_BIT_POS, MTD_7_0010_SPEED_BIT_LENGTH, \
                (speed_bits & MTD_LOWER_BITS_MASK)));
                
        /* Take care of speed bits in 7.8000 (1000BASE-T speed bits) */
        MTD_ATTEMPT(mtdHwSetPhyRegField(devPtr,port,7,0x8000,\
                MTD_7_8000_SPEED_BIT_POS, MTD_7_8000_SPEED_BIT_LENGTH, \
                MTD_GET_1000BT_BITS(speed_bits)));
        
                   
        /* Now take care of bits in 7.0020 (10GBASE-T first) */
        MTD_ATTEMPT(mtdHwXmdioRead(devPtr,port,7,0x0020,&tempRegValue));
        MTD_ATTEMPT(mtdHwSetRegFieldToWord(tempRegValue,MTD_GET_10GBT_BIT(speed_bits),\
                MTD_7_0020_SPEED_BIT_POS, MTD_7_0020_SPEED_BIT_LENGTH, \
                &tempRegValue));
        
        if (speeds_available & (MTD_SPEED_2P5GIG_FD | MTD_SPEED_5GIG_FD))
        {            
            /* Now take care of 2.5G bit in 7.0020 */
            MTD_ATTEMPT(mtdHwSetRegFieldToWord(tempRegValue,MTD_GET_2P5GBT_BIT(speed_bits),\
                    7, 1, \
                    &tempRegValue));

            /* Now take care of 5G bit in 7.0020 */
            MTD_ATTEMPT(mtdHwSetRegFieldToWord(tempRegValue,MTD_GET_5GBT_BIT(speed_bits),\
                    8, 1, \
                    &tempRegValue));                 
        }
    
        /* Now write result back to 7.0020 */
        MTD_ATTEMPT(mtdHwXmdioWrite(devPtr,port,7,0x0020,tempRegValue));

        if (MTD_GET_10GBT_BIT(speed_bits) || MTD_GET_2P5GBT_BIT(speed_bits) || MTD_GET_5GBT_BIT(speed_bits))
        {
            // Set XNP on if any bit that required it was set
            MTD_ATTEMPT(mtdHwSetPhyRegField(devPtr,port,7,0,13,1,1));
        }
    }   

    if(anRestart)
    {
        return ((MTD_STATUS)(mtdAutonegEnable(devPtr,port) ||
                             mtdAutonegRestart(devPtr,port)));
    }

    return MTD_OK;
}

MTD_STATUS mtdEnableAllSpeedsRestartAn
(
    IN MTD_DEV_PTR devPtr,
    IN MTD_U16 port     
)
{
    MTD_U16 speeds;

    MTD_ATTEMPT(mtdGetSpeedsAvailable(devPtr,port,&speeds));
    MTD_ATTEMPT(mtdEnableSpeeds(devPtr,port,speeds,MTD_TRUE));

    return MTD_OK;
}


MTD_STATUS mtdGetSpeedsEnabled
(
    IN MTD_DEV_PTR devPtr,
    IN MTD_U16 port,
    OUT MTD_U16 *speed_bits     
)
{
    MTD_U16 lower_speeds, gig_speeds, xgig_speed, hgig_speeds;

    if (mtdHwGetPhyRegField(devPtr,port, 7, 0x0010, MTD_7_0010_SPEED_BIT_POS, 
            MTD_7_0010_SPEED_BIT_LENGTH, &lower_speeds) == MTD_FAIL)
    {
        *speed_bits = MTD_ADV_NONE;
        return MTD_FAIL;
    }

    if (mtdHwGetPhyRegField(devPtr,port, 7, 0x8000, MTD_7_8000_SPEED_BIT_POS, 
            MTD_7_8000_SPEED_BIT_LENGTH, &gig_speeds) == MTD_FAIL)
    {
        *speed_bits = MTD_ADV_NONE;
        return MTD_FAIL;
    }

    *speed_bits = (lower_speeds | (gig_speeds <<= MTD_GIG_SPEED_POS));

    /* get 10GBT speed bit note that on E21X0 this should always be 0 anyways */
    if (mtdHwGetPhyRegField(devPtr,port, 7, 0x0020, MTD_7_0020_SPEED_BIT_POS, 
            MTD_7_0020_SPEED_BIT_LENGTH, &xgig_speed) == MTD_FAIL)
    {
        *speed_bits = MTD_ADV_NONE;
        return MTD_FAIL;
    }
    
    *speed_bits |= (xgig_speed <<= MTD_XGIG_SPEED_POS);

    /* all chips except X32X0 */
    if (MTD_IS_X32X0_BASE(devPtr->deviceId) == MTD_FALSE)
    {
        /* read 2.5/5G speed bits from 7.0020 */
        if (mtdHwGetPhyRegField(devPtr,port, 7, 0x0020, MTD_7_0020_SPEED_BIT_POS2, 
                MTD_7_0020_SPEED_BIT_LENGTH2, &hgig_speeds) == MTD_FAIL)
        {
            *speed_bits = MTD_ADV_NONE;
            return MTD_FAIL;
        }

        /* need to unpack speed bits 7.0020.8:7>>7 on 88X33X0 A0 and E20X0 A0 and E21X0 and later */
        *speed_bits |= ((hgig_speeds & (1<<1)) ? (1<<MTD_5G_SPEED_POS) : 0);
        *speed_bits |= ((hgig_speeds & (1<<0)) ? (1<<MTD_2P5G_SPEED_POS) : 0);
        
    }

    return MTD_OK;
}

MTD_STATUS mtdGetSpeedsInCommon
(
    IN MTD_DEV_PTR devPtr0,
    IN MTD_U16 port0, 
    IN MTD_DEV_PTR devPtr1,
    IN MTD_U16 port1,
    OUT MTD_U16 *commonSpeeds, /* bit mask based on speed definition of mtdEnableSpeeds() of speeds both ends support */
    OUT MTD_U16 *highestBit,   /* this is the speed in the highest bit position in common, not the greatest bps */
    OUT MTD_U16 *lowestBit,    /* this is the speed in the lowest bit position in common, not the lowest bps */
    OUT MTD_U16 *highestSpeed  /* this is the highest speed in common in terms of bps */
)
{
    MTD_U16 portNum, speedsSupported[2];
    const MTD_U16 ports[2] = {port0, port1};
    const MTD_DEV_PTR devices[2] = {devPtr0, devPtr1};
    MTD_U16 speedsSupportedByBoth;
    MTD_U16 last_speed_to_try,cntr,lowest_speed,highest_speed;
    
    /* Find out which speeds are available on both ends, and what speeds they have in common */
    speedsSupportedByBoth = 0xFFFF;
    for (portNum=0; portNum < 2; portNum++)
    {
        MTD_ATTEMPT(mtdGetSpeedsAvailable(devices[portNum],ports[portNum],&speedsSupported[portNum]));

        speedsSupportedByBoth &= speedsSupported[portNum];
    }

    *commonSpeeds = speedsSupportedByBoth;

    /* Find the first/last speed to try */
    /* Find highest bit first */
    cntr=0;
    last_speed_to_try = speedsSupportedByBoth;
    while(cntr<16 && last_speed_to_try)
    {
        last_speed_to_try >>= 1;
        cntr++;
    }
    highest_speed = (1<<(cntr-1));

    /* Now find lowest bit */
    cntr=0;
    last_speed_to_try = speedsSupportedByBoth;
    while(cntr<16 && last_speed_to_try)
    {
        last_speed_to_try <<= 1;
        cntr++;
    }
    lowest_speed = (0x8000>>(cntr-1));

    *highestBit = highest_speed;
    *lowestBit = lowest_speed;

    /* Now find the highest speed in common (because the bits are not all in order) */
    if ((*highestBit) <= MTD_SPEED_10GIG_FD)
    {
        /* These bits are all in order of speed */
        *highestSpeed = *highestBit;
    }
    else if ((*highestBit) > MTD_SPEED_10GIG_FD)
    {
        /* Highest bit is 5G or 2.5G */
        /* Need to check if 10G was supported or not */
        if ((*commonSpeeds) & MTD_SPEED_10GIG_FD)
        {
            /* Supported either 5G or 2.5G AND 10G, 10G greater */
            *highestSpeed = MTD_SPEED_10GIG_FD;
        }
        else
        {
            /* Supported 5G or 2.5G, but not 10G, so highest bit is highest speed */
            *highestSpeed = *highestBit;
        }
    }
    else
    {
        /* Something wrong (maybe no speeds in common?) */
        *highestSpeed = MTD_ADV_NONE;
    }

    return MTD_OK;
}

MTD_STATUS mtdForceSpeed
(
    IN MTD_DEV_PTR devPtr,
    IN MTD_U16 port,
    IN MTD_U16 forcedSpeedBit 
)
{
    MTD_U16 duplexBit;   

    /* force the speed by setting the correct bits in 1.0.13/6 and the duplex bit */
    /* in 7.A000.4, order is important. speed bits must be set before AN is disabled */

    switch (forcedSpeedBit)
    {
        case MTD_SPEED_10M_HD_AN_DIS:
        case MTD_SPEED_10M_FD_AN_DIS:
            MTD_ATTEMPT(mtdHwSetPhyRegField(devPtr,port,MTD_TUNIT_IEEE_PMA_CTRL1,13,1,0));
            MTD_ATTEMPT(mtdHwSetPhyRegField(devPtr,port,MTD_TUNIT_IEEE_PMA_CTRL1,6,1,0));
            if (forcedSpeedBit == MTD_SPEED_10M_FD_AN_DIS)
            {
                duplexBit = 1;
            }
            else
            {
                duplexBit = 0;
            }
            MTD_ATTEMPT(mtdHwSetPhyRegField(devPtr,port,7,0x8000,4,1,duplexBit));
            break;

        case MTD_SPEED_100M_HD_AN_DIS:
        case MTD_SPEED_100M_FD_AN_DIS:
            MTD_ATTEMPT(mtdHwSetPhyRegField(devPtr,port,MTD_TUNIT_IEEE_PMA_CTRL1,13,1,1));
            MTD_ATTEMPT(mtdHwSetPhyRegField(devPtr,port,MTD_TUNIT_IEEE_PMA_CTRL1,6,1,0));
            if (forcedSpeedBit == MTD_SPEED_100M_FD_AN_DIS)
            {
                duplexBit = 1;
            }
            else
            {
                duplexBit = 0;
            }
            MTD_ATTEMPT(mtdHwSetPhyRegField(devPtr,port,7,0x8000,4,1,duplexBit));
            break;

        default:
            return MTD_FAIL;
            break;
    }

    /* disable an by setting 7.0.12 = 0 */
    /* IMPORTANT, this has to be done after the speed bits are changed, AN is forced enabled in the phy if the speed bits are set to */
    /* 1G or 10G in 1.0 */
    MTD_ATTEMPT(mtdAutonegDisable(devPtr,port));

    /* when speed bits are changed, T unit sw reset is required wait until phy is ready */
    MTD_ATTEMPT(mtdSoftwareReset(devPtr,port,1000));

    return MTD_OK;
}

MTD_STATUS mtdUndoForcedSpeed
(
    IN MTD_DEV_PTR devPtr,
    IN MTD_U16 port,
    IN MTD_BOOL anRestart
)
{

    MTD_ATTEMPT(mtdHwSetPhyRegField(devPtr,port,MTD_TUNIT_IEEE_PMA_CTRL1,13,1,1));
    MTD_ATTEMPT(mtdHwSetPhyRegField(devPtr,port,MTD_TUNIT_IEEE_PMA_CTRL1,6,1,1));

    /* when speed bits are changed, T unit sw reset is required, wait until phy is ready */
    MTD_ATTEMPT(mtdSoftwareReset(devPtr,port,1000));

    if(anRestart)
    {
        return ((MTD_STATUS)(mtdAutonegEnable(devPtr,port) ||
                             mtdAutonegRestart(devPtr,port)));
    }

    return MTD_OK;        
}


MTD_STATUS mtdGetForcedSpeed
(
    IN MTD_DEV_PTR devPtr,
    IN MTD_U16 port,
    OUT MTD_BOOL *speedIsForced,
    OUT MTD_U16 *forcedSpeed 
)
{
    MTD_U16 val,bit0,bit1, forcedSpeedBits, duplexBit;
    MTD_BOOL anDisabled;

    *speedIsForced = MTD_FALSE;
    *forcedSpeed = MTD_ADV_NONE;
    
    /* check if 7.0.12 is 0 or 1 (disabled or enabled) */
    MTD_ATTEMPT(mtdHwGetPhyRegField(devPtr,port,7,0,12,1,&val));

    (val) ? (anDisabled = MTD_FALSE) : (anDisabled = MTD_TRUE);

    if (anDisabled)
    {
        /* autoneg is disabled, see if it's forced to one of the speeds that work without AN */
        MTD_ATTEMPT(mtdHwGetPhyRegField(devPtr,port,MTD_TUNIT_IEEE_PMA_CTRL1,6, 1,&bit0));
        MTD_ATTEMPT(mtdHwGetPhyRegField(devPtr,port,MTD_TUNIT_IEEE_PMA_CTRL1,13, 1,&bit1));

        /* now read the duplex bit setting */
        MTD_ATTEMPT(mtdHwGetPhyRegField(devPtr,port,7,0x8000,4, 1,&duplexBit));

        forcedSpeedBits = 0;
        forcedSpeedBits = bit0 | (bit1<<1);

        if (forcedSpeedBits == 0)
        {
            /* it's set to 10BT */
            if (duplexBit)
            {
                *speedIsForced = MTD_TRUE;
                *forcedSpeed = MTD_SPEED_10M_FD_AN_DIS;
            }
            else
            {
                *speedIsForced = MTD_TRUE;
                *forcedSpeed = MTD_SPEED_10M_HD_AN_DIS;
            }            
        }
        else if (forcedSpeedBits == 2)
        {
            /* it's set to 100BT */ 
            if (duplexBit)
            {
                *speedIsForced = MTD_TRUE;
                *forcedSpeed = MTD_SPEED_100M_FD_AN_DIS;
            }
            else
            {
                *speedIsForced = MTD_TRUE;
                *forcedSpeed = MTD_SPEED_100M_HD_AN_DIS;
            }    
        }                
        /* else it's set to 1G/2.5G/5G/10G which require AN */
    }

    return MTD_OK;    
}


MTD_STATUS mtdSetPauseAdvertisement
(
    IN MTD_DEV_PTR devPtr,
    IN MTD_U16 port,
    IN MTD_U8 pauseType,
    IN MTD_BOOL anRestart
)
{
    /* sets/clears bits 11, 10 (A6,A5 in the tech bit field of 7.16) */
    MTD_ATTEMPT(mtdHwSetPhyRegField(devPtr,port, 7, 0x0010, \
                        10, 2, (MTD_U16)pauseType));

    if(anRestart)
    {
        return ((MTD_STATUS)(mtdAutonegEnable(devPtr,port) ||
                             mtdAutonegRestart(devPtr,port)));
    }

    return MTD_OK;
}

MTD_STATUS mtdGetPauseAdvertisement
(
    IN MTD_DEV_PTR devPtr,
    IN MTD_U16 port,
    OUT MTD_U8 *pauseType
)
{
    MTD_U16 temp;

    /* reads bits 11, 10 (A6,A5 in the tech bit field of 7.16) */
    MTD_ATTEMPT(mtdHwGetPhyRegField(devPtr,port, 7, 0x0010, \
                        10, 2, &temp));

   *pauseType = (MTD_U8)temp;

    return MTD_OK;
}


MTD_STATUS mtdGetLPAdvertisedSpeeds
(
    IN MTD_DEV_PTR devPtr,
    IN MTD_U16 port,
    OUT MTD_U16 *ieeeSpeedBits,
    OUT MTD_U16 *nbasetSpeedBits
)
{
    MTD_BOOL anStatusReady; 
    MTD_U16 val,lower_speeds,gig_speeds,bitset;

    *ieeeSpeedBits = MTD_ADV_NONE;
    *nbasetSpeedBits = MTD_ADV_NONE;

    /* Make sure AN is complete */
    MTD_ATTEMPT(mtdAutonegIsCompleted(devPtr,port,&anStatusReady));

    if (anStatusReady == MTD_FALSE)
    {
        return MTD_FAIL;
    }

    /* check AN LP base page abilities first */
    MTD_ATTEMPT(mtdHwXmdioRead(devPtr,port,7,0x0013,&val));

    /* register layout is the same as 7.0010 */
    mtdHwGetRegFieldFromWord(val, MTD_7_0010_SPEED_BIT_POS, 
        MTD_7_0010_SPEED_BIT_LENGTH, &lower_speeds);

    /* now check 1G AN Status */
    MTD_ATTEMPT(mtdHwXmdioRead(devPtr,port,7,0x8001,&val));

    /* register layout is the same as 7.8001 but shifted up 2 bits */
    mtdHwGetRegFieldFromWord(val, MTD_7_8000_SPEED_BIT_POS+2, 
        MTD_7_8000_SPEED_BIT_LENGTH, &gig_speeds);

    *ieeeSpeedBits = (lower_speeds | (gig_speeds <<= MTD_GIG_SPEED_POS));

    /* now check 10G AN Status */
    MTD_ATTEMPT(mtdHwXmdioRead(devPtr,port,7,0x0021,&val));
    
    mtdHwGetRegFieldFromWord(val, 11, 1, &bitset);
    if (bitset)
    {
        *ieeeSpeedBits |= MTD_SPEED_10GIG_FD;
    }

    if (MTD_IS_X32X0_BASE(devPtr->deviceId))
    {
        return MTD_OK; /* X32X0 has no more speeds, done checking */
    }

    /* check for IEEE 2.5/5G for 33X0 devices */
    mtdHwGetRegFieldFromWord(val, 5, 1, &bitset);
    if (bitset)
    {
        *ieeeSpeedBits |= MTD_SPEED_2P5GIG_FD;
    }

    mtdHwGetRegFieldFromWord(val, 6, 1, &bitset);
    if (bitset)
    {
        *ieeeSpeedBits |= MTD_SPEED_5GIG_FD;
    }        

    /* Check if LP advertised 2.5G/5G via NBASE-T message page */
    MTD_ATTEMPT(mtdHwXmdioRead(devPtr,port,1,0xC035,&val));    
    mtdHwGetRegFieldFromWord(val, 14, 1, &bitset);
    if (bitset)
    {
        *nbasetSpeedBits |= MTD_SPEED_2P5GIG_FD;
    }

    mtdHwGetRegFieldFromWord(val,15,1,&bitset);
    if (bitset)
    {
        *nbasetSpeedBits |= MTD_SPEED_5GIG_FD;
    }        

    return MTD_OK;
}



MTD_STATUS mtdGetLPAdvertisedPause
(
    IN MTD_DEV_PTR devPtr,
    IN MTD_U16 port,
    OUT MTD_U8 *pauseBits
)
{
    MTD_U16 val;
    MTD_BOOL anStatusReady; 

    /* Make sure AN is complete */
    MTD_ATTEMPT(mtdAutonegIsCompleted(devPtr,port,&anStatusReady));

    if (anStatusReady == MTD_FALSE)
    {
        *pauseBits = MTD_CLEAR_PAUSE;
        return MTD_FAIL;
    }

    /* get bits 11, 10 (A6,A5 in the tech bit field of 7.19) */
    if (mtdHwGetPhyRegField(devPtr,port, 7, 19, 
         10, 2, &val) == MTD_FAIL)
    {
        *pauseBits = MTD_CLEAR_PAUSE;
        return MTD_FAIL;
    }

    *pauseBits = (MTD_U8)val;

    return MTD_OK;
}

MTD_STATUS mtdGetTxRxPauseResolution
(
    IN MTD_DEV_PTR devPtr,
    IN MTD_U16 port,
    OUT MTD_BOOL *pauseResolved,
    OUT MTD_BOOL *tx_pause_enabled,
    OUT MTD_BOOL *rx_pause_enabled
)
{
    MTD_U16 val;

    MTD_ATTEMPT(mtdAutonegIsSpeedDuplexResolutionDone(devPtr,port,pauseResolved));

    if (*pauseResolved == MTD_FALSE)
    {
        /* AN hasn't completed far enough to determine this */
        return MTD_OK;
    }
    
    /* get bits 3.8008.9:8 */
    if (mtdHwGetPhyRegField(devPtr,port, 3, 0x8008, 
         8, 2, &val) == MTD_FAIL)
    {
        *tx_pause_enabled = *rx_pause_enabled = MTD_FALSE;
        return MTD_FAIL;
    }

    if (val & 0x1)
    {
        /* Bit 8 was set */
        *rx_pause_enabled = MTD_TRUE;
    }
    else
    {
        *rx_pause_enabled = MTD_FALSE;        
    }

    if (val & 0x2)
    {
        /* Bit 9 was set */
        *tx_pause_enabled = MTD_TRUE;
    }
    else
    {
        *tx_pause_enabled = MTD_FALSE;        
    }

    return MTD_OK;    
}


MTD_STATUS mtdAutonegRestart
(
    IN MTD_DEV_PTR devPtr,
    IN MTD_U16 port
)
{
    MTD_BOOL tUnitReady;

    MTD_ATTEMPT(mtdIsTunitResponsive(devPtr,port,&tUnitReady));

    if (tUnitReady == MTD_FALSE)
    {
        MTD_DBG_ERROR("mtdAutonegRestart: Copper unit is not ready.\n");
        return MTD_FAIL;
    }

    /* set 7.0.9, restart AN */
    return (mtdHwSetPhyRegField(devPtr,port,7,0, 
             9, 1, 1));
}

MTD_STATUS mtdAutonegDisable
(
    IN MTD_DEV_PTR devPtr,
    IN MTD_U16 port
)
{
    /* set 7.0.12=0, disable AN */
    return (mtdHwSetPhyRegField(devPtr,port,7,0, 
             12, 1, 0));
}

MTD_STATUS mtdAutonegEnable
(
    IN MTD_DEV_PTR devPtr,
    IN MTD_U16 port
)
{
    /* set 7.0.12=1, enable AN */
    return (mtdHwSetPhyRegField(devPtr,port,7,0, 
             12, 1, 1));
}

MTD_STATUS mtdAutonegForceMaster
(
    IN MTD_DEV_PTR devPtr,
    IN MTD_U16 port,
    IN MTD_BOOL anRestart
)
{
    /* set bits 14 and 15 in register 7.32 (master, and manual m/s) both 1 */
    MTD_ATTEMPT(mtdHwSetPhyRegField(devPtr,port, 7, 32, \
                            14, 2, 3));
    
    if(anRestart)
    {
        return ((MTD_STATUS)(mtdAutonegEnable(devPtr,port) ||
                             mtdAutonegRestart(devPtr,port)));
    }

    return MTD_OK;
    
}

MTD_STATUS mtdAutonegForceSlave
(
    IN MTD_DEV_PTR devPtr,
    IN MTD_U16 port,
    IN MTD_BOOL anRestart
)
{
    /* set bits 14=0 and 15=1 (slave and manual m/s) in register 7.32 */
    MTD_ATTEMPT(mtdHwSetPhyRegField(devPtr,port, 7, 32, \
                            14, 2, 2));

    if(anRestart)
    {
        return ((MTD_STATUS)(mtdAutonegEnable(devPtr,port) ||
                             mtdAutonegRestart(devPtr,port)));
    }

    return MTD_OK;
    
}

MTD_STATUS mtdAutonegSetAutoMasterSlave
(
    IN MTD_DEV_PTR devPtr,
    IN MTD_U16 port,
    IN MTD_BOOL anRestart
)
{
    /* set bits 14=0 and 15=0 (slave and manual m/s) in register 7.32 */
    MTD_ATTEMPT(mtdHwSetPhyRegField(devPtr,port, 7, 32, \
                            14, 2, 0));

    if(anRestart)
    {
        return ((MTD_STATUS)(mtdAutonegEnable(devPtr,port) ||
                             mtdAutonegRestart(devPtr,port)));
    }

    return MTD_OK;    
}

MTD_STATUS mtdAutonegSetMasterPreference
(
    IN MTD_DEV_PTR devPtr,
    IN MTD_U16 port,
    IN MTD_BOOL anRestart
)
{

    /* set bit 15=0 and bit 13=1 in 7.32 (manual m/s and multiport bit */
    /* making preference to be master */
    if ((mtdHwSetPhyRegField(devPtr,port, 7, 32, 
                            15, 1, 0) == MTD_FAIL) ||
        (mtdHwSetPhyRegField(devPtr,port, 7, 32, 
                            13, 1, 1) == MTD_FAIL))
    {
        return MTD_FAIL;
    }
    
    if(anRestart)
    {
        return ((MTD_STATUS)(mtdAutonegEnable(devPtr,port) ||
                             mtdAutonegRestart(devPtr,port)));
    }

    return MTD_OK;
    
}

MTD_STATUS mtdAutonegSetSlavePreference
(
    IN MTD_DEV_PTR devPtr,
    IN MTD_U16 port,
    IN MTD_BOOL anRestart
)
{
    /* set bit 15=0 and bit 13=0 in 7.32 (manual m/s and multiport bit to single port */
    /* making preference to be slave */
    if ((mtdHwSetPhyRegField(devPtr,port, 7, 32, 
                            15, 1, 0) == MTD_FAIL) ||
        (mtdHwSetPhyRegField(devPtr,port, 7, 32, 
                            13, 1, 0) == MTD_FAIL))
    {
        return MTD_FAIL;
    }

    if(anRestart)
    {
        return ((MTD_STATUS)(mtdAutonegEnable(devPtr,port) ||
                             mtdAutonegRestart(devPtr,port)));
    }

    return MTD_OK;
    
}

MTD_STATUS mtdAutonegGetMasterSlavePreference
(
    IN MTD_DEV_PTR devPtr,
    IN MTD_U16 port,
    OUT MTD_U16 *msPreference
)
{
    MTD_U16 anMsBits, portType, msConfig, msManConfig;

    MTD_ATTEMPT(mtdHwGetPhyRegField(devPtr, port, 7, 32, 13, 3, &anMsBits));
    mtdHwGetRegFieldFromWord(anMsBits, 13, 1, &portType);
    mtdHwGetRegFieldFromWord(anMsBits, 14, 1, &msConfig);
    mtdHwGetRegFieldFromWord(anMsBits, 15, 1, &msManConfig);

    if (msManConfig)
    {
        /* it is forced */
        if (msConfig)
        {
            *msPreference = MTD_AN_MS_FORCED_MASTER;            
        }
        else
        {   
            *msPreference = MTD_AN_MS_FORCED_SLAVE;
        }
    }
    else
    {
        /* it is auto */
        if (portType)
        {
            *msPreference = MTD_AN_MS_PREFER_MASTER;            
        }
        else
        {   
            *msPreference = MTD_AN_MS_PREFER_SLAVE;
        }

    }
    
    return MTD_OK;
}


MTD_STATUS mtdAutonegIsResolutionMasterOrSlave
(
    IN MTD_DEV_PTR devPtr,
    IN MTD_U16 port,
    OUT MTD_U8 *anMsResolution
)
{
    MTD_BOOL anSpeedDuplexDone, msBit, cfgFault, anStatusReady;

    *anMsResolution = MTD_AN_RESOLUTION_UNKNOWN;

    MTD_ATTEMPT(mtdAutonegIsSpeedDuplexResolutionDone(devPtr,port, &anSpeedDuplexDone));

    if (anSpeedDuplexDone == MTD_TRUE)
    {
        /* now check if AN has completed */
        /* this will clear the config fault, so don't do this unless the speed/duplex are resolved */
        /* or do this after checking for a config fault if there's a possibility that happened */
        MTD_ATTEMPT(mtdAutonegIsCompleted(devPtr,port,&anStatusReady));

        if (anStatusReady == MTD_TRUE)
        {                    
            MTD_ATTEMPT(mtdReadMasterOrSlaveBit(devPtr,port, &msBit));

            (msBit) ? (*anMsResolution = MTD_AN_RESOLUTION_MASTER) : (*anMsResolution = MTD_AN_RESOLUTION_SLAVE);
        }
    }
    else
    {
        /* might not be done/completed because there's a config fault, check for that */
        MTD_ATTEMPT(mtdGetMasterSlaveConfigFault(devPtr,port,&cfgFault));

        if (cfgFault == MTD_TRUE)
        {
            *anMsResolution = MTD_AN_RESOLUTION_FAULT;
        }
        else
        {
            *anMsResolution = MTD_AN_RESOLUTION_UNKNOWN;
        }
    }

    return MTD_OK;
}


MTD_STATUS mtdGetAutonegSpeedDuplexResolution
(
    IN MTD_DEV_PTR devPtr,
    IN MTD_U16 port,
    OUT MTD_U16 *speedResolution
)
{
    MTD_U16 val, speed, speed2, duplex;
    MTD_BOOL resDone;

    *speedResolution = MTD_ADV_NONE;
    
    /* check if AN is enabled */
    MTD_ATTEMPT(mtdHwGetPhyRegField(devPtr,port, \
            7, 0, 12, 1, &val));
    
    if (val)
    {
        /* an is enabled, check if speed is resolved */
        MTD_ATTEMPT(mtdAutonegIsSpeedDuplexResolutionDone(devPtr,port, &resDone));
        
        if (resDone)
        {
            MTD_ATTEMPT(mtdHwGetPhyRegField(devPtr,port, \
                    3, 0x8008, 14, 2, &speed));
            
            MTD_ATTEMPT(mtdHwGetPhyRegField(devPtr,port, \
                    3, 0x8008, 13, 1, &duplex));
            
            switch (speed)
            {
                case MTD_CU_SPEED_10_MBPS:
                    if (duplex)
                    {
                        *speedResolution = MTD_SPEED_10M_FD;
                    }
                    else
                    {
                        *speedResolution = MTD_SPEED_10M_HD;
                    }
                    break;
                case MTD_CU_SPEED_100_MBPS:
                    if (duplex)
                    {
                        *speedResolution = MTD_SPEED_100M_FD;
                    }
                    else
                    {
                        *speedResolution = MTD_SPEED_100M_HD;
                    }
                    break;
                case MTD_CU_SPEED_1000_MBPS:
                    if (duplex)
                    {
                        *speedResolution = MTD_SPEED_1GIG_FD;
                    }
                    else
                    {
                        *speedResolution = MTD_SPEED_1GIG_HD;
                    }
                    break;
                case MTD_CU_SPEED_10_GBPS: /* also MTD_CU_SPEED_NBT */
                    if (MTD_IS_X32X0_BASE(devPtr->deviceId))
                    {
                        *speedResolution = MTD_SPEED_10GIG_FD; /* 10G has only full duplex, ignore duplex bit */
                    }
                    else
                    {
                        MTD_ATTEMPT(mtdHwGetPhyRegField(devPtr,port, \
                                3, 0x8008, 2, 2, &speed2));

                        switch (speed2)
                        {
                            case MTD_CU_SPEED_NBT_10G:
                                *speedResolution = MTD_SPEED_10GIG_FD;
                                break;
                                
                            case MTD_CU_SPEED_NBT_5G:
                                *speedResolution = MTD_SPEED_5GIG_FD;
                                break;
                                
                            case MTD_CU_SPEED_NBT_2P5G:
                                *speedResolution = MTD_SPEED_2P5GIG_FD;
                                break;
                                
                            default:
                                /* this is an error */
                                return MTD_FAIL;                                
                                break;
                        }
                    }
                    break;
                default:
                    /* this is an error */
                    return MTD_FAIL;
                    break;
            }
            
        }

    }

    return MTD_OK;   
}


MTD_STATUS mtdIsBaseTUp(
    IN MTD_DEV_PTR devPtr,
    IN MTD_U16 port,
    OUT MTD_U16 *speed,
    OUT MTD_BOOL *linkUp
)
{
    MTD_BOOL speedIsForced; 
    MTD_U16 forcedSpeed, cuSpeed, cuLinkStatus;

    *linkUp = MTD_FALSE;
    *speed = MTD_ADV_NONE;
    
    /* first check if speed is forced to one of the speeds not requiring AN to train */
    MTD_ATTEMPT(mtdGetForcedSpeed(devPtr,port,&speedIsForced, &forcedSpeed));
    
    if (speedIsForced)
    {
        /* check if the link is up at the speed it's forced to */
        MTD_ATTEMPT(mtdHwGetPhyRegField(devPtr,port,3,0x8008,14,2,&cuSpeed));
        MTD_ATTEMPT(mtdHwGetPhyRegField(devPtr,port,3,0x8008,10,1,&cuLinkStatus));
       
        switch (forcedSpeed)
        {
            case MTD_SPEED_10M_HD_AN_DIS:
            case MTD_SPEED_10M_FD_AN_DIS: /* might want to add checking the duplex to make sure there */
                                          /* is no duplex mismatch */
                if (cuSpeed == MTD_CU_SPEED_10_MBPS)
                {
                    *speed = forcedSpeed;
                }
                else
                {
                    *speed = MTD_SPEED_MISMATCH;
                }
                if (cuLinkStatus)
                {
                    *linkUp = MTD_TRUE;
                }
                break;
            
            case MTD_SPEED_100M_HD_AN_DIS:
            case MTD_SPEED_100M_FD_AN_DIS:/* might want to add checking the duplex to make sure there */
                                          /* is no duplex mismatch */
                if (cuSpeed == MTD_CU_SPEED_100_MBPS)
                {
                    *speed = forcedSpeed;
                }
                else
                {
                    *speed = MTD_SPEED_MISMATCH;
                }
                if (cuLinkStatus)
                {
                    *linkUp = MTD_TRUE;
                }                                    
                break;
            
            default:
                return MTD_FAIL;
                break;
        }
    }
    else
    {
        /* must be going through AN */
        MTD_ATTEMPT(mtdGetAutonegSpeedDuplexResolution(devPtr,port,speed));
        
        if (*speed != MTD_ADV_NONE)
        {
            /* check if the link is up at the speed it's AN to */            
            MTD_ATTEMPT(mtdHwGetPhyRegField(devPtr,port,3,0x8008,10,1,&cuLinkStatus));
            
            switch(*speed)
            {
                case MTD_SPEED_10M_HD:
                case MTD_SPEED_10M_FD:  
                case MTD_SPEED_100M_HD:                 
                case MTD_SPEED_100M_FD:                                       
                case MTD_SPEED_1GIG_HD:
                case MTD_SPEED_1GIG_FD:                                       
                case MTD_SPEED_10GIG_FD:
                case MTD_SPEED_2P5GIG_FD:
                case MTD_SPEED_5GIG_FD:
                    if (cuLinkStatus)
                    {
                        *linkUp = MTD_TRUE;
                    }                                    
                    break;                
                default:
                    return MTD_FAIL;
                    break;
            }
            
        }
        /* else link is down, and AN is in progress, */        
    }

    if (*speed == MTD_SPEED_MISMATCH)
    {
        return MTD_FAIL;
    }
    else
    {
        return MTD_OK;
    }
   
}

MTD_STATUS mtdMDIXControl
(
    IN MTD_DEV_PTR devPtr,
    IN MTD_U16 port,
    IN MTD_U16 MDIXOptions, 
    IN MTD_BOOL swReset
)
{
    if (MDIXOptions != MTD_FORCE_MDI &&
        MDIXOptions != MTD_FORCE_MDIX &&
        MDIXOptions != MTD_AUTO_MDI_MDIX)
    {
        return MTD_FAIL;
    }
    
    MTD_ATTEMPT(mtdHwSetPhyRegField(devPtr,port,3,\
                            0x8000,5,2,MDIXOptions));
   
    if(swReset)
    {
        MTD_ATTEMPT(mtdSoftwareReset(devPtr,port,1000)); /* this needs to be polled for 'done' before
                                                        issuing any other MDIO commands
                                                        it self clears when the reset is finished */
    }

    return MTD_OK;
}

MTD_STATUS mtdGetMDIXControl
(
    IN MTD_DEV_PTR devPtr,
    IN MTD_U16 port,
    OUT MTD_U16 *MDIXOptions
)
{
    MTD_ATTEMPT(mtdHwGetPhyRegField(devPtr,port,3,0x8000,5,2,MDIXOptions));

    return MTD_OK;
}

MTD_STATUS mtdGetMDIXResolution
(
    IN MTD_DEV_PTR devPtr,
    IN MTD_U16 port,
    OUT MTD_U8 *MDIX
)
{
    MTD_BOOL anDone;
    MTD_U16 bitVal;

    MTD_ATTEMPT(mtdAutonegIsSpeedDuplexResolutionDone(devPtr,port,&anDone));

    if (anDone == MTD_FALSE)
    {
        *MDIX = MTD_MDI_MDIX_NOT_RESOLVED;
        return MTD_OK;
    }

    MTD_ATTEMPT(mtdHwGetPhyRegField(devPtr,port,3,0x8008,6,1,&bitVal));

    (bitVal) ? (*MDIX = MTD_MDIX_RESOLVED) : (*MDIX = MTD_MDI_RESOLVED);

    return MTD_OK;
}

MTD_STATUS mtdAutonegSetNbaseTAdvertisement
(
    IN MTD_DEV_PTR devPtr,
    IN MTD_U16 port,
    IN MTD_BOOL enableNbaseTMP,
    IN MTD_BOOL anRestart
)
{
    MTD_U16 availableSpeeds;

    MTD_ATTEMPT(mtdGetSpeedsAvailable(devPtr, port, &availableSpeeds));

    if ((availableSpeeds & (MTD_SPEED_2P5GIG_FD | MTD_SPEED_5GIG_FD)) == 0)
    {
        MTD_DBG_ERROR("mtdAutonegSetNbaseTAdvertisement: NbaseT Advertisement is not supported on this device.\n");
        return MTD_FAIL;
    }

    if (enableNbaseTMP)
    {
        MTD_ATTEMPT(mtdHwSetPhyRegField(devPtr,port, \
                1,0xC000,10,1,0)); /* enables NBASE-T OUI Message Page */
    }
    else
    {
        MTD_ATTEMPT(mtdHwSetPhyRegField(devPtr,port, \
                1,0xC000,10,1,1)) /* disables NBASE-T OUI Message Page */
    }       

    if(anRestart)
    {
        return ((MTD_STATUS)(mtdAutonegEnable(devPtr,port) ||
                             mtdAutonegRestart(devPtr,port)));
    }

    return MTD_OK;
}


MTD_STATUS mtdAutonegSetNbaseTDownshiftControl
(
    IN MTD_DEV_PTR devPtr,
    IN MTD_U16 port,
    IN MTD_U16 downshiftThreshold,
    IN MTD_BOOL noEnergyReset,
    IN MTD_BOOL downshiftEnable,
    IN MTD_BOOL anRestart
)
{
    MTD_U16 temp;
    MTD_U16 availableSpeeds;

    MTD_ATTEMPT(mtdGetSpeedsAvailable(devPtr, port, &availableSpeeds));

    if ((availableSpeeds & (MTD_SPEED_2P5GIG_FD | MTD_SPEED_5GIG_FD)) == 0)
    {
        MTD_DBG_ERROR("mtdAutonegSetNbaseTDownshiftControl: NbaseT Downshift is not supported on this device.\n");
        return MTD_FAIL;
    }

    if (downshiftThreshold == 0)
    {
        downshiftEnable = MTD_FALSE; /* in PHY supposed to work this way anyways */
                                     /* threshold of 0 means to disable downshift */
    }
    
    MTD_ATTEMPT(mtdHwXmdioRead(devPtr,port,1,0xC034,&temp));
    MTD_ATTEMPT(mtdHwSetRegFieldToWord(temp,MTD_GET_BOOL_AS_BIT(downshiftEnable),4,1,&temp));
    MTD_ATTEMPT(mtdHwSetRegFieldToWord(temp,MTD_GET_BOOL_AS_BIT(noEnergyReset),5,1,&temp));
    MTD_ATTEMPT(mtdHwSetRegFieldToWord(temp,downshiftThreshold,0,4,&temp));
    MTD_ATTEMPT(mtdHwXmdioWrite(devPtr,port,1,0xC034,temp));

    if(anRestart)
    {
        return ((MTD_STATUS)(mtdAutonegEnable(devPtr,port) ||
                             mtdAutonegRestart(devPtr,port)));
    }    

    return MTD_OK;
}

MTD_STATUS mtdAutonegGetNbaseTDownshiftControl
(
    IN MTD_DEV_PTR devPtr,
    IN MTD_U16 port,
    IN MTD_U16 *downshiftThreshold,
    IN MTD_BOOL *noEnergyReset,
    IN MTD_BOOL *downshiftEnable
)
{
    MTD_U16 temp,temp2;
    MTD_U16 availableSpeeds;

    MTD_ATTEMPT(mtdGetSpeedsAvailable(devPtr, port, &availableSpeeds));

    if ((availableSpeeds & (MTD_SPEED_2P5GIG_FD | MTD_SPEED_5GIG_FD)) == 0)
    {
        MTD_DBG_ERROR("mtdAutonegGetNbaseTDownshiftControl: NbaseT Downshift is not supported on this device.\n");
        return MTD_FAIL;
    }
    
    MTD_ATTEMPT(mtdHwXmdioRead(devPtr,port,1,0xC034,&temp));
    
    MTD_ATTEMPT(mtdHwGetRegFieldFromWord(temp,4,1,&temp2));
    *downshiftEnable = MTD_GET_BIT_AS_BOOL(temp2);    

    MTD_ATTEMPT(mtdHwGetRegFieldFromWord(temp,5,1,&temp2));
    *noEnergyReset= MTD_GET_BIT_AS_BOOL(temp2);        

    MTD_ATTEMPT(mtdHwGetRegFieldFromWord(temp,0,4,downshiftThreshold));    
    
    return MTD_OK;
}

MTD_STATUS mtdAutonegGetNbaseTDownshiftStatus
(
    IN MTD_DEV_PTR devPtr,
    IN MTD_U16 port,
    IN MTD_BOOL *downshiftFrom10G,
    IN MTD_BOOL *downshiftFrom5G,
    IN MTD_BOOL *downshiftFrom2P5G,
    IN MTD_BOOL *downshiftFrom1G,
    IN MTD_U16 *downshiftCount
)
{
    MTD_U16 temp,temp2;
    MTD_U16 availableSpeeds;

    MTD_ATTEMPT(mtdGetSpeedsAvailable(devPtr, port, &availableSpeeds));

    if ((availableSpeeds & (MTD_SPEED_2P5GIG_FD | MTD_SPEED_5GIG_FD)) == 0)
    {
        MTD_DBG_ERROR("mtdAutonegGetNbaseTDownshiftStatus: NbaseT Downshift is not supported on this device.\n");
        return MTD_FAIL;
    }

    MTD_ATTEMPT(mtdHwXmdioRead(devPtr,port,1,0xC035,&temp));
    
    MTD_ATTEMPT(mtdHwGetRegFieldFromWord(temp,0,1,&temp2));
    *downshiftFrom1G = MTD_GET_BIT_AS_BOOL(temp2);    

    MTD_ATTEMPT(mtdHwGetRegFieldFromWord(temp,1,1,&temp2));
    *downshiftFrom2P5G = MTD_GET_BIT_AS_BOOL(temp2); 

    MTD_ATTEMPT(mtdHwGetRegFieldFromWord(temp,2,1,&temp2));
    *downshiftFrom5G = MTD_GET_BIT_AS_BOOL(temp2); 
    
    MTD_ATTEMPT(mtdHwGetRegFieldFromWord(temp,3,1,&temp2));
    *downshiftFrom10G = MTD_GET_BIT_AS_BOOL(temp2); 
     
    MTD_ATTEMPT(mtdHwGetRegFieldFromWord(temp,9,4,downshiftCount));        

    return MTD_OK;
}

/*******************************************************************
  T unit interrupts (copper interrupts)
 *******************************************************************/
MTD_STATUS mtdEnableDisableTunitInterrupts
(
    IN MTD_DEV_PTR devPtr,
    IN MTD_U16  port,
    IN MTD_U32  interrupt_sel,
    IN MTD_BOOL enable
)
{
    MTD_U16 regValue1,regValue2;

    MTD_ATTEMPT(mtdHwXmdioRead(devPtr,port,MTD_TUNIT_INTERRUPT_CTRL1,&regValue1));
    MTD_ATTEMPT(mtdHwXmdioRead(devPtr,port,MTD_TUNIT_INTERRUPT_CTRL2,&regValue2));

    if (enable)
    {
        regValue1 |= (interrupt_sel & 0xFFFF);
        regValue2 |= ((interrupt_sel>>16) & 0xFFFF);
    }
    else
    {
        regValue1 &= ~(interrupt_sel & 0xFFFF);
        regValue2 &= ~((interrupt_sel>>16) & 0xFFFF);
    }

    MTD_ATTEMPT(mtdHwXmdioWrite(devPtr,port,MTD_TUNIT_INTERRUPT_CTRL1,regValue1));
    MTD_ATTEMPT(mtdHwXmdioWrite(devPtr,port,MTD_TUNIT_INTERRUPT_CTRL2,regValue2));

    return MTD_OK;

}

MTD_STATUS mtdGetTunitInterruptsEnabled
(
    IN MTD_DEV_PTR devPtr,
    IN MTD_U16   port,
    OUT MTD_U32 *interruptEnabled
)
{
    MTD_U16 regValue1,regValue2;
    MTD_U32 temp;

    MTD_ATTEMPT(mtdHwXmdioRead(devPtr,port,MTD_TUNIT_INTERRUPT_CTRL1,&regValue1));
    MTD_ATTEMPT(mtdHwXmdioRead(devPtr,port,MTD_TUNIT_INTERRUPT_CTRL2,&regValue2));

    temp = (((MTD_U32)regValue2 << 16) & 0xFFFF0000)|((MTD_U32)regValue1 & 0x0000FFFF);
    *interruptEnabled = temp;

    return MTD_OK;

}

MTD_STATUS mtdGetTunitInterruptStatus
(
    IN MTD_DEV_PTR devPtr,
    IN MTD_U16   port,
    OUT MTD_U32 *interruptStatus
)
{
    MTD_U16 regValue1,regValue2;
    MTD_U32 temp;

    MTD_ATTEMPT(mtdHwXmdioRead(devPtr,port,MTD_TUNIT_INTERRUPT_STATUS1,&regValue1));
    MTD_ATTEMPT(mtdHwXmdioRead(devPtr,port,MTD_TUNIT_INTERRUPT_STATUS2,&regValue2));

    temp = (((MTD_U32)regValue2 << 16) & 0xFFFF0000)|((MTD_U32)regValue1 & 0x0000FFFF);
    *interruptStatus = temp;

    return MTD_OK;
}

/*******************************************************************
  Processor interrupts (only on phys X35X0 and after)
 *******************************************************************/
MTD_STATUS mtdEnableDisableProcInterrupts
(
    IN MTD_DEV_PTR devPtr,
    IN MTD_U16  port,
    IN MTD_U16  interrupt_sel,
    IN MTD_BOOL enable
)
{
    MTD_U16 regValue;

    if (MTD_PROC_INTR_SUPPORT(devPtr) == MTD_FALSE)
    {
        MTD_DBG_ERROR("mtdEnableDisableProcInterrupts() - Doesn't support this device.\n");
        return MTD_FAIL;
    }

    /* only allow valid interrupt source 15:6 bit being set/unset */
    if (((interrupt_sel & MTD_ALL_PROC_INTR) == 0) || ((interrupt_sel & ~MTD_ALL_PROC_INTR) != 0))
    {
        MTD_DBG_ERROR("mtdEnableDisableProcInterrupts() - Invalid interrupt source.\n");
        return MTD_FAIL;
    }

    MTD_ATTEMPT(mtdHwXmdioRead(devPtr, port, MTD_PROC_INTR_CTRL, &regValue));

    if (enable)
    {
        regValue = (regValue | interrupt_sel | MTD_PROC_INTR_TOP);
    }
    else
    {
        regValue &= ~interrupt_sel;

        if ((regValue & MTD_ALL_PROC_INTR) == 0)
        {
            /* If no other interrupt source has been set, turn off the overall interrupt bit as well */
            regValue &= ~MTD_PROC_INTR_TOP;
        }
    }

    MTD_ATTEMPT(mtdHwXmdioWrite(devPtr, port, MTD_PROC_INTR_CTRL, regValue));

    return MTD_OK;
}


MTD_STATUS mtdGetProcInterruptsEnabled
(
    IN MTD_DEV_PTR devPtr,
    IN MTD_U16 port,
    OUT MTD_U16 *interruptEnabled
)
{
    if (MTD_PROC_INTR_SUPPORT(devPtr) == MTD_FALSE)
    {
        MTD_DBG_ERROR("mtdGetProcInterruptsEnabled() - Doesn't support this device.\n");
        return MTD_FAIL;
    }

     MTD_ATTEMPT(mtdHwXmdioRead(devPtr, port, MTD_PROC_INTR_CTRL, interruptEnabled));

     if ((*interruptEnabled & MTD_PROC_INTR_TOP) == 0) /* If overall bit is disabled */
     {
         *interruptEnabled = 0;
     }
     else
     {
         /* only report valid source bits to user */
        *interruptEnabled = *interruptEnabled & MTD_ALL_PROC_INTR;
     }

     return MTD_OK;
}


MTD_STATUS mtdGetProcInterruptStatus
(
    IN MTD_DEV_PTR devPtr,
    MTD_U16 port,
    OUT MTD_U16 *interruptStatus
)
{
    if (MTD_PROC_INTR_SUPPORT(devPtr) == MTD_FALSE)
    {
        MTD_DBG_ERROR("mtdGetProcInterruptStatus() - Doesn't support this device.\n");
        return MTD_FAIL;
    }

    MTD_ATTEMPT(mtdHwXmdioRead(devPtr, port, MTD_PROC_INTR_STAUS, interruptStatus));
    MTD_ATTEMPT(mtdHwXmdioWrite(devPtr, port, MTD_PROC_INTR_STAUS, MTD_PROC_INTR_TOP)); /* clear all interrupt status */

    return MTD_OK;
}


/******************************************************************************
 Functions shared between H unit and X unit for 1000BX/SGMII
 Fiber interface (X unit) and Host Interface (H unit) Control/Status
******************************************************************************/

MTD_STATUS mtdSet1000BXSGMIIControl
(
    IN MTD_DEV_PTR devPtr,
    IN MTD_U16 port,
    IN MTD_U16 HorXunit,
    IN MTD_BOOL loopback,
    IN MTD_U16 speed,
    IN MTD_BOOL an_enable,
    IN MTD_BOOL power_down,
    IN MTD_BOOL restart_an,
    IN MTD_BOOL sw_reset
)
{       
    MTD_U16 bit6, bit13, temp;

    if (HorXunit == MTD_X_UNIT && !MTD_HAS_X_UNIT(devPtr))
    {
        MTD_DBG_ERROR("mtdSet1000BXSGMIIControl: This device doesn't have X Unit(Media/Fiber Interface).\n");
        return MTD_FAIL;
    }

    if (speed > MTD_SGMII_SPEED_ASIS)
    {
        return MTD_FAIL;
    }

    MTD_ATTEMPT(mtdHwXmdioRead(devPtr,port,HorXunit,MTD_1000X_SGMII_CONTROL,&temp));
  
    MTD_ATTEMPT(mtdHwSetRegFieldToWord(temp,MTD_GET_BOOL_AS_BIT(loopback),14,1,&temp));

    if (speed != MTD_SGMII_SPEED_ASIS)
    {
        switch (speed)
        {
            case MTD_SGMII_SPEED_10M:
                bit6 = bit13 = 0;
                break;

            case MTD_SGMII_SPEED_100M:
                bit6 = 0;
                bit13 = 1;                
                break;

            case MTD_SGMII_SPEED_1G:
                bit6 = 1;
                bit13 = 0;
                break;

            default:
                return MTD_FAIL;
                break;
        }

        MTD_ATTEMPT(mtdHwSetRegFieldToWord(temp,bit6,6,1,&temp));
        MTD_ATTEMPT(mtdHwSetRegFieldToWord(temp,bit13,13,1,&temp));
        
    }

      
    MTD_ATTEMPT(mtdHwSetRegFieldToWord(temp,MTD_GET_BOOL_AS_BIT(an_enable),12,1,&temp));  
    MTD_ATTEMPT(mtdHwSetRegFieldToWord(temp,MTD_GET_BOOL_AS_BIT(power_down),11,1,&temp));
    MTD_ATTEMPT(mtdHwSetRegFieldToWord(temp,MTD_GET_BOOL_AS_BIT(restart_an),9,1,&temp)); 
    MTD_ATTEMPT(mtdHwSetRegFieldToWord(temp,MTD_GET_BOOL_AS_BIT(sw_reset),15,1,&temp));

    MTD_ATTEMPT(mtdHwXmdioWrite(devPtr,port,HorXunit,MTD_1000X_SGMII_CONTROL,temp));
    
    return MTD_OK;     
}

MTD_STATUS mtdGet1000BXSGMIIControl
(
    IN MTD_DEV_PTR devPtr,
    IN MTD_U16 port,
    IN MTD_U16 HorXunit,
    OUT MTD_BOOL *loopback,
    OUT MTD_U16 *speed,
    OUT MTD_BOOL *an_enable,
    OUT MTD_BOOL *power_down
)
{
    MTD_U16 temp,bit6,bit13; 

    if (HorXunit == MTD_X_UNIT && !MTD_HAS_X_UNIT(devPtr))
    {
        MTD_DBG_ERROR("mtdGet1000BXSGMIIControl: This device doesn't have X Unit(Media/Fiber Interface).\n");
        return MTD_FAIL;
    }

    MTD_ATTEMPT(mtdHwGetPhyRegField(devPtr,port,HorXunit,MTD_1000X_SGMII_CONTROL,14,1,&temp));
    MTD_CONVERT_UINT_TO_BOOL(temp,*loopback);
    
    MTD_ATTEMPT(mtdHwGetPhyRegField(devPtr,port,HorXunit,MTD_1000X_SGMII_CONTROL,6,1,&bit6));   
    MTD_ATTEMPT(mtdHwGetPhyRegField(devPtr,port,HorXunit,MTD_1000X_SGMII_CONTROL,13,1,&bit13));
    *speed = (bit6<<1) | bit13;

    MTD_ATTEMPT(mtdHwGetPhyRegField(devPtr,port,HorXunit,MTD_1000X_SGMII_CONTROL,12,1,&temp));
    MTD_CONVERT_UINT_TO_BOOL(temp,*an_enable);
    
    MTD_ATTEMPT(mtdHwGetPhyRegField(devPtr,port,HorXunit,MTD_1000X_SGMII_CONTROL,11,1,&temp));
    MTD_CONVERT_UINT_TO_BOOL(temp,*power_down);
    
    return MTD_OK;
}

MTD_STATUS mtdGet1000BXSGMIIStatus
(
    IN MTD_DEV_PTR devPtr,
    IN MTD_U16 port,
    IN MTD_U16 HorXunit,
    OUT MTD_BOOL *an_complete,
    OUT MTD_BOOL *remote_fault,
    OUT MTD_BOOL *link_status_latched,
    OUT MTD_BOOL *link_status_current
)
{
    MTD_U16 temp,temp2;

    if (HorXunit == MTD_X_UNIT && !MTD_HAS_X_UNIT(devPtr))
    {
        MTD_DBG_ERROR("mtdGet1000BXSGMIIStatus: This device doesn't have X Unit(Media/Fiber Interface).\n");
        return MTD_FAIL;
    }

    /* read once to get all bits including latched low link status (may clear LL link status) */
    MTD_ATTEMPT(mtdHwXmdioRead(devPtr,port,HorXunit,MTD_1000X_SGMII_STATUS,&temp));

    mtdHwGetRegFieldFromWord(temp,5,1,&temp2);
    MTD_CONVERT_UINT_TO_BOOL(temp2,*an_complete);

    mtdHwGetRegFieldFromWord(temp,4,1,&temp2);
    MTD_CONVERT_UINT_TO_BOOL(temp2,*remote_fault);

    mtdHwGetRegFieldFromWord(temp,2,1,&temp2);
    MTD_CONVERT_UINT_TO_BOOL(temp2,*link_status_latched);

    /* read it again to get current link status */
    MTD_ATTEMPT(mtdHwGetPhyRegField(devPtr,port,HorXunit,MTD_1000X_SGMII_STATUS,2,1,&temp));
    MTD_CONVERT_UINT_TO_BOOL(temp,*link_status_current);
    
    return MTD_OK;
}

/******************************************************************************
 Functions shared between H unit and X unit for 10GBASE-R PCS (XFI/SFI)
 X unit (Fiber interface) and H unit (Host interface) Control/Status
******************************************************************************/

#define MTD_USXGMII_AN_CNTL MTD_T_UNIT_PMA_PMD, 0xC049 /* disable/enable AN for USXGMII Mac Type*/
MTD_STATUS mtdUSXGMIIAutoNeg
(
    IN MTD_DEV_PTR devPtr,
    IN MTD_U16 port,
    IN MTD_U16 enableAutoNeg,
    IN MTD_BOOL do10GBRSwReset
)
{
    MTD_BOOL loopback;
    MTD_BOOL power_down;
    MTD_BOOL sw_reset = MTD_TRUE;

    if (MTD_IS_X33X0_BASE(devPtr->deviceId) || MTD_IS_E21X0_BASE(devPtr->deviceId))
    {
        /* Control register logic: 1 for disable, 0 for enable */
        if (enableAutoNeg == MTD_DISABLE)
        {
            /* Set bit to disable the AutoNeg */
            MTD_ATTEMPT(mtdHwSetPhyRegField(devPtr,port,MTD_USXGMII_AN_CNTL,0,1,0x1));
        }
        else if (enableAutoNeg == MTD_ENABLE)
        {
            /* Clear bit to enable the AutoNeg */
            MTD_ATTEMPT(mtdHwSetPhyRegField(devPtr,port,MTD_USXGMII_AN_CNTL,0,1,0x0));
        }
        else
        {
            MTD_DBG_ERROR("mtdUSXGMIIAutoNeg: Incorrect Auto-Neg Option\n");
        }

        if (do10GBRSwReset == MTD_TRUE)
        {
            MTD_ATTEMPT(mtdGetUSXGMIIControl(devPtr,port,MTD_H_UNIT,&loopback,&power_down));
            MTD_ATTEMPT(mtdSetUSXGMIIControl(devPtr,port,MTD_H_UNIT,loopback,power_down,sw_reset));
        }
    }
    else
    {
        if (enableAutoNeg == MTD_DISABLE)
        {
            MTD_ATTEMPT(mtdHwSetPhyRegField(devPtr,port,4,0xF0A0,2,2,0x2));
        }
        else if (enableAutoNeg == MTD_ENABLE)
        {
            MTD_ATTEMPT(mtdHwSetPhyRegField(devPtr,port,4,0xF0A0,2,2,0x3));
        }
        else
        {
            MTD_DBG_ERROR("mtdUSXGMIIAutoNeg: Incorrect Auto-Neg Option\n");
        }
    }
    
    return MTD_OK;
}

MTD_STATUS mtdSet10GBRControl
(
    IN MTD_DEV_PTR devPtr,
    IN MTD_U16 port,
    IN MTD_U16 HorXunit,
    IN MTD_BOOL loopback,
    IN MTD_BOOL power_down,
    IN MTD_BOOL sw_reset
)
{
    MTD_U16 temp;

    if (HorXunit == MTD_X_UNIT && !MTD_HAS_X_UNIT(devPtr))
    {
        MTD_DBG_ERROR("mtdSet10GBRControl/mtdSetUSXGMIIControl: This device doesn't have X Unit(Media/Fiber Interface).\n");
        return MTD_FAIL;
    }

    MTD_ATTEMPT(mtdHwXmdioRead(devPtr,port,HorXunit,MTD_10GBR_PCS_CONTROL,&temp));    

    MTD_ATTEMPT(mtdHwSetRegFieldToWord(temp,MTD_GET_BOOL_AS_BIT(loopback),14,1,&temp)); 
    
    MTD_ATTEMPT(mtdHwSetRegFieldToWord(temp,MTD_GET_BOOL_AS_BIT(power_down),11,1,&temp)); 

    MTD_ATTEMPT(mtdHwSetRegFieldToWord(temp,MTD_GET_BOOL_AS_BIT(sw_reset),15,1,&temp)); 

    MTD_ATTEMPT(mtdHwXmdioWrite(devPtr,port,HorXunit,MTD_10GBR_PCS_CONTROL,temp));

    if (MTD_IS_E21X0_BASE(devPtr->deviceId))
    {
        /* Needs workaround for loopback to work in E21X0 devices */
        if (loopback)
        {
            MTD_ATTEMPT(mtdHwSetPhyRegField(devPtr,port,4,0xF07E,13,3,5)); /* force signal lock */ 
        }
        else
        {
            MTD_ATTEMPT(mtdHwSetPhyRegField(devPtr,port,4,0xF07E,13,3,1)); /* clear overwrite bit */
        }
    }
    
    return MTD_OK;

}

MTD_STATUS mtdGet10GBRControl
(
    IN MTD_DEV_PTR devPtr,
    IN MTD_U16 port,
    IN MTD_U16 HorXunit,
    OUT MTD_BOOL *loopback,
    OUT MTD_BOOL *power_down
)
{
    MTD_U16 lb,pd,temp;

    if (HorXunit == MTD_X_UNIT && !MTD_HAS_X_UNIT(devPtr))
    {
        MTD_DBG_ERROR("mtdGet10GBRControl/mtdGetUSXGMIIControl: This device doesn't have X Unit(Media/Fiber Interface).\n");
        return MTD_FAIL;
    }

    MTD_ATTEMPT(mtdHwXmdioRead(devPtr,port,HorXunit,MTD_10GBR_PCS_CONTROL,&temp));
    mtdHwGetRegFieldFromWord(temp,14,1,&lb);
    mtdHwGetRegFieldFromWord(temp,11,1,&pd);
    MTD_CONVERT_UINT_TO_BOOL(lb, *loopback);
    MTD_CONVERT_UINT_TO_BOOL(pd, *power_down);
        
    return MTD_OK;
}

MTD_STATUS mtdGet10GBRStatus1
(
    IN MTD_DEV_PTR devPtr,
    IN MTD_U16 port,
    IN MTD_U16 HorXunit,
    OUT MTD_BOOL *tx_lpi_latch,
    OUT MTD_BOOL *rx_lpi_latch,
    OUT MTD_BOOL *tx_lpi_current,
    OUT MTD_BOOL *rx_lpi_current,
    OUT MTD_BOOL *fault,
    OUT MTD_BOOL *link_status_latch,
    OUT MTD_BOOL *link_status_current    
)
{
    MTD_U16 temp1,temp2,temp3;

    if (HorXunit == MTD_X_UNIT && !MTD_HAS_X_UNIT(devPtr))
    {
        MTD_DBG_ERROR("mtdGet10GBRStatus1/mtdGetUSXGMIIStatus1: This device doesn't have X Unit(Media/Fiber Interface).\n");
        return MTD_FAIL;
    }

    MTD_ATTEMPT(mtdHwXmdioRead(devPtr,port,HorXunit,MTD_10GBR_PCS_STATUS1,&temp1)); /* latched value */

    mtdHwGetRegFieldFromWord(temp1,11,1,&temp3);
    MTD_CONVERT_UINT_TO_BOOL(temp3, *tx_lpi_latch);
    mtdHwGetRegFieldFromWord(temp1,10,1,&temp3);
    MTD_CONVERT_UINT_TO_BOOL(temp3, *rx_lpi_latch);
    mtdHwGetRegFieldFromWord(temp1,2,1,&temp3);
    MTD_CONVERT_UINT_TO_BOOL(temp3, *link_status_latch);
            
    MTD_ATTEMPT(mtdHwXmdioRead(devPtr,port,HorXunit,MTD_10GBR_PCS_STATUS1,&temp2)); /* current value */

    mtdHwGetRegFieldFromWord(temp2,9,1,&temp3);
    MTD_CONVERT_UINT_TO_BOOL(temp3, *tx_lpi_current);
    mtdHwGetRegFieldFromWord(temp2,8,1,&temp3);
    MTD_CONVERT_UINT_TO_BOOL(temp3, *rx_lpi_current);
    mtdHwGetRegFieldFromWord(temp2,7,1,&temp3);
    MTD_CONVERT_UINT_TO_BOOL(temp3, *fault);
    mtdHwGetRegFieldFromWord(temp2,2,1,&temp3);
    MTD_CONVERT_UINT_TO_BOOL(temp3, *link_status_current);
        
    return MTD_OK;
}

MTD_STATUS mtdGet10GBRFault
(
    IN MTD_DEV_PTR devPtr,
    IN MTD_U16 port,
    IN MTD_U16 HorXunit,    
    OUT MTD_BOOL *tx_fault_latch,
    OUT MTD_BOOL *rx_fault_latch,
    OUT MTD_BOOL *tx_fault_current,
    OUT MTD_BOOL *rx_fault_current
)
{
    MTD_U16 temp1, fault;

    if (HorXunit == MTD_X_UNIT && !MTD_HAS_X_UNIT(devPtr))
    {
        MTD_DBG_ERROR("mtdGet10GBRFault/mtdGetUSXGMIIFault: This device doesn't have X Unit(Media/Fiber Interface).\n");
        return MTD_FAIL;
    }

    MTD_ATTEMPT(mtdHwXmdioRead(devPtr,port,HorXunit,MTD_10GBR_PCS_FAULT,&temp1)); /* latched value */
    mtdHwGetRegFieldFromWord(temp1,11,1,&fault);
    MTD_CONVERT_UINT_TO_BOOL(fault, *tx_fault_latch);
    mtdHwGetRegFieldFromWord(temp1,10,1,&fault);
    MTD_CONVERT_UINT_TO_BOOL(fault, *rx_fault_latch);

    
    MTD_ATTEMPT(mtdHwXmdioRead(devPtr,port,HorXunit,MTD_10GBR_PCS_FAULT,&temp1)); /* current value */
    mtdHwGetRegFieldFromWord(temp1,11,1,&fault);
    MTD_CONVERT_UINT_TO_BOOL(fault, *tx_fault_current);
    mtdHwGetRegFieldFromWord(temp1,10,1,&fault);
    MTD_CONVERT_UINT_TO_BOOL(fault, *rx_fault_current);

    return MTD_OK;
}

MTD_STATUS mtdGet10GBRReceiveStatus
(
    IN MTD_DEV_PTR devPtr,
    IN MTD_U16 port,
    IN MTD_U16 HorXunit, 
    OUT MTD_BOOL *link_up,
    OUT MTD_BOOL *high_ber,
    OUT MTD_BOOL *block_lock    
)
{
    MTD_U16 temp1, temp2;

    if (HorXunit == MTD_X_UNIT && !MTD_HAS_X_UNIT(devPtr))
    {
        MTD_DBG_ERROR("mtdGet10GBRReceiveStatus/mtdGetUSXGMIIReceiveStatus: This device doesn't have X Unit(Media/Fiber Interface).\n");
        return MTD_FAIL;
    }

    MTD_ATTEMPT(mtdHwXmdioRead(devPtr,port,HorXunit,MTD_10GBR_PCS_RXSTATUS,&temp1)); /* current value */
    mtdHwGetRegFieldFromWord(temp1,12,1,&temp2);
    MTD_CONVERT_UINT_TO_BOOL(temp2, *link_up);
    mtdHwGetRegFieldFromWord(temp1,1,1,&temp2);
    MTD_CONVERT_UINT_TO_BOOL(temp2, *high_ber);
    mtdHwGetRegFieldFromWord(temp1,0,1,&temp2);
    MTD_CONVERT_UINT_TO_BOOL(temp2, *block_lock);
    
    
    return MTD_OK;
}

MTD_STATUS mtdGet10GBRStatus2
(
    IN MTD_DEV_PTR devPtr,
    IN MTD_U16 port,
    IN MTD_U16 HorXunit, 
    OUT MTD_BOOL *has_block_lock,
    OUT MTD_BOOL *reported_high_ber,
    OUT MTD_U8 *ber_counter,
    OUT MTD_U8 *errored_blocks_counter    
)
{
    MTD_U16 temp1, temp2;

    if (HorXunit == MTD_X_UNIT && !MTD_HAS_X_UNIT(devPtr))
    {
        MTD_DBG_ERROR("mtdGet10GBRStatus2/mtdGetUSXGMIIStatus2: This device doesn't have X Unit(Media/Fiber Interface).\n");
        return MTD_FAIL;
    }

    MTD_ATTEMPT(mtdHwXmdioRead(devPtr,port,HorXunit,MTD_10GBR_PCS_STATUS2,&temp1)); /* latch value, clears latch/counters */
    mtdHwGetRegFieldFromWord(temp1,15,1,&temp2);

    MTD_CONVERT_UINT_TO_BOOL(temp2, *has_block_lock);

    mtdHwGetRegFieldFromWord(temp1,14,1,&temp2);
    MTD_CONVERT_UINT_TO_BOOL(temp2, *reported_high_ber);

    mtdHwGetRegFieldFromWord(temp1,8,6,&temp2);
    *ber_counter = (MTD_U8)temp2;
    
    mtdHwGetRegFieldFromWord(temp1,0,8,&temp2);
    *errored_blocks_counter = (MTD_U8)temp2;
    
    return MTD_OK;
}

/*******************************************************************
  X unit (Fiber Interface) and H unit (Host Interface) Interrupts
  for BASE-R PCS
 *******************************************************************/
MTD_STATUS mtdEnableDisableBaseRInterrupts
(
    IN MTD_DEV_PTR devPtr,
    IN MTD_U16  port,
    IN MTD_U16  HorXunit,
    IN MTD_U16  interrupt_sel,
    IN MTD_BOOL enable
)
{
    MTD_U16 regValue;
    
    if ((HorXunit==MTD_X_UNIT)&&(!MTD_HAS_X_UNIT(devPtr)))
    {
        MTD_DBG_ERROR("mtdEnableDisableBaseRInterrupts() - No X-unit on this device.\n");
        return MTD_FAIL;
    }

    MTD_ATTEMPT(mtdHwXmdioRead(devPtr,port,HorXunit,MTD_10GBR_INTERRUPT_CTRL,&regValue));

    if (enable)
    {
        regValue |= interrupt_sel;
    }
    else
    {
        regValue &= ~interrupt_sel;
    }

    MTD_ATTEMPT(mtdHwXmdioWrite(devPtr,port,HorXunit,MTD_10GBR_INTERRUPT_CTRL,regValue));

    return MTD_OK;
}


MTD_STATUS mtdGetBaseRInterruptsEnabled
(
    IN MTD_DEV_PTR devPtr,
    IN MTD_U16   port,
    IN MTD_U16   HorXunit,
    OUT MTD_U16 *interruptEnabled
)
{

    if ((HorXunit==MTD_X_UNIT)&&(!MTD_HAS_X_UNIT(devPtr)))
    {
        MTD_DBG_ERROR("mtdGetBaseRInterruptsEnabled() - No X-unit on this PHY.\n");
        return MTD_FAIL;
    }

    MTD_ATTEMPT(mtdHwXmdioRead(devPtr,port,HorXunit,MTD_10GBR_INTERRUPT_CTRL,interruptEnabled));

    return MTD_OK;
}


MTD_STATUS mtdGetBaseRInterruptStatus
(
    IN MTD_DEV_PTR devPtr,
    IN MTD_U16   port,
    IN MTD_U16   HorXunit,
    OUT MTD_U16 *interruptStatus
)
{

    if ((HorXunit==MTD_X_UNIT)&&(!MTD_HAS_X_UNIT(devPtr)))
    {
        MTD_DBG_ERROR("mtdGetBaseRInterruptStatus() - No X-unit on this PHY.\n");
        return MTD_FAIL;
    }

    MTD_ATTEMPT(mtdHwXmdioRead(devPtr,port,HorXunit,MTD_10GBR_INTERRUPT_STATUS,interruptStatus));

    return MTD_OK;
}

MTD_STATUS mtdGetBaseRrealTimeStatus
(
    IN MTD_DEV_PTR devPtr,
    IN MTD_U16   port,
    IN MTD_U16   HorXunit,
    OUT MTD_U16 *realTimeStatus
)
{

    if ((HorXunit==MTD_X_UNIT)&&(!MTD_HAS_X_UNIT(devPtr)))
    {
        MTD_DBG_ERROR("mtdGetBaseRrealTimeStatus() - No X-unit on this PHY.\n");
        return MTD_FAIL;
    }

    MTD_ATTEMPT(mtdHwXmdioRead(devPtr,port,HorXunit,MTD_10GBR_REALTIME_STATUS,realTimeStatus));

    return MTD_OK;
}

/*******************************************************************
  X unit (Fiber interface) and H unit (Host interface) Interrupts
  for BASE-X PCS
 *******************************************************************/
MTD_STATUS mtdEnableDisableBaseXInterrupts
(
    IN MTD_DEV_PTR devPtr,
    IN MTD_U16  port,
    IN MTD_U16  HorXunit,
    IN MTD_U16  interrupt_sel,
    IN MTD_BOOL enable
)
{
    MTD_U16 regValue;

    if ((HorXunit==MTD_X_UNIT)&&(!MTD_HAS_X_UNIT(devPtr)))
    {
        MTD_DBG_ERROR("mtdEnableDisableBaseXInterrupts() - No X-unit on this PHY.\n");
        return MTD_FAIL;
    }

    MTD_ATTEMPT(mtdHwXmdioRead(devPtr,port,HorXunit,MTD_1GBX_INTERRUPT_CTRL,&regValue));

    if (enable)
    {
        regValue |= interrupt_sel;
    }
    else
    {
        regValue &= ~interrupt_sel;
    }

    MTD_ATTEMPT(mtdHwXmdioWrite(devPtr,port,HorXunit,MTD_1GBX_INTERRUPT_CTRL,regValue));

    return MTD_OK;
}


MTD_STATUS mtdGetBaseXInterruptsEnabled
(
    IN MTD_DEV_PTR devPtr,
    IN MTD_U16   port,
    IN MTD_U16   HorXunit,
    OUT MTD_U16 *interruptEnabled
)
{

    if ((HorXunit==MTD_X_UNIT)&&(!MTD_HAS_X_UNIT(devPtr)))
    {
        MTD_DBG_ERROR("mtdGetBaseXInterruptsEnabled() - No X-unit on this PHY.\n");
        return MTD_FAIL;
    }

    MTD_ATTEMPT(mtdHwXmdioRead(devPtr,port,HorXunit,MTD_1GBX_INTERRUPT_CTRL,interruptEnabled));

    return MTD_OK;
}


MTD_STATUS mtdGetBaseXInterruptStatus
(
    IN MTD_DEV_PTR devPtr,
    IN MTD_U16   port,
    IN MTD_U16   HorXunit,
    OUT MTD_U16 *interruptStatus
)
{

    if ((HorXunit==MTD_X_UNIT)&&(!MTD_HAS_X_UNIT(devPtr)))
    {
        MTD_DBG_ERROR("mtdGetBaseXInterruptStatus() - No X-unit on this PHY.\n");
        return MTD_FAIL;
    }

    MTD_ATTEMPT(mtdHwXmdioRead(devPtr,port,HorXunit,MTD_1GBX_INTERRUPT_STATUS,interruptStatus));

    return MTD_OK;
}


MTD_STATUS mtdGetBaseXrealTimeStatus
(
    IN MTD_DEV_PTR devPtr,
    IN MTD_U16   port,
    IN MTD_U16   HorXunit,
    OUT MTD_U16 *realTimeStatus
)
{
    if ((HorXunit==MTD_X_UNIT)&&(!MTD_HAS_X_UNIT(devPtr)))
    {
        MTD_DBG_ERROR("mtdGetBaseXrealTimeStatus() - No X-unit on this PHY.\n");
        return MTD_FAIL;
    }

    MTD_ATTEMPT(mtdHwXmdioRead(devPtr,port,HorXunit,MTD_1GBX_REALTIME_STATUS,realTimeStatus));

    return MTD_OK;
}

/******************************************************************************
 Functions for accessing the temperature sensor
******************************************************************************/

/* Determines which port the temperature sensor is located on. Not every port on */
/* every PHY has a temperature sensor. */
MTD_STATUS mtdGetTempSensorMDIOPort
(
    IN MTD_DEV_PTR devPtr,
    IN MTD_U16   port,
    OUT MTD_U16 *sensorPort
)
{
    MTD_U16 numPortsPerDevice, basePort, portOffset;

    if (devPtr->numPortsPerDevice == 0)
    {
        MTD_DBG_ERROR("mtdGetTempSensorMDIOPort: numPortsPerDevice cannot be 0\n");
        return MTD_FAIL;
    }

    numPortsPerDevice = devPtr->numPortsPerDevice;
    basePort = (port/numPortsPerDevice) * numPortsPerDevice;   
    portOffset = port - basePort;  /* take the relative port number from MDIO port */

    if (MTD_IS_E21X0_BASE(devPtr->deviceId))
    {
        if (numPortsPerDevice == 1) /* E2110 */
        {
            *sensorPort = basePort;
        }
        else if (numPortsPerDevice == 4) /* E2140 */
        {
            *sensorPort = basePort + 1;
        }
        else /* E2180 */
        {
            if ((portOffset >=2) && (portOffset <=5))
            {
                *sensorPort = basePort + 3; /* 2nd DIE */
            }
            else
            {
                *sensorPort = basePort + 1; /* 1st DIE */
            }
        }
    }
    else if (MTD_IS_X32X0_BASE(devPtr->deviceId))
    {
        if (numPortsPerDevice == 2) /* X3220 */
        {
            *sensorPort = basePort + 1;
        }
        else /* X3240 */
        {
            *sensorPort = basePort + 3;
        }
    }
    else if (MTD_IS_X33X0_BASE(devPtr->deviceId) ||
             MTD_IS_X35X0_BASE(devPtr->deviceId))
    {
        *sensorPort = port;
    }
    else if (MTD_IS_X36X0_BASE(devPtr->deviceId))
    {
        if (numPortsPerDevice == 8)
        {
            *sensorPort = basePort;
        }
        else
        {
            *sensorPort = port;
        }
    }
    else
    {
        MTD_DBG_ERROR("mtdGetTempSensorMDIOPort: Device is not supported.\n");
        return MTD_FAIL;
    }

    return MTD_OK;
}

MTD_STATUS mtdEnableTemperatureSensor
(
    IN MTD_DEV_PTR devPtr,
    IN MTD_U16 port
)
{
    MTD_U16 sensorPort;
    MTD_U16 e21x0SensorSetting, x36x0SensorSetting;

    if (MTD_IS_E21X0_BASE(devPtr->deviceId) || MTD_IS_X35X0_BASE(devPtr->deviceId))
    {
        /* some early firmware was not configuring it, check if it's configured */
        /* if not, configure it. if it's already configured, leave it alone */

        if (mtdGetTempSensorMDIOPort(devPtr, port, &sensorPort) == MTD_FAIL)
        {
            MTD_DBG_ERROR("mtdGetTempSensorMDIOPort() failed.\n");
            return MTD_FAIL;
        }

        /* Check if the sensor is configured for average mode over the period/samples in 3.8042 */ 
        MTD_ATTEMPT(mtdHwGetPhyRegField(devPtr,sensorPort,MTD_TUNIT_TEMP_SENSOR2,14,2,&e21x0SensorSetting));

        if (e21x0SensorSetting != 0x01)
        {
            /* firmware did not configure it, it's old firmware, so configure it */
            /* note: on older firmware, a T unit soft reset will clear the setting and it will have to be */
            /* re-enabled */
            if (mtdHwSetPhyRegField(devPtr,sensorPort,MTD_TUNIT_TEMP_SENSOR2,8,8,0x45) != MTD_OK)
            {
                MTD_DBG_ERROR("Enable Tunit temperature sensor failed.\n");
                return MTD_FAIL;
            }            
        } /* else it's already configured by firmware, leave it alone */
                    
        return MTD_OK; 
    }

    if (MTD_IS_X32X0_X33X0_BASE(devPtr->deviceId))
    {
        if (mtdGetTempSensorMDIOPort(devPtr, port, &sensorPort) == MTD_FAIL)
        {
            MTD_DBG_ERROR("mtdGetTempSensorMDIOPort() Failed.\n");
            return MTD_FAIL;
        }
       
        /* Enable Cunit temperature sensor to use 2^9 samples at a period of 1.36ms */
        if (mtdHwSetPhyRegField(devPtr,sensorPort,MTD_CUNIT_TEMP_SENSOR_CTRL_STAT,8,8,0x45) != MTD_OK)
        {
            MTD_DBG_ERROR("Enable Cunit temperature sensor failed.\n");
            return MTD_FAIL;
        }
    }
    else if (MTD_IS_X36X0_BASE(devPtr->deviceId))
    {
        if (mtdGetTempSensorMDIOPort(devPtr, port, &sensorPort) == MTD_FAIL)
        {
            MTD_DBG_ERROR("mtdGetTempSensorMDIOPort() Failed.\n");
            return MTD_FAIL;
        }

        MTD_ATTEMPT(mtdHwGetPhyRegField(devPtr, sensorPort, MTD_TUNIT_TESNE_CTL0, 0, 2, &x36x0SensorSetting));

        if (x36x0SensorSetting == 3) 
        {
            return MTD_OK;
        }
        else 
        {
            return MTD_FAIL;
        }
    }
    else
    {
        MTD_DBG_ERROR("mtdEnableTemperatureSensor: device not supported.\n");
        return MTD_FAIL;
    }

    return MTD_OK;
}

MTD_STATUS mtdReadTemperature
(
    IN MTD_DEV_PTR devPtr,
    IN MTD_U16   port,
    OUT MTD_S16 *temperature
)
{
    MTD_U16 sensorPort, temp;
    MTD_U32 temp32;
    MTD_32 stemp32, tesneCelsius = 0;

    if (mtdGetTempSensorMDIOPort(devPtr, port, &sensorPort) == MTD_FAIL)
    {
        MTD_DBG_ERROR("mtdGetTempSensorMDIOPort() Failed.\n");
        return MTD_FAIL;
    }    

    if (MTD_IS_E21X0_BASE(devPtr->deviceId) || MTD_IS_X35X0_BASE(devPtr->deviceId))
    {
        MTD_ATTEMPT(mtdHwXmdioRead(devPtr,sensorPort,MTD_TUNIT_TEMP_SENSOR2,&temp)); /* need to read the average temperature */
    }
    else if (MTD_IS_X32X0_X33X0_BASE(devPtr->deviceId))
    {
        MTD_ATTEMPT(mtdHwXmdioRead(devPtr,sensorPort,MTD_CUNIT_TEMP_SENSOR_CTRL_STAT,&temp)); /* need to read the average temperature */
    }
    else if (MTD_IS_X36X0_BASE(devPtr->deviceId)) 
    {
        MTD_ATTEMPT(mtdHwXmdioRead(devPtr, sensorPort, MTD_TUNIT_TESNE_DATA, &temp));
        
        /* convert 12 bit signed integer to 32 bit*/
        temp &= 0xFFF;
        temp32 = (temp & 0x800) ? (0xFFFFF000 | (MTD_U32)temp) : (MTD_U32)temp;
        stemp32 = (MTD_32)temp32;

        /* convert to Celsius, ref to TESNE doc but scale up 10000 to avoid float type */
        tesneCelsius = (stemp32 * 905 + 1122000) / 10000;
    }
    else
    {
        MTD_DBG_ERROR("mtdReadTemperature: Device is not supported.\n");
        return MTD_FAIL;
    }

    if (MTD_IS_X36X0_BASE(devPtr->deviceId))
    {
        *temperature = (MTD_S16)tesneCelsius;
    }
    else
    {
        *temperature = ((MTD_S16)(temp & 0xFF)) - 75;
    }

    return MTD_OK;
}

/*********************************************************************
   Some utilities that are used in the APIs for setting/getting bit fields
 *********************************************************************/


/* 
    This macro calculates the mask for partial read/write of register's data.
*/
#define MTD_CALC_MASK(fieldOffset,fieldLen,mask)        \
            if((fieldLen + fieldOffset) >= 16)      \
                mask = (0 - (1 << fieldOffset));    \
            else                                    \
                mask = (((1 << (fieldLen + fieldOffset))) - (1 << fieldOffset))

MTD_STATUS mtdHwGetPhyRegField
(
    IN  MTD_DEV_PTR devPtr,
    IN  MTD_U16      port, 
    IN  MTD_U16      dev, 
    IN  MTD_U16      regAddr,
    IN  MTD_U8       fieldOffset,
    IN  MTD_U8       fieldLength,
    OUT MTD_U16      *data
)
{
    MTD_U16 tmpData;
    MTD_STATUS   retVal;

    retVal = mtdHwXmdioRead(devPtr,port, dev, regAddr, &tmpData);

    if(retVal != MTD_OK)
    {
        MTD_DBG_ERROR("Failed to read register \n");
        return MTD_FAIL;
    }
    
    mtdHwGetRegFieldFromWord(tmpData,fieldOffset,fieldLength, data);

    /* MTD_DBG_INFO("fOff %d, fLen %d, data 0x%04X.\n",(int)fieldOffset,(int)fieldLength,(int)*data); */

    return MTD_OK;
}

MTD_STATUS mtdHwSetPhyRegField
(
    IN MTD_DEV_PTR devPtr,
    IN MTD_U16      port, 
    IN MTD_U16      dev, 
    IN MTD_U16      regAddr,
    IN MTD_U8       fieldOffset,
    IN MTD_U8       fieldLength,
    IN MTD_U16      data
)
{
    MTD_U16 tmpData,newData;
    MTD_STATUS   retVal;

    retVal = mtdHwXmdioRead(devPtr,port, dev, regAddr, &tmpData);
    if(retVal != MTD_OK)
    {
        MTD_DBG_ERROR("Failed to read register \n");
        return MTD_FAIL;
    }

    mtdHwSetRegFieldToWord(tmpData,data,fieldOffset,fieldLength,&newData);

    retVal = mtdHwXmdioWrite(devPtr,port, dev, regAddr, newData);

    if(retVal != MTD_OK)
    {
        MTD_DBG_ERROR("Failed to write register \n");
        return MTD_FAIL;
    }

    /*
    MTD_DBG_INFO("fieldOff %d, fieldLen %d, data 0x%x.\n",fieldOffset,\
                  fieldLength,data); 
    */
    return MTD_OK;
}

MTD_STATUS mtdHwGetRegFieldFromWord
(
    IN  MTD_U16      regData,
    IN  MTD_U8       fieldOffset,
    IN  MTD_U8       fieldLength,
    OUT MTD_U16      *data
)
{
    MTD_U16 mask;            /* Bits mask to be read */

    MTD_CALC_MASK(fieldOffset,fieldLength,mask);

    *data = (regData & mask) >> fieldOffset;

    return MTD_OK;
}

MTD_STATUS mtdHwSetRegFieldToWord
(
    IN  MTD_U16      regData,
    IN  MTD_U16      bitFieldData,
    IN  MTD_U8       fieldOffset,
    IN  MTD_U8       fieldLength,
    OUT MTD_U16      *data
)
{
    MTD_U16 mask;            /* Bits mask to be read */

    MTD_CALC_MASK(fieldOffset,fieldLength,mask);

    /* Set the desired bits to 0.                       */
    regData &= ~mask;
    /* Set the given data into the above reset bits.    */
    regData |= ((bitFieldData << fieldOffset) & mask);

    *data = regData;

    return MTD_OK;
}


/*********************************************************************
 Local functions
 *********************************************************************/

MTD_STATIC MTD_STATUS mtdAutonegIsSpeedDuplexResolutionDone
(
    IN MTD_DEV_PTR devPtr,
    IN MTD_U16 port,
    OUT MTD_BOOL *anSpeedResolutionDone
)
{
    MTD_U16 val;
 
    /* read speed/duplex resolution done bit in 3.8008 bit 11 */
    if (mtdHwGetPhyRegField(devPtr,port,
            3, 0x8008, 11, 1, &val) == MTD_FAIL)
    {
        *anSpeedResolutionDone = MTD_FALSE;
        return MTD_FAIL;
    }
 
    (val)?(*anSpeedResolutionDone = MTD_TRUE):(*anSpeedResolutionDone = MTD_FALSE);
  
    return MTD_OK;
}


MTD_STATIC MTD_STATUS mtdAutonegIsCompleted
(
    IN MTD_DEV_PTR devPtr,
    IN MTD_U16 port,
    OUT MTD_BOOL *anStatusReady
)
{
    MTD_U16 val;
 
    /* read an completed, 7.1.5 bit */
    if (mtdHwGetPhyRegField(devPtr,port,
            7, 1, 5, 1, &val) == MTD_FAIL)
    {
        *anStatusReady = MTD_FALSE;
        return MTD_FAIL;
    }
 
    (val)?(*anStatusReady = MTD_TRUE):(*anStatusReady = MTD_FALSE);
  
    return MTD_OK;    
}


MTD_STATIC MTD_STATUS mtdReadMasterOrSlaveBit
(
    IN MTD_DEV_PTR devPtr,
    IN MTD_U16 port,
    OUT MTD_BOOL *msBitFlag
)
{
    MTD_U16 val = MTD_FALSE, speed;
    
    *msBitFlag = MTD_FALSE;

    /* this functions assumes previously that an is "done" and no config fault occurred */

    /* check the speed resolution so we know which master/slave bit */
    /* to look at */

    MTD_ATTEMPT(mtdHwGetPhyRegField(devPtr,port, \
            3, 0x8008, 14, 2, &speed));

    if (speed == 3)
    {
        /* speed is 10GBT */
        /* read 7.33.14, this will clear the LH bit in 7.33.15 */
        MTD_ATTEMPT(mtdHwGetPhyRegField(devPtr,port,
                7, 33, 14, 1, &val));

    }
    else if (speed == 2)
    {
        /* speed is 1000BT */
        /* read 7.32769.14, this will clear the LH bit in 7.32769.15 */
        MTD_ATTEMPT(mtdHwGetPhyRegField(devPtr,port,
                7, 32769, 14, 1, &val));

    }
    else
    {
        /* this is an error (it means the speed is 10BT or 100BT and reading this */
        /* bit makes no sense) */
        return MTD_FAIL;
    }

    (val) ? (*msBitFlag = MTD_TRUE) : (*msBitFlag = MTD_FALSE);

    return MTD_OK;
}


MTD_STATIC MTD_STATUS mtdGetMasterSlaveConfigFault
(
    IN MTD_DEV_PTR devPtr,
    IN MTD_U16 port,
    OUT MTD_BOOL *msConfigFaultFlag
)
{
    MTD_U16 val = MTD_FALSE, val2 = MTD_FALSE;
    
    *msConfigFaultFlag = MTD_FALSE;

    /* there's no "done" indication config fault */
    /* the bit in 7.33.15 is set if the resolution was 10GBT */
    /* the bit in 7.32769.15 is set if the resolution was 1000BT */

    /* read 7.33.15, this will clear this LH bit */
    MTD_ATTEMPT(mtdHwGetPhyRegField(devPtr,port, \
            7, 33, 15, 1, &val));

    /* read 7.32769.15, this will clear this LH bit */
    MTD_ATTEMPT(mtdHwGetPhyRegField(devPtr,port, \
            7, 32769, 15, 1, &val2));

    val |= val2; /* either bit being set means there was a config fault */

    (val) ? (*msConfigFaultFlag = MTD_TRUE) : (*msConfigFaultFlag = MTD_FALSE);

    return MTD_OK;
}

