/* 
 Copyright (c) 2014-2024 Marvell International Ltd., All Rights Reserved
 Redistribution and use in source and binary forms, with or without modification, 
 are permitted provided that the following conditions are met:

 1. Redistributions of source code must retain the above copyright notice, this list 
    of conditions and the following disclaimer.
 2. Redistributions in binary form must reproduce the above copyright notice, this list 
    of conditions and the following disclaimer in the documentation and/or other materials 
    provided with the distribution.
    
 THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY EXPRESS 
 OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY 
 AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR 
 CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR 
 CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR 
 SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY 
 THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR 
 OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE 
 POSSIBILITY OF SUCH DAMAGE.
*/

#ifndef MTD_API_H
#define MTD_API_H

#if C_LINKAGE
#if defined __cplusplus
    extern "C" {
#endif
#endif

#define MTD_API_MAJOR_VERSION 105
#define MTD_API_MINOR_VERSION 0
#define MTD_API_BUILD_ID      1

/* To select H or X unit */
#define MTD_X_UNIT          3 /* Fiber interface (only for 88X33X0 and 88X32X0 parts) */
#define MTD_H_UNIT          4 /* Host interface */


/* These defines are used for some registers which represent the copper
   speed as a 2-bit binary number */
#define MTD_CU_SPEED_10_MBPS    0 /* copper is 10BASE-T */
#define MTD_CU_SPEED_100_MBPS   1 /* copper is 100BASE-TX */
#define MTD_CU_SPEED_1000_MBPS  2 /* copper is 1000BASE-T */
#define MTD_CU_SPEED_10_GBPS    3 /* copper is 10GBASE-T */

/* for 88X33X0/E21X0/X35X0/X36X0/E2780 family: */
#define MTD_CU_SPEED_NBT        3 /* copper is NBASE-T */
#define MTD_CU_SPEED_NBT_10G    0 /* copper is 10GBASE-T  */
#define MTD_CU_SPEED_NBT_5G     2 /* copper is 5GBASE-T */
#define MTD_CU_SPEED_NBT_2P5G   1 /* copper is 2.5GBASE-T */

/*******************************************************************
 Register definitions
 *******************************************************************/
#define MTD_T_UNIT_PMA_PMD  1
#define MTD_T_UNIT_PCS_CU   3
#define MTD_T_UNIT_AN       7
#define MTD_C_UNIT_GENERAL  31
#define MTD_M_UNIT          31

#define MTD_TUNIT_IEEE_PMA_CTRL1       MTD_T_UNIT_PMA_PMD,0x0000
#define MTD_TUNIT_IEEE_PMA_DEVID2      MTD_T_UNIT_PMA_PMD,0x0003
#define MTD_TUNIT_PHY_EXT_CTRL_1       MTD_T_UNIT_PMA_PMD,0xC000
#define MTD_TUNIT_XG_EXT_STATUS        MTD_T_UNIT_PMA_PMD,0xC001
#define MTD_TUNIT_BIST_STATUS_REG      MTD_T_UNIT_PMA_PMD,0xC00C
#define MTD_TUNIT_PHY_REV_INFO_REG     MTD_T_UNIT_PMA_PMD,0xC04E
#define MTD_BOOT_STATUS_REG            MTD_T_UNIT_PMA_PMD,0xC050

#define MTD_TUNIT_IEEE_PCS_CTRL1       MTD_T_UNIT_PCS_CU,0x0000

#define MTD_CUNIT_PORT_CTRL             MTD_C_UNIT_GENERAL,0xF001 
#define MTD_CUNIT_E21X0_PORT_CTRL       MTD_T_UNIT_PMA_PMD,0xC04A 

#define MTD_1000X_SGMII_CONTROL               0x2000
#define MTD_1000X_SGMII_STATUS                0x2001

#define MTD_10GBR_PCS_CONTROL                 0x1000
#define MTD_10GBR_PCS_STATUS1                 0x1001
#define MTD_10GBR_PCS_FAULT                   0x1008 
#define MTD_10GBR_PCS_RXSTATUS                0x1020 
#define MTD_10GBR_PCS_STATUS2                 0x1021

#define MTD_TUNIT_INTERRUPT_CTRL1      MTD_T_UNIT_PCS_CU,0x8010
#define MTD_TUNIT_INTERRUPT_CTRL2      MTD_T_UNIT_PCS_CU,0x8012
#define MTD_TUNIT_INTERRUPT_STATUS1    MTD_T_UNIT_PCS_CU,0x8011
#define MTD_TUNIT_INTERRUPT_STATUS2    MTD_T_UNIT_PCS_CU,0x8013

#define MTD_10GBR_INTERRUPT_CTRL              0x9000
#define MTD_10GBR_INTERRUPT_STATUS            0x9001
#define MTD_10GBR_REALTIME_STATUS             0x9002

#define MTD_1GBX_INTERRUPT_CTRL               0xA001
#define MTD_1GBX_INTERRUPT_STATUS             0xA002
#define MTD_1GBX_REALTIME_STATUS              0xA003

#define MTD_TUNIT_TESNE_CTL0            MTD_T_UNIT_PCS_CU,0xDB09 
#define MTD_TUNIT_TESNE_DATA            MTD_T_UNIT_PCS_CU,0xDB0C 
#define MTD_TUNIT_TEMP_SENSOR2          MTD_T_UNIT_PCS_CU,0x8042 
#define MTD_CUNIT_TEMP_SENSOR_CTRL_STAT MTD_C_UNIT_GENERAL,0xF08A

/* 88E2110/2180 LED Registers */
#define CUNIT_SLED_CONTROL        0xF03A
#define CUNIT_GPIO_CONTROL        0xF015
#define CUNIT_LED0_CONTROL        0xF020
#define CUNIT_LED1_CONTROL        0xF021
#define CUNIT_LED2_CONTROL        0xF022


/*******************************************************************
 API Version
 Resets
 Download Mode Selection/Query
 Phy Low Power Mode
 Tunit Low Power Mode
 Parallel Write Mode
 MDIO Download Mode
 *******************************************************************/

/******************************************************************************
 MTD_VOID mtdGetAPIVersion
 (
     OUT MTD_U8 *major,
     OUT MTD_U8 *minor,
     OUT MTD_U8 *buildID
 );


 Inputs:
    None

 Outputs:
    major - major version number
    minor - minor version number
    buildID - if non-released software, this number will be non-zero
              indicating the interim version between releases

 Returns:
    None
    
 Description:
    Returns the version number of this API. The non-zero build ID 
    is used for test and preview releases.  General release build ID is 0.

 Side effects:
    None

 Notes/Warnings:
    None

******************************************************************************/
MTD_VOID mtdGetAPIVersion
(
    OUT MTD_U8 *major,
    OUT MTD_U8 *minor,
    OUT MTD_U8 *buildID
);

/******************************************************************************
MTD_STATUS mtdSoftwareReset
(
    IN MTD_DEV_PTR devPtr,
    IN MTD_U16 port,
    IN MTD_U16 timeoutMs    
);

 Inputs:
    devPtr - pointer to MTD_DEV 
    port - MDIO port address, 0-31
    timeoutMs - 0 will not wait for reset to complete, otherwise
                waits 'timeout' milliseconds for reset to complete

 Outputs:
    None

 Returns:
    MTD_OK or MTD_FAIL if IO error or timed out

 Description:
    Issues a software reset (1.0.15 <= 1) command. Resets firmware and
    hardware state machines and returns non-retain bits to their hardware
    reset values and retain bits keep their values through the reset.

    If timeoutMs is 0, returns immediately. If timeoutMs is non-zero, 
    waits up to 'timeoutMs' milliseconds looking for the reset to complete
    before returning. Returns MTD_FAIL if times out.

 Side effects:
    All "retain" bits keep their values through this reset. Non-"retain"-type
    bits are returned to their hardware reset values following this reset.
    See the Datasheet for a list of retain bits.

 Notes/Warnings:
    Use mtdIsPhyReadyAfterReset() to see if the software reset is complete
    before issuing any other MDIO commands following this reset or pass
    in non-zero timeoutMs to have this function do it for you.

    This is a T unit software reset only. It may only be issued if the T
    unit is ready (1.0.15 is 0).

******************************************************************************/
MTD_STATUS mtdSoftwareReset
(
    IN MTD_DEV_PTR devPtr,
    IN MTD_U16 port,
    IN MTD_U16 timeoutMs
);

/******************************************************************************
 MTD_STATUS mtdChipHardwareReset
 (
     IN MTD_DEV_PTR devPtr,
     IN MTD_U16 port
 )

 Inputs:
    devPtr - pointer to MTD_DEV of the type of device being accessed
    port - this can be any port on the chip, entire chip is reset though

 Outputs:
    None

 Returns:
    MTD_OK or MTD_FAIL

 Description:
    Performs a chip-level hardware reset, 31.F001.14 <= 1. This should reset
    all state machines and restore all registers to their power-on state.

    All ports are reset. This is equivalent to a chip level hardware reset.

 Side effects:
    See below.

 Notes/Warnings:
    Any established data links will drop. The chip will return to the same
    state as the power-on state and must be reinitialized.

    For X3580, will issue 2 x chip resets, one to the lower half and one to 
    the upper half, resulting in all 8 ports reset. This reset can take
    up to 450ms to complete. Both upper and lower ports need to be polled for
    1.0.15 is 0 to see firmware loading is done and device is completely out
    of reset. This function has a large delay embedded in it between the
    upper and lower half resets. It can take up to 235 ms before returning
    for X3580 chip reset.
    
******************************************************************************/
MTD_STATUS mtdChipHardwareReset
(
    IN MTD_DEV_PTR devPtr,
    IN MTD_U16 port
);


/******************************************************************************
MTD_STATUS mtdIsPhyInMdioDownloadMode
(
    IN MTD_DEV_PTR devPtr,
    IN MTD_U16 port,
    OUT MTD_BOOL *downloadMode
);

 Inputs:
    devPtr - pointer to MTD_DEV 
    port - MDIO port address, 0-31

 Outputs:
    downloadMode - MTD_TRUE   if it is waiting for download via MDIO or FR2R
                   MTD_FALSE  if it is not waiting for download.

 Returns:
    MTD_OK or MTD_FAIL if function failed to read status


 Description:
    Reads certain bits of the boot status register and returns 
    MTD_TRUE if waiting for firmware download and MTD_FALSE if not.

 Side effects:
    None.

 Notes/Warnings:
    None.

******************************************************************************/
MTD_STATUS mtdIsPhyInMdioDownloadMode
(
    IN MTD_DEV_PTR devPtr,
    IN MTD_U16 port,
    OUT MTD_BOOL *downloadMode
);
 
/******************************************************************************
MTD_STATUS mtdDidPhyAppCodeStart
(
    IN MTD_DEV_PTR devPtr,
    IN MTD_U16 port,
    OUT MTD_BOOL *appStarted
);
                                    

 Inputs:
    devPtr - pointer to MTD_DEV 
    port - MDIO port address, 0-31

 Outputs:
    appStarted - MTD_TRUE if it started, MTD_FALSE if it did not

 Returns:
    MTD_OK if query was successful, MTD_FAIL if not

 Description:
    Reads certain bits of the boot status register and returns the above values.

 Side effects:
    None.

 Notes/Warnings:
    None.

******************************************************************************/
MTD_STATUS mtdDidPhyAppCodeStart
(
    IN MTD_DEV_PTR devPtr,
    IN MTD_U16 port,
    OUT MTD_BOOL *appStarted
);

/******************************************************************************
MTD_STATUS mtdIsTunitResponsive
(
    IN MTD_DEV_PTR devPtr,
    IN MTD_U16 port,
    OUT MTD_BOOL *tunitReady
)              

 Inputs:
    devPtr - pointer to MTD_DEV initialized by mtdLoadDriver() call
    port - MDIO port address, 0-31

 Outputs:
    tunitReady - MTD_TRUE if T-unit firmware is ready to accept commands
               MTD_FALSE if T-unit is not ready 

 Returns:
    MTD_OK if query was successful, MTD_FAIL if not

 Description:
    This function reads several registers to check if the T-unit firmware 
    is in a state to respond to MDIO write commands.

    If tunitReady is MTD_FALSE, the T-unit is not able to process 
    commands. If tunitReady is MTD_TRUE, the T-unit should be able to
    process commands.

    Suggest calling this function after any reset, or removing low power
    mode to make sure the T unit is ready before making changes to any
    configuration in the PHY.

 Side effects:

 Notes/Warnings:
    This functions checks for several conditions that could cause the T-unit
    (copper unit)to not be able to respond to commands. These conditions 
    are as follows:

    1. A reset is still in process (PHY cannot be configured during resets)
    2. Fatal BIST error during a reset
    3. T-unit in download mode or low-power mode/powered down
    4. Repeated fatal boot errors (will show as still in reset state)
    5. T-unit placed in a test mode

******************************************************************************/
MTD_STATUS mtdIsTunitResponsive
(
    IN MTD_DEV_PTR devPtr,
    IN MTD_U16 port,
    OUT MTD_BOOL *tunitReady
);  

/******************************************************************************
MTD_STATUS mtdIsPhyReadyAfterReset
(
    IN MTD_DEV_PTR devPtr,
    IN MTD_U16 port,
    OUT MTD_BOOL *phyReady
)              

 Inputs:
    devPtr - pointer to MTD_DEV 
    port - MDIO port address, 0-31

 Outputs:
    phyReady - MTD_TRUE if reset has completed (firmware is up and running)
               MTD_FALSE if reset is in progress or PHY is in download mode

 Returns:
    MTD_OK if query was successful, MTD_FAIL if not

 Description:
    This function reads the register x.1.0.15 and checks if the phy application 
    code has loaded and started. It returns a value MTD_TRUE meaning reset is 
    completed and code started and a MTD_FALSE if reset is in progress or if the 
    T-unit is in download mode (no firmware loaded yet and/or running yet).

    MDIO commands (other than download-related and hardware reset commands) 
    should not be sent before this function returns MTD_TRUE.

    MTD_TRUE indicates the T-unit has completed its reset.

 Side effects:

 Notes/Warnings:
    To query if reset has completed and T-unit is ready to accept
    MDIO commands (app code started successfully and reset is complete).

    Should complete within 500ms following a special software reset, power-on
    reset or hardware reset. Time may be longer on octal phys.

    If Phy is in download mode (pin SPI_CONFIG[1] is high or override is 1), 
    this function will always return MTD_FALSE.

******************************************************************************/
MTD_STATUS mtdIsPhyReadyAfterReset
(
    IN MTD_DEV_PTR devPtr,
    IN MTD_U16 port,
    OUT MTD_BOOL *phyReady
);

/******************************************************************************
MTD_STATUS mtdPutPhyInLowPowerMode
(
    IN MTD_DEV_PTR devPtr,
    IN MTD_U16 port
);

 Inputs:
    devPtr - pointer to MTD_DEV 
    port - MDIO port address, 0-31

 Outputs:
    None

 Returns:
    MTD_OK if action was successfully taken, MTD_FAIL if not

 Description:
    This function puts the T-unit and the XFI portion of the
    H and X-unit, and the 1000BX/SGMII interface of the
    X-unit into a powered down mode.

 Side effects:

 Notes/Warnings:
    The PHY can be removed from low power mode by either calling
    mtdRemovePhyLowPowerMode() or by doing a hardware reset on the PHY (a
    software reset will not remove the low power mode, because the low power
    bits are retain bits), assuming the strap PDSTATE is strapped
    to have the PHY not come up in low power mode. If PDSTATE is strapped
    to have the PHY come up in low power mode, then the only way to restore
    power is to call mtdRemovePhyLowPowerMode().

******************************************************************************/
MTD_STATUS mtdPutPhyInLowPowerMode
(
    IN MTD_DEV_PTR devPtr,
    IN MTD_U16 port
);

/******************************************************************************
MTD_STATUS mtdRemovePhyLowPowerMode
(
    IN MTD_DEV_PTR devPtr,
    IN MTD_U16 port
)


 Inputs:
    devPtr - pointer to MTD_DEV 
    port - MDIO port address, 0-31

 Outputs:
    None

 Returns:
    MTD_OK if action was successfully taken, MTD_FAIL if not

 Description:
    This function removes the T-unit and the XFI portion of the
    H and X-unit, and the 1000BX/SGMII interface of the
    X-unit from a powered down mode.


 Side effects:
    H/X-Unit SGMII and T-unit AN will start (if they are enabled).

 Notes/Warnings:
    The PHY can be removed from low power mode by either calling
    mtdRemovePhyLowPowerMode() or by doing a hardware reset on the PHY (a
    software reset will not remove the low power mode, because the low power
    bits are retain bits), assuming the strap PDSTATE is strapped
    to have the PHY not come up in low power mode. If PDSTATE is strapped
    to have the PHY come up in low power mode, then the only way to restore
    power is to call mtdRemovePhyLowPowerMode().

    If the PHY was strapped with PDSTATE to have the PHY come up in low power
    mode, mtdRemovePhyLowPowerMode() must be called before the PHY can
    complete its initialization and be configured.

    After removing the PHY from low power mode, make sure the T unit is ready
    by calling mtdIsTunitResponsive() before writing any T unit registers.

******************************************************************************/
MTD_STATUS mtdRemovePhyLowPowerMode
(
    IN MTD_DEV_PTR devPtr,
    IN MTD_U16 port
);


/******************************************************************************
MTD_STATUS mtdPutTunitInLowPowerMode
(
    IN MTD_DEV_PTR devPtr,
    IN MTD_U16 port
);

 Inputs:
    devPtr - pointer to MTD_DEV 
    port - MDIO port address, 0-31

 Outputs:
    None

 Returns:
    MTD_OK if action was successfully taken, MTD_FAIL if not

 Description:
    This function uses a firmware register bit to put the T-unit in low power 
    mode by writing a '1' to 1.0.11.

 Side effects:

 Notes/Warnings:
    The T-unit can be removed from low power mode by either calling
    mtdRemoveTunitPhyLowPowerMode() or by doing a hardware reset on the PHY (a
    software reset will not remove the low power mode, because the low power
    bits are retain bits). The hardware reset behavior depends on the
    PDSTATE strap pin ultimately. This note assumes the PDSTATE strap
    is set to not come up in power-down. A hardware reset may also
    remove other units from a power-down state depending on the PDSTATE
    as well. See the datasheet for details.

    Calling this function will not remove the T-unit from low power mode if
    the T-unit is powered down because the PHY has been put in fiber-only
    mode or if the T-unit is powered down because of the auto-media detect.

******************************************************************************/
MTD_STATUS mtdPutTunitInLowPowerMode
(
    IN MTD_DEV_PTR devPtr,
    IN MTD_U16 port
);

/******************************************************************************
MTD_STATUS mtdRemoveTunitLowPowerMode
(
    IN MTD_DEV_PTR devPtr,
    IN MTD_U16 port
)

 Inputs:
    devPtr - pointer to MTD_DEV 
    port - MDIO port address, 0-31

 Outputs:
    None

 Returns:
    MTD_OK if action was successfully taken, MTD_FAIL if not

 Description:
    This function sets both 1.0.11 and 3.0.11 to 0, which should remove
    the low power mode for the T-unit. Restarts AN (if it is enabled).


 Side effects:
    AN will start (if it is enabled).

 Notes/Warnings:
    The T-unit can be removed from low power mode by either calling
    mtdRemoveTunitLowPowerMode() or by doing a hardware reset on the PHY (a
    software reset will not remove the low power mode, because the low power
    bits are retain bits). The hardware reset behavior depends on the
    PDSTATE strap pin ultimately. This note assumes the PDSTATE strap
    is set to not come up in power-down. A hardware reset may also
    remove other units from a power-down state depending on the PDSTATE
    as well. See the datasheet for details.

    After removing the T unit from low power mode, make sure the T unit is ready
    by calling mtdIsTunitResponsive() before writing any T unit registers.    

******************************************************************************/
MTD_STATUS mtdRemoveTunitLowPowerMode
(
    IN MTD_DEV_PTR devPtr,
    IN MTD_U16 port
);

/******************************************************************************
MTD_STATUS mtdIsTunitInLowPowerMode
(
    IN MTD_DEV_PTR devPtr,
    IN MTD_U16 port,
    OUT MTD_BOOL *inLowPowerMode
)

 Inputs:
    devPtr - pointer to MTD_DEV 
    port - MDIO port address, 0-31

 Outputs:
    inLowPowerMode - MTD_TRUE if T-unit is in low power mode, MTD_FALSE if not

 Returns:
    MTD_OK if action was successfully taken, MTD_FAIL if not

 Description:
    This function checks 1.0.11 and 3.0.11, and also checks BIST status and
    internal T-unit state/mode variable to see if T-unit is powered down.

    When T-unit is powered-down, it will not respond to commands.

    T-unit can be powered-down explicitly through 1.0.11 or 3.0.11, or it
    can be powered-down conditionally based on the PHY operating mode based
    on settings in 31.F000 and 31.F001 (for example, in fiber-only mode, the
    T-unit is powered down).

 Side effects:
    None

 Notes/Warnings:
    If the T-unit was powered down explicitly it can be powered-up by
    calling mtdRemoveTunitLowPowerMode(). If it was powered-down indirectly
    by PHY settings in 31.F000 and 31.F001,  it will stay powered down
    until those conditions change. For example, if the PHY was put in
    fiber-only mode, the T-unit is powered down until the PHY is removed
    from fiber-only mode.

******************************************************************************/
MTD_STATUS mtdIsTunitInLowPowerMode
(
    IN MTD_DEV_PTR devPtr,
    IN MTD_U16 port,
    OUT MTD_BOOL *inLowPowerMode
);


/******************************************************************************
MTD_STATUS mtdPutPhyInParallelWriteMode
(
    IN MTD_DEV_PTR devPtr,
    IN MTD_U16 port
);

 Inputs:
    devPtr - pointer to MTD_DEV initialized by mtdLoadDriver() call
    port - MDIO port address, 0-31

 Outputs:
    None

 Returns:
    MTD_OK or MTD_FAIL

 Description:
    Sets a bit on the given port to put the PHY into parallel write mode. 

 Side effects:
    Setting this bit will causes the PHY to ignore the MDIO port address on 
    writes.


 Notes/Warnings:
    1. Reset is not needed after setting this pin to 1.
    2. The read will not work with this bit set. Therefore to read, this bit has 
    to be set to '0' and read has to be done one port at a time. The function 
    mtdRemovePhyFromParallelWriteMode() can be used to set bit back to 0.
******************************************************************************/
MTD_STATUS mtdPutPhyInParallelWriteMode
(
    IN MTD_DEV_PTR devPtr,
    IN MTD_U16 port
);

/******************************************************************************
MTD_STATUS mtdRemovePhyFromParallelWriteMode
(
    IN MTD_DEV_PTR devPtr,
    IN MTD_U16 port
);

 Inputs:
    devPtr - pointer to MTD_DEV initialized by mtdLoadDriver() call
    port - MDIO port address, 0-31

 Outputs:
    None

 Returns:
    MTD_OK or MTD_FAIL

 Description:
    Sets 31.D101.14 to 0 (off) to take PHY out of parallel write mode. 

 Side effects:
    None.
    
 Notes/Warnings:
    1. Reset is not needed after setting this bit to 0.
    2. For normal MDIO operation the bit must be 0.
    3. MDIO read on all ports will not work until this bit is set back to 0 on
       all ports on the MDIO bus.
******************************************************************************/
MTD_STATUS mtdRemovePhyFromParallelWriteMode
(
    IN MTD_DEV_PTR devPtr,
    IN MTD_U16 port
);



/******************************************************************************
MTD_STATUS mtdPutPhyInMdioDownloadMode
(
    IN MTD_DEV_PTR devPtr,
    IN MTD_U16 port
);

 Inputs:
    devPtr - pointer to MTD_DEV 
    port - MDIO port address, 0-31

 Outputs:
    None

 Returns:
    MTD_OK or MTD_FAIL

 Description:
    Sets override of SPI_CONFIG to 1, then resets the T-unit so boot code will
    wait for download of firmawre via MDIO. This function is needed to be 
    called if the device is strapped for flash loading to allow application
    code to be downloaded to the RAM instead.

    If the device is strapped for MDIO loading, no need to call this. The
    hardware reset will place the ports in download mode.

 Side effects:
    None

 Notes/Warnings:
    On X3680/E2780 devices, port pairs share the same code RAM. On these devices
    it's not expected to download to one port and not the other. This
    function, however, controls them individually. To put both ports into
    download mode, set the strap override on the next (odd) port and
    call this function to finish putting both ports into the download mode.

    Both port pairs must be put into download mode to download these devices.

******************************************************************************/
MTD_STATUS mtdPutPhyInMdioDownloadMode
(
    IN MTD_DEV_PTR devPtr,
    IN MTD_U16 port
);

/******************************************************************************
MTD_STATUS mtdRemovePhyMdioDownloadMode
(
    IN MTD_DEV_PTR devPtr,
    IN MTD_U16 port
);

 Inputs:
    devPtr - pointer to MTD_DEV 
    port - MDIO port address, 0-31

 Outputs:
    None

 Returns:
    MTD_OK or MTD_FAIL

 Description:
    Sets override of SPI_CONFIG to 0, then resets the T-unit so code will
    load from flash. (Assuming device is strapped for flash loading.)

    If device is strapped for flash loading, resetting the device will cause
    it to load from flash.

 Side effects:
    None

 Notes/Warnings:
    This function does a T unit hardware reset which will cause the
    firmware to be reloaded from flash (if the strap is set to cause this
    behavior). Reloading from flash can take several hundred milliseconds
    depending on the number of ports.

    To check if firmware is loaded, check all ports using mtdDidPhyAppCodeStart()
    before continuing any T unit configuration (such as removing low power
    mode).

    On X3680/E2780 devices, port pairs share the same code RAM. On these devices
    it's not expected to download to one port and not the other. This
    function, however, controls them individually. To remove both ports from
    download mode, set the strap override on the next (odd) port and
    call this function to finish removing both ports from the download mode. 

    Both port pairs must have the download mode removed for flash loading
    to occur.

******************************************************************************/
MTD_STATUS mtdRemovePhyMdioDownloadMode
(
    IN MTD_DEV_PTR devPtr,
    IN MTD_U16 port
);

/******************************************************************************
MTD_STATUS mtdGetFirmwareVersion
(
    IN MTD_DEV_PTR devPtr,
    IN MTD_U16 port, 
    OUT MTD_U8 *major, 
    OUT MTD_U8 *minor, 
    OUT MTD_U8 *inc, 
    OUT MTD_U8 *test
);

 Inputs:
    devPtr - pointer to MTD_DEV 
    port - MDIO port address, 0-31

 Outputs:
    major - major version, X.Y.Z.W, the X
    minor - minor version, X.Y.Z.W, the Y
    inc   - incremental version, X.Y.Z.W, the Z
    test  - test version, X.Y.Z.W, the W, should be 0 for released code, 
            non-zero indicates this is a non-released code

 Returns:
    MTD_FAIL if version can't be queried or phy is in download mode 
    (meaning all version numbers are 0), MTD_OK otherwise

 Description:
    This function reads the firmware version number and stores it in the
    pointers passed in by the user.

 Side effects:
    None

 Notes/Warnings:
    This function returns all 0's if the phy is in download mode. The phy 
    application code must have started and be ready before issuing this 
    command.

******************************************************************************/
MTD_STATUS mtdGetFirmwareVersion
(
    IN MTD_DEV_PTR devPtr,
    IN MTD_U16 port, 
    OUT MTD_U8 *major, 
    OUT MTD_U8 *minor, 
    OUT MTD_U8 *inc, 
    OUT MTD_U8 *test
);



/*******************************************************************
   802.3 Clause 28 and Clause 45
   Auto-negotiation Related Control & Status
 *******************************************************************/

/*******************************************************************
 Enabling speeds for autonegotiation
 Reading speeds enabled for autonegotiation
 Set/get pause advertisement for autonegotiation
 Other Autoneg-related Control and Status (restart,disable/enable,
 force master/slave/auto, checking for autoneg resolution, etc.)
 *******************************************************************/
#define MTD_ADV_NONE           0x0000 /* No speeds to be advertised */
#define MTD_SPEED_10M_HD       0x0001 /* 10BT half-duplex */
#define MTD_SPEED_10M_FD       0x0002 /* 10BT full-duplex */
#define MTD_SPEED_100M_HD      0x0004 /* 100BASE-TX half-duplex */
#define MTD_SPEED_100M_FD      0x0008 /* 100BASE-TX full-duplex */
#define MTD_SPEED_1GIG_HD      0x0010 /* 1000BASE-T half-duplex */
#define MTD_SPEED_1GIG_FD      0x0020 /* 1000BASE-T full-duplex */
#define MTD_SPEED_10GIG_FD     0x0040 /* 10GBASE-T full-duplex */
#define MTD_SPEED_2P5GIG_FD    0x0800 /* 2.5GBASE-T full-duplex */
#define MTD_SPEED_5GIG_FD      0x1000 /* 5GBASE-T full-duplex */

/* Will query the device type and pick all copper speeds available on this device */
#define MTD_ALL_SPEEDS_AVAILABLE 0xFFFF /* Pass this to dynamically pick one of the below defines to select all 
                                           speeds available on this PHY device */

/* All speeds available on X32X0 devices, left for backward compatibility */
#define MTD_SPEED_ALL          (MTD_SPEED_10M_HD | \
                                MTD_SPEED_10M_FD | \
                                MTD_SPEED_100M_HD | \
                                MTD_SPEED_100M_FD | \
                                MTD_SPEED_1GIG_HD | \
                                MTD_SPEED_1GIG_FD | \
                                MTD_SPEED_10GIG_FD)

/* All speeds available on X32X0 devices */
#define MTD_SPEED_ALL_32X0 MTD_SPEED_ALL

/* All speeds available on X33X0 and X35X0 devices */
#define MTD_SPEED_ALL_33X0_35X0 (MTD_SPEED_10M_HD | \
                                 MTD_SPEED_10M_FD | \
                                 MTD_SPEED_100M_HD | \
                                 MTD_SPEED_100M_FD | \
                                 MTD_SPEED_1GIG_HD | \
                                 MTD_SPEED_1GIG_FD | \
                                 MTD_SPEED_10GIG_FD | \
                                 MTD_SPEED_2P5GIG_FD |\
                                 MTD_SPEED_5GIG_FD)
                                 
/* All speeds available on X36X0 devices */
#define MTD_SPEED_ALL_36X0 (MTD_SPEED_ALL_33X0_35X0)

/* All speeds available on E20X0, E21X0 and E25X0 devices */
#define MTD_SPEED_ALL_2XX0 (MTD_SPEED_10M_HD | \
                            MTD_SPEED_10M_FD | \
                            MTD_SPEED_100M_HD | \
                            MTD_SPEED_100M_FD | \
                            MTD_SPEED_1GIG_HD | \
                            MTD_SPEED_1GIG_FD | \
                            MTD_SPEED_2P5GIG_FD |\
                            MTD_SPEED_5GIG_FD)


/* left for backward compatibility */
#define MTD_SPEED_ALL_21X0_20X0 MTD_SPEED_ALL_2XX0

/* All speeds available on E21X1 devices */
#define MTD_SPEED_ALL_21X1 (MTD_SPEED_10M_HD | \
                            MTD_SPEED_10M_FD | \
                            MTD_SPEED_100M_HD | \
                            MTD_SPEED_100M_FD | \
                            MTD_SPEED_1GIG_HD | \
                            MTD_SPEED_1GIG_FD | \
                            MTD_SPEED_2P5GIG_FD)

/* All speeds available on E2780 device */
#define MTD_SPEED_ALL_E2780 (MTD_SPEED_ALL_21X1)

/* these bits are for forcing the speed and disabling autonegotiation */
#define MTD_SPEED_10M_HD_AN_DIS  0x0080 /* Speed forced to 10BT half-duplex */
#define MTD_SPEED_10M_FD_AN_DIS  0x0100 /* Speed forced to 10BT full-duplex */
#define MTD_SPEED_100M_HD_AN_DIS 0x0200 /* Speed forced to 100BT half-duplex */
#define MTD_SPEED_100M_FD_AN_DIS 0x0400 /* Speed forced to 100BT full-duplex */

/* this value is returned for the speed when the link status is checked and the speed has been */
/* forced to one speed but the link is up at a different speed. it indicates an error. */
#define MTD_SPEED_MISMATCH       0x8000 /* Speed is forced to one speed, but status indicates another */

/******************************************************************************
 MTD_STATUS mtdGetSpeedsAvailable
 (
     IN MTD_DEV_PTR devPtr,
     IN MTD_U16 port,
     OUT MTD_U16 *speed_bits,     
 );

 Inputs:
    devPtr - pointer to MTD_DEV 
    port - MDIO port address, 0-31

 Outputs:
    speed_bits - depending on the device id, returns one of the following:
        MTD_ADV_NONE (if fails)
        MTD_SPEED_ALL_2XX0
        MTD_SPEED_ALL_32X0
        MTD_SPEED_ALL_33X0_35X0
        MTD_SPEED_ALL_21X1
        MTD_SPEED_ALL_2XX0
        MTD_SPEED_ALL_36X0
        MTD_SPEED_ALL_E2780

 Returns:
    MTD_OK or MTD_FAIL, if query succeeded or failed

 Description:
    This function returns a bit mask of the copper speeds available on
    the T unit based on the device id.

 Side effects:

 Notes/Warnings:
    
******************************************************************************/
MTD_STATUS mtdGetSpeedsAvailable
(
    IN MTD_DEV_PTR devPtr,
    IN MTD_U16 port,
    OUT MTD_U16 *speed_bits     
);

/******************************************************************************
 MTD_STATUS mtdEnableSpeeds
 (
     IN MTD_DEV_PTR devPtr,
     IN MTD_U16 port,
     IN MTD_U16 speed_bits, 
     IN MTD_BOOL anRestart
 );

 Inputs:
    devPtr - pointer to MTD_DEV 
    port - MDIO port address, 0-31
    speed_bits - speeds to be advertised during auto-negotiation. One or more 
                 of the following (bits logically OR together):
                MTD_ADV_NONE (no bits set, will disable all speeds)

                Individual speed bits (can be logically ORed together)
                MTD_SPEED_10M_HD
                MTD_SPEED_10M_FD
                MTD_SPEED_100M_HD
                MTD_SPEED_100M_FD
                MTD_SPEED_1GIG_HD
                MTD_SPEED_1GIG_FD
                MTD_SPEED_10GIG_FD
                MTD_SPEED_2P5GIG_FD
                MTD_SPEED_5GIG_FD

                Groups of bits to get speeds for a particular device type:
                MTD_SPEED_ALL (10G and below, excludes 2.5G/5G)
                MTD_SPEED_ALL_32X0 (10G and below, excludes 2.5G/5G)                
                MTD_SPEED_ALL_33X0_35X0 (10G and below includes 2.5G/5G)
                MTD_SPEED_ALL_36X0 (10G and below includes 2.5G/5G)
                MTD_SPEED_ALL_2XX0 (5G and below)
                MTD_SPEED_ALL_21X0_20X0 (5G and below)
                MTD_SPEED_ALL_21X1 (2.5G and below)
                MTD_SPEED_ALL_E2780 (2.5G and below)

                This option will get the speeds available on a device and advertise all of them:
                MTD_ALL_SPEEDS_AVAILABLE (dynamically picks all speeds available based on PHY type)

    anRestart - this takes the value of MTD_TRUE or MTD_FALSE and indicates 
                if auto-negotiation should be restarted following the speed 
                enable change. If this is MTD_FALSE, the change will not
                take effect until AN is restarted in some other way (link
                drop, toggle low power, toggle AN enable, toggle soft reset).

                If this is MTD_TRUE and AN has been disabled, it will be
                enabled before being restarted.

 Outputs:
    None

 Returns:
    MTD_OK if action was successfully taken, MTD_FAIL if not. Also returns
    MTD_FAIL if try to force the speed or try to advertise a speed not supported
    on this PHY.

 Description:
    This function allows the user to select the speeds to be advertised to the 
    link partner during auto-negotiation.

    First, this function enables auto-negotiation and XNPs by calling 
    mtdUndoForcedSpeed(). 

    The function takes in a 16 bit value and sets the appropriate bits in MMD
    7 to have those speeds advertised.

    The function also checks if the input parameter is MTD_ADV_NONE, in which case 
    all speeds are disabled effectively disabling the phy from training 
    (but not disabling auto-negotiation).

    If anRestart is MTD_TRUE, an auto-negotiation restart is issued making the change 
    immediate. If anRestart is MTD_FALSE, the change will not take effect until the 
    next time auto-negotiation restarts.


 Side effects:
    Setting speed in 1.0 to 10GBASE-T has the effect of enabling XNPs in 7.0 and
    enabling auto-negotiation in 7.0.

 Notes/Warnings:

    Example:
    To train the highest speed matching the far end among
    either 1000BASE-T Full-duplex or 10GBASE-T:
    mtdEnableSpeeds(devPtr,port,MTD_SPEED_1GIG_FD | MTD_SPEED_10GIG_FD, MTD_TRUE);

    To allow only 10GBASE-T to train:
    mtdEnableSpeeds(devPtr,port,MTD_SPEED_10GIG_FD, MTD_TRUE);

    To disable all speeds (but AN will still be running, just advertising no
    speeds)
    mtdEnableSpeeds(devPtr,port,MTD_ADV_NONE, MTD_TRUE);

    This function is not to be used to disable autonegotiation and force the speed
    to 10BASE-T or 100BASE-TX. Use mtdForceSpeed() for this.

******************************************************************************/
MTD_STATUS mtdEnableSpeeds
(
    IN MTD_DEV_PTR devPtr,
    IN MTD_U16 port,
    IN MTD_U16 speed_bits, 
    IN MTD_BOOL anRestart
);

/******************************************************************************
 MTD_STATUS mtdEnableAllSpeedsRestartAn
 (
     IN MTD_DEV_PTR devPtr,
     IN MTD_U16 port     
 );


 Inputs:
    devPtr - pointer to MTD_DEV 
    port - MDIO port address, 0-31

 Outputs:
    None

 Returns:
    MTD_OK or MTD_FAIL, if query succeeded or failed

 Description:
    This function enables all the speeds available on this type
    of PHY based on the device id and restarts AN.

 Side effects:

 Notes/Warnings:
    Equivalent to
    
    MTD_U16 speeds;

    MTD_ATTEMPT(mtdGetSpeedsAvailable(devPtr,port,&speeds));
    MTD_ATTEMPT(mtdEnableSpeeds(devPtr,port,speeds,MTD_TRUE));

******************************************************************************/
MTD_STATUS mtdEnableAllSpeedsRestartAn
(
    IN MTD_DEV_PTR devPtr,
    IN MTD_U16 port     
);

/******************************************************************************
 MTD_STATUS mtdGetSpeedsEnabled
 (
     IN MTD_DEV_PTR devPtr,
     IN MTD_U16 port,
     OUT MTD_U16 *speed_bits,     
 );

 Inputs:
    devPtr - pointer to MTD_DEV 
    port - MDIO port address, 0-31

 Outputs:
    speed_bits - bits set according to which speeds are currently being
        advertised. One or more bits set according to the defines above
        in mtdEnableSpeeds().

 Returns:
    MTD_OK or MTD_FAIL, if query succeeded or failed

 Description:
    This function returns the speeds that have been enabled.

 Side effects:

 Notes/Warnings:
    Speeds enabled don't take effect until auto-negotiation is restarted.

******************************************************************************/
MTD_STATUS mtdGetSpeedsEnabled
(
    IN MTD_DEV_PTR devPtr,
    IN MTD_U16 port,
    OUT MTD_U16 *speed_bits     
);

/******************************************************************************
 MTD_STATUS mtdGetSpeedsInCommon
 (
     IN MTD_DEV_PTR devPtr0,
     IN MTD_U16 port0, 
     IN MTD_DEV_PTR devPtr1,
     IN MTD_U16 port1,
     OUT MTD_U16 *commonSpeeds,
     OUT MTD_U16 *highestBit,
     OUT MTD_U16 *lowestBit,
     OUT MTD_U16 *highestSpeed
 )

 Inputs:
    devPtr0 - pointer of the first port/device to MTD_DEV
    port0 - MDIO port address 0, 0-31
    devPtr1 - pointer of the second port/device to MTD_DEV
    port1 - MDIO port address 1, 0-31

 Outputs:
    commonSpeeds - bit mask based on speed definition of mtdEnableSpeeds() of speeds
                   both ends support
    highestBit - this is the speed in the highest bit position in common,
                 not the greatest bps
    lowestBit - this is the speed in the lowest bit position in common,
                not the lowest bps
    highestSpeed - this is the highest speed in common in terms of bps

 Returns:
    MTD_OK or MTD_FAIL, if query succeeded or failed

 Description:
    This function take device pointer and port for near end and far end port/device.
    It returns the common speeds bits mask among near end and far end port.
    The highest common speed is returned separately with the common speeds.
    It also returns the highest/lowest set bit in the common speeds.

 Side effects:
   None.

 Notes/Warnings:
    None.

******************************************************************************/
MTD_STATUS mtdGetSpeedsInCommon
(
    IN MTD_DEV_PTR devPtr0,
    IN MTD_U16 port0, 
    IN MTD_DEV_PTR devPtr1,
    IN MTD_U16 port1,
    OUT MTD_U16 *commonSpeeds,
    OUT MTD_U16 *highestBit,
    OUT MTD_U16 *lowestBit,
    OUT MTD_U16 *highestSpeed
);

/*****************************************************************************
MTD_STATUS mtdForceSpeed
(
    IN MTD_DEV_PTR devPtr,
    IN MTD_U16 port,
    IN MTD_U16 forcedSpeedBit 
);

 Inputs:
    devPtr - pointer to MTD_DEV 
    port - MDIO port address, 0-31
    forcedSpeedBit - one of the following
        MTD_SPEED_10M_HD_AN_DIS  - speed forced to 10BT half-duplex
        MTD_SPEED_10M_FD_AN_DIS  - speed forced to 10BT full-duplex
        MTD_SPEED_100M_HD_AN_DIS - speed forced to 100BT half-duplex
        MTD_SPEED_100M_FD_AN_DIS - speed forced to 100BT full-duplex
 
 Outputs:
    None

 Returns:
    MTD_OK if the change was successful, or MTD_FAIL if not
    
 Description:
    Disables autonegotiation, and forces the T-unit to one of the speeds above. 
    Will drop the link and attempt to train the link at the forced speed.

    Modifies the speed bits in 1.0 to one of the speeds above.

    Does a software reset of the T unit and wait until it is complete before 
    returning.

 Notes/Warnings:
    None.

******************************************************************************/
MTD_STATUS mtdForceSpeed
(
    IN MTD_DEV_PTR devPtr,
    IN MTD_U16 port,
    IN MTD_U16 forcedSpeedBit 
);

/*****************************************************************************
MTD_STATUS mtdUndoForcedSpeed
(
    IN MTD_DEV_PTR devPtr,
    IN MTD_U16 port,
    IN MTD_BOOL anRestart
);

 Inputs:
    devPtr - pointer to MTD_DEV 
    port - MDIO port address, 0-31
    anRestart - this takes the value of MTD_TRUE or MTD_FALSE and indicates 
                if auto-negotiation should be restarted following the speed 
                enable change. If this is MTD_FALSE, the change will not
                take effect until AN is restarted in some other way (link
                drop, toggle low power, toggle AN enable, toggle soft reset).

                If this is MTD_TRUE and AN has been disabled, it will be
                enabled before being restarted.
 
 Outputs:
    None

 Returns:
    MTD_OK if the change was successful, or MTD_FAIL if not
    
 Description:
    Sets the speed bits in 1.0 back to the power-on default of 11b 
    (10GBASE-T). Enables auto-negotiation.

    Does a software reset of the T unit and wait until it is complete before 
    enabling AN and returning.   

 Notes/Warnings:
    None.

******************************************************************************/
MTD_STATUS mtdUndoForcedSpeed
(
    IN MTD_DEV_PTR devPtr,
    IN MTD_U16 port,
    IN MTD_BOOL anRestart
);


/*****************************************************************************
MTD_STATUS mtdGetForcedSpeed
(
    IN MTD_DEV_PTR devPtr,
    IN MTD_U16 port,
    OUT MTD_BOOL *speedIsForced,
    OUT MTD_U16 *forcedSpeed 
);

 Inputs:
    devPtr - pointer to MTD_DEV 
    port - MDIO port address, 0-31
 
 Outputs:
    speedIsForced - MTD_TRUE if an is disabled (1.0.12 == 0) AND
        the speed in 1.0.13/6 is set to 10BT or 100BT (speeds which do
        not require an to train).
    forcedSpeed - one of the following if speedIsForced is MTD_TRUE
        MTD_SPEED_10M_HD_AN_DIS  - speed forced to 10BT half-duplex
        MTD_SPEED_10M_FD_AN_DIS  - speed forced to 10BT full-duplex
        MTD_SPEED_100M_HD_AN_DIS - speed forced to 100BT half-duplex
        MTD_SPEED_100M_FD_AN_DIS - speed forced to 100BT full-duplex

 Returns:
    MTD_OK if the query was successful, or MTD_FAIL if not
    
 Description:
    Checks if AN is disabled (7.0.12=0) and if the speed select in
    register 1.0.13 and 1.0.6 is set to either 10BT or 100BT speeds. If
    all of this is true, returns MTD_TRUE in speedIsForced along with 
    the speed/duplex setting in forcedSpeedBits. If any of this is 
    false (AN is enabled, or the speed is set to 1000BT or 10GBT), then
    speedIsForced is returned MTD_FALSE and the forcedSpeedBit value
    is invalid.

 Notes/Warnings:
    None.

******************************************************************************/
MTD_STATUS mtdGetForcedSpeed
(
    IN MTD_DEV_PTR devPtr,
    IN MTD_U16 port,
    OUT MTD_BOOL *speedIsForced,
    OUT MTD_U16 *forcedSpeed 
);



#define MTD_CLEAR_PAUSE     0 /*  clears both pause bits */
#define MTD_SYM_PAUSE       1 /*  for symmetric pause only */
#define MTD_ASYM_PAUSE      2 /*  for asymmetric pause only */
#define MTD_SYM_ASYM_PAUSE  3 /*  for both */
/******************************************************************************
MTD_STATUS mtdSetPauseAdvertisement
(
    IN MTD_DEV_PTR devPtr,
    IN MTD_U16 port,
    IN MTD_U8 pauseType,
    IN MTD_BOOL anRestart
);
                               
 Inputs:
    devPtr - pointer to MTD_DEV 
    port - MDIO port address, 0-31
    pauseType - one of the following: 
                MTD_SYM_PAUSE, 
                MTD_ASYM_PAUSE, 
                MTD_SYM_ASYM_PAUSE or 
                MTD_CLEAR_PAUSE.
    anRestart - this takes the value of MTD_TRUE or MTD_FALSE and indicates 
                if auto-negotiation should be restarted following the speed 
                enable change. If this is MTD_FALSE, the change will not
                take effect until AN is restarted in some other way (link
                drop, toggle low power, toggle AN enable, toggle soft reset).

                If this is MTD_TRUE and AN has been disabled, it will be
                enabled before being restarted.

 Outputs:
    None

 Returns:
    MTD_OK or MTD_FAIL, if action was successful or failed

 Description:
    This function sets the asymmetric and symmetric pause bits in the technology 
    ability field in the AN Advertisement register and optionally restarts 
    auto-negotiation to use the new values. This selects what type of pause 
    is to be advertised to the far end MAC during auto-negotiation. If 
    auto-negotiation is restarted, it is enabled first.

    Sets entire 2-bit field to the value passed in pauseType.

    To clear both bits, pass in MTD_CLEAR_PAUSE.

 Side effects:
    None

 Notes/Warnings:
    This function will not take effect unless the auto-negotiation is restarted.

******************************************************************************/
MTD_STATUS mtdSetPauseAdvertisement
(
    IN MTD_DEV_PTR devPtr,
    IN MTD_U16 port,
    IN MTD_U8 pauseType,
    IN MTD_BOOL anRestart
);

/******************************************************************************
MTD_STATUS mtdGetPauseAdvertisement
(
    IN MTD_DEV_PTR devPtr,
    IN MTD_U16 port,
    OUT MTD_U8 *pauseType
);

                               
 Inputs:
    devPtr - pointer to MTD_DEV initialized by mtdLoadDriver() call
    port - MDIO port address, 0-31


 Outputs:
    pauseType - one of the following: 
                MTD_SYM_PAUSE, 
                MTD_ASYM_PAUSE, 
                MTD_SYM_ASYM_PAUSE or 
                MTD_CLEAR_PAUSE.

 Returns:
    MTD_OK or MTD_FAIL, if action was successful or failed

 Description:
    This function reads the asymmetric and symmetric pause bits in the technology 
    ability field in the AN Advertisement register. S

 Side effects:
    None

 Notes/Warnings:
    The current register setting does not indicate what is currently being
    sent. The setting does not take effect until after auto-negotiation
    is restarted.

******************************************************************************/
MTD_STATUS mtdGetPauseAdvertisement
(
    IN MTD_DEV_PTR devPtr,
    IN MTD_U16 port,
    OUT MTD_U8 *pauseType
);

/******************************************************************************
MTD_STATUS mtdGetLPAdvertisedSpeeds
(
    IN MTD_DEV_PTR devPtr,
    IN MTD_U16 port,
    OUT MTD_U16 *ieeeSpeedBits,
    OUT MTD_U16 *nbasetSpeedBits
);

 Inputs:
    devPtr - pointer to MTD_DEV 
    port - MDIO port address, 0-31

 Outputs:
    ieeeSpeedBits - the speeds the link partner advertised via the base page,
        10GBASE-T message page or 1GBASE-T message page
        One or all of the following (bit mask)
            MTD_ADV_NONE
            MTD_SPEED_10M_HD
            MTD_SPEED_10M_FD
            MTD_SPEED_100M_HD
            MTD_SPEED_100M_FD
            MTD_SPEED_1GIG_HD
            MTD_SPEED_1GIG_FD
            MTD_SPEED_10GIG_FD
            MTD_SPEED_2P5GIG_FD
            MTD_SPEED_5GIG_FD
        
    nbasetSpeedBits - the speeds the link partner advertised via the
        NBASE-T message page
        One or both of the following (bit mask)
            MTD_ADV_NONE (always this value if the device is not 88X33X0/88E20X0 device)
            MTD_SPEED_2P5GIG_FD
            MTD_SPEED_5GIG_FD

           

 Returns:
    MTD_OK or MTD_FAIL, based on whether the query succeeded or failed. Returns
    MTD_FAIL and no speeds bits will be set if AN is not complete.

 Description:
    This function reads the AN status registers and reports the speeds the
    link partner advertised during AN.

    These values are only valid after AN completes.

 Side effects:
    None

 Notes/Warnings:
    AN must be complete in order for this function to return valid results.

    Check 7.1.5, AN complete or check the return status to know if the result
    is valid. Returns MTD_FAIL if AN is not complete.

    88X32X0 device family always returns MTD_ADV_NONE for nbasetSpeedBits
    regardless if LP advertised 2.5G/5G or not.

******************************************************************************/
MTD_STATUS mtdGetLPAdvertisedSpeeds
(
    IN MTD_DEV_PTR devPtr,
    IN MTD_U16 port,
    OUT MTD_U16 *ieeeSpeedBits,
    OUT MTD_U16 *nbasetSpeedBits /* this is always MTD_ADV_NONE for !(88X33X0/88E20X0/E21X0) devices */
);


/******************************************************************************
MTD_STATUS mtdGetLPAdvertisedPause
(
    IN MTD_DEV_PTR devPtr,
    IN MTD_U16 port,
    OUT MTD_U8 *pauseBits
);

 Inputs:
    devPtr - pointer to MTD_DEV 
    port - MDIO port address, 0-31

 Outputs:
    pauseBits - setting of link partner's pause bits based on bit definitions above in 
                mtdmtdSetPauseAdvertisement()

 Returns:
    MTD_OK or MTD_FAIL, based on whether the query succeeded or failed. Returns
    MTD_FAIL and MTD_CLEAR_PAUSE if AN is not complete.

 Description:
    This function reads 7.19 (LP Base page ability) and returns the advertised
    pause setting that was received from the link partner.

 Side effects:
    None

 Notes/Warnings:
    AN must be complete in order for this function to return valid results.

    Check 7.1.5, AN complete or check the return status to know if the result
    is valid. Returns MTD_FAIL if AN is not complete.

******************************************************************************/
MTD_STATUS mtdGetLPAdvertisedPause
(
    IN MTD_DEV_PTR devPtr,
    IN MTD_U16 port,
    OUT MTD_U8 *pauseBits
);


/******************************************************************************
MTD_STATUS mtdGetTxRxPauseResolution
(
    IN MTD_DEV_PTR devPtr,
    IN MTD_U16 port,
    OUT MTD_BOOL *tx_pause_enabled,
    OUT MTD_BOOL *rx_pause_enabled
);

 Inputs:
    devPtr - pointer to MTD_DEV 
    port - MDIO port address, 0-31

 Outputs:
    pauseResolved - MTD_TRUE/MTD_FALSE based on whether AN speed/duplex
                    is resolved or not
    tx_pause_enabled - MTD_TRUE/MTD_FALSE based on local and LP pause
                       advertisement
    rx_pause_enabled - MTD_TRUE/MTD_FALSE based on local and LP pause
                       advertisement

 Returns:
    MTD_OK or MTD_FAIL, based on whether the query succeeded or failed

 Description:
    This function first checks 3.8008.11 to insure that autonegotiation
    has resolved the pause settings and sets pauseResolved to 
    MTD_TRUE or MTD_FALSE based on this register bit.
 
    This function reads 3.8008.9:8 and sets corresponding flags based
    on bits.

 Side effects:
    None

 Notes/Warnings:
    None

******************************************************************************/
MTD_STATUS mtdGetTxRxPauseResolution
(
    IN MTD_DEV_PTR devPtr,
    IN MTD_U16 port,
    OUT MTD_BOOL *pauseResolved,
    OUT MTD_BOOL *tx_pause_enabled,
    OUT MTD_BOOL *rx_pause_enabled
);


/******************************************************************************
 MTD_STATUS mtdAutonegRestart
 (
     IN MTD_DEV_PTR devPtr,
     IN MTD_U16 port
 );

 Inputs:
    devPtr - pointer to MTD_DEV 
    port - MDIO port address, 0-31

 Outputs:
    None

 Returns:
    MTD_OK or MTD_FAIL, depending on if action was successful

 Description:
    Restarts auto-negotiation. The bit is self-clearing. If the link is up,
    the link will drop and auto-negotiation will start again.

 Side effects:
    None.

 Notes/Warnings:
    Restarting auto-negotiation will have no effect if auto-negotiation is 
    disabled.

    This function is important as it is necessary to restart auto-negotiation 
    after changing many auto-negotiation settings before the changes will take 
    effect.

    Will return MTD_FAIL if the T unit is not in a mode which it can respond to.
    For example, low power mode, diagnostic modes, BIST failure or in the middle 
    of a software reset, no firmware downloaded, etc..    

******************************************************************************/
MTD_STATUS mtdAutonegRestart
(
    IN MTD_DEV_PTR devPtr,
    IN MTD_U16 port
);

/******************************************************************************
 MTD_STATUS mtdAutonegDisable
 (
     IN MTD_DEV_PTR devPtr,
     IN MTD_U16 port
 );

 Inputs:
    devPtr - pointer to MTD_DEV 
    port - MDIO port address, 0-31

 Outputs:
    None

 Returns:
    MTD_OK or MTD_FAIL, if action was successful or not

 Description:
    Turns off the enable auto-negotiation bit disabling auto-negotiation. 
    The phy will not train with auto-negotiation disabled (except in 100M/10BT 
    forced mode).

 Side effects:

 Notes/Warnings:
    Restart autonegation will not take effect if AN is disabled.

    IMPORTANT. AN cannot be disabled if the speed in 1.0 is set to 1G...10G
    speeds. The PHY will ignore writes to 7.0.12 = 0 if the speed is
    1G...10G. This speed in 1.0 must first be set to 10M or 100M
    before calling this function for it to have any effect on 7.0.12.

    To keep the PHY from coming up at any speed, set speed advertisement
    to MTD_ADV_NONE and restart AN.

    To make PHY quiet (no transmission at all) put PHY into low
    power mode, mtdPutTunitInLowPowerMode().

******************************************************************************/
MTD_STATUS mtdAutonegDisable
(
    IN MTD_DEV_PTR devPtr,
    IN MTD_U16 port
);

/******************************************************************************
 MTD_STATUS mtdAutonegEnable
 (
     IN MTD_DEV_PTR devPtr,
     IN MTD_U16 port
 );

 Inputs:
    devPtr - pointer to MTD_DEV 
    port - MDIO port address, 0-31

 Outputs:
    None

 Returns:
    MTD_OK or MTD_FAIL, if action was successful or not

 Description:
    Re-enables auto-negotiation.

 Side effects:

 Notes/Warnings:
    Restart autonegation will not take effect if AN is disabled.

******************************************************************************/
MTD_STATUS mtdAutonegEnable
(
    IN MTD_DEV_PTR devPtr,
    IN MTD_U16 port
);

/******************************************************************************
 MTD_STATUS mtdAutonegForceMaster
 (
     IN MTD_DEV_PTR devPtr,
     IN MTD_U16 port,
     IN MTD_BOOL anRestart
 );

 Inputs:
    devPtr - pointer to MTD_DEV 
    port - MDIO port address, 0-31
    anRestart - this takes the value of MTD_TRUE or MTD_FALSE and indicates 
                if auto-negotiation should be restarted following the speed 
                enable change. If this is MTD_FALSE, the change will not
                take effect until AN is restarted in some other way (link
                drop, toggle low power, toggle AN enable, toggle soft reset).

                If this is MTD_TRUE and AN has been disabled, it will be
                enabled before being restarted.


 Outputs:
    None

 Returns:
    MTD_OK or MTD_FAIL, if action was successful or not

 Description:
    Sets auto-negotiation to manual and forced as a master.

    Restarts auto-negotiation by calling the function only if the input 
    parameter anRestart was selected. Also re-enables auto-negotiation before 
    restarting auto-negotiation.

 Side effects:
    auto-negotiation has to be restarted for it to take effect

 Notes/Warnings:
    Auto-negotiation will not complete if far end is also forced as a master.

******************************************************************************/
MTD_STATUS mtdAutonegForceMaster
(
    IN MTD_DEV_PTR devPtr,
    IN MTD_U16 port,
    IN MTD_BOOL anRestart
);

/******************************************************************************
 MTD_STATUS mtdAutonegForceSlave
 (
     IN MTD_DEV_PTR devPtr,
     IN MTD_U16 port,
     IN MTD_BOOL anRestart
 );
                          

 Inputs:
    devPtr - pointer to MTD_DEV 
    port - MDIO port address, 0-31
    anRestart - this takes the value of MTD_TRUE or MTD_FALSE and indicates 
                if auto-negotiation should be restarted following the speed 
                enable change. If this is MTD_FALSE, the change will not
                take effect until AN is restarted in some other way (link
                drop, toggle low power, toggle AN enable, toggle soft reset).

                If this is MTD_TRUE and AN has been disabled, it will be
                enabled before being restarted.

 Outputs:
    None

 Returns:
    MTD_OK or MTD_FAIL, if action was successful or not

 Description:
    Sets auto-negotiation to manual and forced as a slave.

    Restarts auto-negotiation by calling the function only if the input parameter 
    anRestart was selected. Also re-enables auto-negotiation before restarting 
    auto-negotiation.

 Side effects:
    Auto-negotiation has to be restarted for it to take effect

 Notes/Warnings:
    Auto-negotiation will not complete if far end is also forced as a slave.

******************************************************************************/
MTD_STATUS mtdAutonegForceSlave
(
    IN MTD_DEV_PTR devPtr,
    IN MTD_U16 port,
    IN MTD_BOOL anRestart
);


/******************************************************************************
 MTD_STATUS mtdAutonegSetAutoMasterSlave
 (
     IN MTD_DEV_PTR devPtr,
     IN MTD_U16 port,
     IN MTD_BOOL anRestart
 );

 Inputs:
    devPtr - pointer to MTD_DEV 
    port - MDIO port address, 0-31
    anRestart - this takes the value of MTD_TRUE or MTD_FALSE and indicates 
                if auto-negotiation should be restarted following the speed 
                enable change. If this is MTD_FALSE, the change will not
                take effect until AN is restarted in some other way (link
                drop, toggle low power, toggle AN enable, toggle soft reset).

                If this is MTD_TRUE and AN has been disabled, it will be
                enabled before being restarted.

 Outputs:
    None

 Returns:
    MTD_OK or MTD_FAIL, if action was successful or not

 Description:
    Sets the master/slave determination to "auto" which means the 
    master/slave setting will be random from one auto-negotiation to the next 
    auto-negotiation.

    Restarts auto-negotiation by calling the function only if the input 
    parameter anRestart was selected. Also re-enables auto-negotiation before 
    restarting auto-negotiation.

 Side effects:
    Auto-negotiation has to be restarted for it to take effect

******************************************************************************/
MTD_STATUS mtdAutonegSetAutoMasterSlave
(
    IN MTD_DEV_PTR devPtr,
    IN MTD_U16 port,
    IN MTD_BOOL anRestart
);

/******************************************************************************
 MTD_STATUS mtdAutonegSetMasterPreference
 (
     IN MTD_DEV_PTR devPtr,
     IN MTD_U16 port,
     IN MTD_BOOL anRestart
 );

 Inputs:
    devPtr - pointer to MTD_DEV 
    port - MDIO port address, 0-31
    anRestart - this takes the value of MTD_TRUE or MTD_FALSE and indicates 
                if auto-negotiation should be restarted following the speed 
                enable change. If this is MTD_FALSE, the change will not
                take effect until AN is restarted in some other way (link
                drop, toggle low power, toggle AN enable, toggle soft reset).

                If this is MTD_TRUE and AN has been disabled, it will be
                enabled before being restarted.

 Outputs:
    None

 Returns:
    MTD_OK or MTD_FAIL, if action was successful or not

 Description:
    Sets master/slave setting to "auto" and selects multi-port (preference is
    to be a master).

    Restarts auto-negotiation by calling the function only if the input parameter 
    anRestart was selected. Also re-enables auto-negotiation before restarting 
    auto-negotiation.

 Side effects:
    None

 Notes/Warnings:
     auto-negotiation has to be restarted for it to take effect


******************************************************************************/
MTD_STATUS mtdAutonegSetMasterPreference
(
    IN MTD_DEV_PTR devPtr,
    IN MTD_U16 port,
    IN MTD_BOOL anRestart
);

/******************************************************************************
 MTD_STATUS mtdAutonegSetSlavePreference
 (
     IN MTD_DEV_PTR devPtr,
     IN MTD_U16 port,
     IN MTD_BOOL anRestart
 );
 Inputs:
    devPtr - pointer to MTD_DEV 
    port - MDIO port address, 0-31
    anRestart - this takes the value of MTD_TRUE or MTD_FALSE and indicates 
                if auto-negotiation should be restarted following the speed 
                enable change. If this is MTD_FALSE, the change will not
                take effect until AN is restarted in some other way (link
                drop, toggle low power, toggle AN enable, toggle soft reset).

                If this is MTD_TRUE and AN has been disabled, it will be
                enabled before being restarted.

 Outputs:
    None

 Returns:
    MTD_OK or MTD_FAIL, if action was successful or not

 Description:
    Sets master/slave setting to "auto" and selects single-port (preference is
    to be a slave).

    Restarts auto-negotiation by calling the function only if the input 
    parameter anRestart was selected. Also re-enables auto-negotiation before 
    restarting auto-negotiation.

 Side effects:
    None

 Notes/Warnings:
    Auto-negotiation has to be restarted for it to take effect

******************************************************************************/
MTD_STATUS mtdAutonegSetSlavePreference
(
    IN MTD_DEV_PTR devPtr,
    IN MTD_U16 port,
    IN MTD_BOOL anRestart
);


#define MTD_AN_MS_PREFER_MASTER     (1) /* Auto, preference is to be master */
#define MTD_AN_MS_PREFER_SLAVE      (2) /* Auto, preference is to be slave */
#define MTD_AN_MS_FORCED_MASTER     (3) /* Manual, forced master */
#define MTD_AN_MS_FORCED_SLAVE      (4) /* Manual, forced slave */

/******************************************************************************
 MTD_STATUS mtdAutonegGetMasterSlavePreference
 (
     IN MTD_DEV_PTR devPtr,
     IN MTD_U16 port,
     OUT MTD_U16 *msPreference
 );
 
 Inputs:
    devPtr - pointer to MTD_DEV initialized by mtdLoadDriver() call
    port - MDIO port address, 0-31

 Outputs:
    msPreference - one of the following:
        MTD_AN_MS_PREFER_MASTER: AN is set auto to prefer master
        MTD_AN_MS_PREFER_SLAVE: AN is set auto to prefer slave
        MTD_AN_MS_FORCED_MASTER: AN is set to manual and forced master
        MTD_AN_MS_FORCED_SLAVE: AN is set to manual and forced as slave

 Returns:
    MTD_OK or MTD_FAIL, if action was successful or not

 Description:
    Check the master/slave setting for autonegotiation and returns
    what the setting is.

 Side effects:
    None

 Notes/Warnings:
    None

******************************************************************************/
MTD_STATUS mtdAutonegGetMasterSlavePreference
(
    IN MTD_DEV_PTR devPtr,
    IN MTD_U16 port,
    OUT MTD_U16 *msPreference
);


#define MTD_AN_RESOLUTION_FAULT   (2) /* if a config fault was detected */
#define MTD_AN_RESOLUTION_MASTER  (1)
#define MTD_AN_RESOLUTION_SLAVE   (0)
#define MTD_AN_RESOLUTION_UNKNOWN (0xFF) /* During autonegotiation */
/******************************************************************************
 MTD_STATUS mtdAutonegIsResolutionMasterOrSlave
 (
     IN MTD_DEV_PTR devPtr,
     IN MTD_U16 port,
     OUT MTD_U8 *anMsResolution
 );

 Inputs:
    devPtr - pointer to MTD_DEV 
    port - MDIO port address, 0-31

 Outputs:
    anMsResolution - one of the following
        MTD_AN_RESOLUTION_MASTER if the PHY is a MASTER and autoneg was resolved
        MTD_AN_RESOLUTION_SLAVE if the PHY is a SLAVE and autoneg was resolved        
        MTD_AN_RESOLUTION_UNKNOWN if it is not resolved yet
        MTD_AN_RESOLUTION_FAULT if a config fault is detected (for example, both
            sides forced to slave, etc.)

 Returns:
    MTD_OK or MTD_FAIL, if query was successful or not

 Description:
    The function checks if auto-negotiation speed is resolved by calling the
    mtdAutonegIsSpeedDuplexResolutionDone(). If speed/duplex is not resolved,
    it then checks for a config fault. If a config fault exists, it
    returns MTD_AN_RESOLUTION_FAULT for anMsResolution. If a config
    fault doesn't exist, it returns MTD_AN_RESOLUTION_UNKNOWN since
    AN is in progress.

    If AN is done (meaning duplex/speed is resolved AND autonegotiation
    is completed), the master/slave bit from the correct register either 7.33
    or 7.32769 is read and a value of AN_MTD_RESOLUTION_MASTER/SLAVE is returned 
    for anMsResolution.

    If AN is done but not completed (meaning speed/duplex are resolved,
    but the link has not trained up), AN_MTD_RESOLUTION_UNKNOWN is returned.
    Master/slave status bits are not valid until AN is complete (7.1.5 is 1).

 Side effects:
    Calling this function will clear the auto-negotiation config fault bit in
    register 7.33/32769 since it is a latched bit.

 Notes/Warnings:
    None

******************************************************************************/
MTD_STATUS mtdAutonegIsResolutionMasterOrSlave
(
    IN MTD_DEV_PTR devPtr,
    IN MTD_U16 port,
    OUT MTD_U8 *anMsResolution
);


/******************************************************************************
 MTD_STATUS mtdGetAutonegSpeedDuplexResolution
 (
     IN MTD_DEV_PTR devPtr,
     IN MTD_U16 port,
     OUT MTD_U16 *speedResolution
 );

 Inputs:
    devPtr - pointer to MTD_DEV 
    port - MDIO port address, 0-31

 Outputs: 
    speedResolution - one of the following:
        MTD_ADV_NONE - autonegotiation is in progress or disabled
        MTD_SPEED_10M_HD - speed is resolved to 10BT half-duplex
        MTD_SPEED_10M_FD - speed is resolved to 10BT full-duplex
        MTD_SPEED_100M_HD - speed is resolved to 100BASE-TX half-duplex
        MTD_SPEED_100M_FD - speed is resolved to 100BASE-TX full-duplex
        MTD_SPEED_1GIG_HD - speed is resolved to 1000BASE-T half-duplex
        MTD_SPEED_1GIG_FD - speed is resolved to 1000BASE-T full-duplex
        MTD_SPEED_10GIG_FD - speed is resolved to 10GBASE-T    
        MTD_SPEED_2P5GIG_FD - speed is resolved to 2.5GBASE-T (88X33X0/88E20X0/88E21X0 family only)
        MTD_SPEED_5GIG_FD - speed is resolved to 5GBASE-T (88X33X0/88E20X0/88E21X0 family only)

  Returns: 
    MTD_OK or MTD_FAIL, if query was successful or not.

 Description:
    Checks if autonegoation is disabled, and if so, returns MTD_ADV_NONE.
    If autonegotiation is enabled, checks if autonegotiation speed/duplex is ready
    by calling mtdAutonegIsSpeedDuplexResolutionDone(), if not returns MTD_ADV_NONE,
    if it is ready, then it reads the speed and duplex resolution
    in 3.8008.15:13 and returns one of the values above based on these
    three bits.

    Speed/duplex being resolved does not imply autonegotiation is completed
    (that values in 7.1, 7.33, etc. are valid). Check autonegiation complete,
    mtdAutonegIsCompleted(), before reading any autonegotiation registers
    in MMD 7.

    This function returns and early indication of what speed and duplex has
    been autonegotiated and will attempt to train (or trained up if called 
    after the link is up).

 Side effects:
    None

 Notes/Warnings:
    If speed is forced to 100BASE-TX or 10BASE-T through register 1.0,
    this function should not be called. It may return incorrect results. 
    If the speed may be forced (meaning autonegotiation is disabled) to 
    one of these, first check if the speed is forced by calling 
    mtdGetForcedSpeed(). mtdGetAutonegSpeedDuplexResolution() is only to be used
    if autonegotiation is enabled.

    The value returned gives the result of autonegoation, but does not
    mean the copper link is up. It could be training or re-training. To
    check if the copper link is up and at what speed, call mtdIsBaseTUp().

******************************************************************************/
MTD_STATUS mtdGetAutonegSpeedDuplexResolution
(
    IN MTD_DEV_PTR devPtr,
    IN MTD_U16 port,
    OUT MTD_U16 *speedResolution
);

/******************************************************************************
MTD_STATUS mtdIsBaseTUp(
    IN MTD_DEV_PTR devPtr,
    MTD_U16 port,
    MTD_U16 *speed,
    MTD_BOOL *linkUp
);
                                  

 Inputs:
    devPtr - pointer to MTD_DEV 
    port - MDIO port address, 0-31

 Outputs:
    speed - which speed was resolved or will be resolved by auto-negotiation, 
            one of following
             MTD_ADV_NONE - autonegotiation is in progress or disabled
             MTD_SPEED_10M_HD - speed is resolved to 10BT half-duplex
             MTD_SPEED_10M_FD - speed is resolved to 10BT full-duplex
             MTD_SPEED_100M_HD - speed is resolved to 100BASE-TX half-duplex
             MTD_SPEED_100M_FD - speed is resolved to 100BASE-TX full-duplex
             MTD_SPEED_1GIG_HD - speed is resolved to 1000BASE-T half-duplex
             MTD_SPEED_1GIG_FD - speed is resolved to 1000BASE-T full-duplex
             MTD_SPEED_10GIG_FD - speed is resolved to 10GBASE-T  
             MTD_SPEED_2P5GIG_FD - speed is resolved to 2.5GBASE-T (88X33X0/88E20X0/88E21X0 family only)
             MTD_SPEED_5GIG_FD - speed is resolved to 5GBASE-T (88X33X0/88E20X0/88E21X0 family only)

             MTD_SPEED_10M_HD_AN_DIS - speed forced to 10BT half-duplex
             MTD_SPEED_10M_FD_AN_DIS - speed forced to 10BT full-duplex
             MTD_SPEED_100M_HD_AN_DIS - speed forced to 100BT half-duplex
             MTD_SPEED_100M_FD_AN_DIS - speed forced to 100BT full-duplex
             MTD_SPEED_MISMATCH - speed is forced to one of above speeds, but
                                  indicated up at a different speed (this is an error)
             
    linkUp - MTD_TRUE if the copper link is up at the speed above
             MTD_FALSE if the link is down or autonegotiation is in progress or
             there's an error

 Returns:
    MTD_OK, if query was successful, MTD_FAIL if query was unsuccessful or if there
    was an error such as MTD_SPEED_MISMATCH. MTD_OK indicates speed and link
    status are valid. MTD_FAIL indicates speed and link status are invalid.

 Description:

    First checks if the speed is forced to 10BT or 100BTX, and if it is returns
    one of the following combinations

    Speed Forced Value           Link Status    Status
    ------------------           -----------    ------
    MTD_SPEED_10M_HD_AN_DIS      MTD_TRUE/FALSE MTD_OK - query successful, link status/speed valid
    MTD_SPEED_10M_FD_AN_DIS      MTD_TRUE/FALSE MTD_OK - query successful, link status/speed valid
    MTD_SPEED_100M_HD_AN_DIS     MTD_TRUE/FALSE MTD_OK - query successful, link status/speed valid
    MTD_SPEED_100M_FD_AN_DIS     MTD_TRUE/FALSE MTD_OK - query successful, link status/speed valid 
    any of the above with        MTD_FALSE      MTD_FAIL - query failed

    MTD_SPEED_MISMATCH           MTD_FALSE      MTD_FAIL - link at different speed than forced value
                                                This is an error. Link may be up at wrong speed
                                                or training at wrong speed.

    If speed is not forced, then checks if autonegotiation is resolved or in-progress and
    returns one of the following combinations:

    Autonegotiation Result        Link Status    Status
    ---------------------        -----------    ------
    MTD_ADV_NONE                 MTD_FALSE      MTD_OK - autonegotiation is in progress (link is down)
    MTD_SPEED_10M_HD             MTD_TRUE/FALSE MTD_OK - query successful, link status/speed valid
    MTD_SPEED_10M_FD             MTD_TRUE/FALSE MTD_OK - query successful, link status/speed valid
    MTD_SPEED_100M_HD            MTD_TRUE/FALSE MTD_OK - query successful, link status/speed valid
    MTD_SPEED_100M_FD            MTD_TRUE/FALSE MTD_OK - query successful, link status/speed valid
    MTD_SPEED_1GIG_HD            MTD_TRUE/FALSE MTD_OK - query successful, link status/speed valid 
    MTD_SPEED_1GIG_FD            MTD_TRUE/FALSE MTD_OK - query successful, link status/speed valid 
    MTD_SPEED_10GIG_FD           MTD_TRUE/FALSE MTD_OK - query successful, link status/speed valid
    MTD_SPEED_2P5GIG_FD          MTD_TRUE/FALSE MTD_OK - query successful, link status/speed valid
    MTD_SPEED_5GIG_FD            MTD_TRUE/FALSE MTD_OK - query successful, link status/speed valid
    
    any of the above with        MTD_FALSE      MTD_FAIL - query failed

 Side effects:
    None

 Notes/Warnings:
    Gives current real-time status of the copper interface.
******************************************************************************/
MTD_STATUS mtdIsBaseTUp(
    IN MTD_DEV_PTR devPtr,
    IN MTD_U16 port,
    OUT MTD_U16 *speed,
    OUT MTD_BOOL *linkUp
);

#define MTD_FORCE_MDI        0x00
#define MTD_FORCE_MDIX       0x01
/* 0x02 is reserved */
#define MTD_AUTO_MDI_MDIX    0x03
/*****************************************************************************
MTD_STATUS mtdMDIXControl
(
    IN MTD_DEV_PTR devPtr,
    IN MTD_U16 port,
    IN MTD_U16 MDIXOptions, 
    IN MTD_BOOL swReset
);

 Inputs:
    devPtr - pointer to MTD_DEV 
    port - MDIO port address, 0-31
    MDIXOptions - 0x00 - MTD_FORCE_MDI
                  0x01 - MTD_FORCE_MDIX
                  0x03 - MTD_AUTO_MDI_MDIX
    swReset - MTD_TRUE is to do a software reset
              MTD_FALSE is not to do a software reset, it should
              be done later for the change to take effect
 Outputs:
    None

 Returns:
    MTD_OK or MTD_FAIL, if action was successful or not

 Description:
    The function forces the MDI/MDI-X or sets it to auto MDI/MDI-X based on the
    input. Optionally the software reset is issued following the change (this
    change requires a software reset to take effect).

 Side effects:
    None

 Notes/Warnings:
    Following changes to MDI/MDI-X, a software reset must be performed to
    restart the state machine. This will drop the link. If swReset is set to
    MTD_FALSE, it must be done later to make change become active.

    Following a software reset, the PHY should be polled looking for the
    software reset to be completed before interacting with the PHY via any
    other MDIO registers by polling mtdIsPhyReadyAfterReset().
******************************************************************************/
MTD_STATUS mtdMDIXControl
(
    IN MTD_DEV_PTR devPtr,
    IN MTD_U16 port,
    IN MTD_U16 MDIXOptions, 
    IN MTD_BOOL swReset
);


/*****************************************************************************
MTD_STATUS mtdGetMDIXControl
(
    IN MTD_DEV_PTR devPtr,
    IN MTD_U16 port,
    OUT MTD_U16 *MDIXOptions,
);

 Inputs:
    devPtr - pointer to MTD_DEV 
    port - MDIO port address, 0-31

 Outputs:
    MDIXOptions - 0x00 - MTD_FORCE_MDI
                  0x01 - MTD_FORCE_MDIX
                  0x03 - MTD_AUTO_MDI_MDIX

 Returns:
    MTD_OK or MTD_FAIL, if action was successful or not

 Description:
    The function queries the current setting of the MDIX control
    bits.

    NOTE: The current value of the register bits are returned.
    These bits only become valid following a software reset. If
    the register was changed, but no software reset was issued,
    the new value will be returned but won't have taken effect
    yet.

 Side effects:
    None

 Notes/Warnings:
    Following changes to MDI/MDI-X, a software reset must be performed to
    restart the state machine. This will drop the link. If swReset is set to
    MTD_FALSE, it must be done later to make change become active.

    Following a software reset, the PHY should be polled looking for the
    software reset to be completed before interacting with the PHY via any
    other MDIO registers by polling mtdIsPhyReadyAfterReset().
******************************************************************************/
MTD_STATUS mtdGetMDIXControl
(
    IN MTD_DEV_PTR devPtr,
    IN MTD_U16 port,
    OUT MTD_U16 *MDIXOptions
);

#define MTD_MDI_RESOLVED           0x00
#define MTD_MDIX_RESOLVED          0x01
#define MTD_MDI_MDIX_NOT_RESOLVED  0x02
/*****************************************************************************
 MTD_STATUS mtdGetMDIXResolution
 (
     IN MTD_DEV_PTR devPtr,
     IN MTD_U16 port,
     OUT MTD_U8 *MDIX
 );


 Inputs:
    devPtr - pointer to MTD_DEV 
    port - MDIO port address, 0-31

 Outputs:
    MDIX - MTD_MDIX_RESOLVED means AN resolved to MDIX (local device transmitting on pair
           B and receiving on pair A)
           MTD_MDI_RESOLVED means AN resolved to MDI (local device transmitting on pair
           A and receiving on pair B)
           MTD_MDI_MDIX_NOT_RESOLVED means resolution is not ready yet (link could
           be unplugged, or AN could be in progress but not to the point of reporting this)

 Returns:
    MTD_OK or MTD_FAIL, if query was successful or not

 Description:
    Checks if speed/duplex has been resolved (3.8008.11 is 1). If it has not,
    returns MTD_MDI_MDIX_NOT_RESOLVED.

    If speed/duplex has been resolved, reads the MDI/MDI-X resolution in
    3.8008.6 and returns the value in MDIX along with MTD_OK for the return
    status.

 Side effects:
    None

 Notes/Warnings:
    This function checks for MDI/MDI-X setting in the vendor specific register
    3.8008. For 10GBASE-T, there is also an IEEE defined bit field in 1.130.1:0
    which can be checked once the 10GBASE-T link is up. However, that bit field
    is only valid in 10GBASE-T operation. 3.8008.6 is valid for all speeds
    where MDI/MDI-X is relevant (100BT, 1000BT, 10GBT).
******************************************************************************/
MTD_STATUS mtdGetMDIXResolution
(
    IN MTD_DEV_PTR devPtr,
    IN MTD_U16 port,
    OUT MTD_U8 *MDIX
);


/******************************************************************************
 MTD_STATUS mtdAutonegSetNbaseTAdvertisement
 (
     IN MTD_DEV_PTR devPtr,
     IN MTD_U16 port,
     IN MTD_BOOL enableNbaseTMP,
     IN MTD_BOOL anRestart
 );
 
 Inputs:
    devPtr - pointer to MTD_DEV 
    port - MDIO port address, 0-31
    enableNbaseTMP - set to MTD_TRUE to enable sending the NBASE-T message page
                     or set to MTD_FALSE to disable sending the NBASE-T message
                     page
    anRestart - this takes the value of MTD_TRUE or MTD_FALSE and indicates 
                if auto-negotiation should be restarted following the change. 
                If this is MTD_FALSE, the change will not
                take effect until AN is restarted in some other way (link
                drop, toggle low power, toggle AN enable, toggle soft reset).

                MTD_TRUE will both enable AN and restart it.

 Outputs:
    None

 Returns:
    MTD_OK or MTD_FAIL, if action was successful or not

 Description:
    Either enables sending the NBASE-T OUI Message Page or disables it
    When enabled, will use the speed control bits in 7.32 to advertise support
    of 2.5G or 5G NBASE-T for interoperability with 2.5/5G NBASE-T
    only devices.

    If disabled, will not send the NBASE-T Message Page and will not
    interoperate with devices which only support NBASE-T 2.5/5G.

 Side effects:
    None

 Notes/Warnings:
    Auto-negotiation has to be restarted for it to take effect

******************************************************************************/
MTD_STATUS mtdAutonegSetNbaseTAdvertisement
(
    IN MTD_DEV_PTR devPtr,
    IN MTD_U16 port,
    IN MTD_BOOL enableNbaseTMP,
    IN MTD_BOOL anRestart
);


/******************************************************************************
MTD_STATUS mtdAutonegSetNbaseTDownshiftControl
(
    IN MTD_DEV_PTR devPtr,
    IN MTD_U16 port,
    IN MTD_U16 downshiftThreshold,
    IN MTD_BOOL noEnergyReset,
    IN MTD_BOOL downshiftEnable,
    IN MTD_BOOL anRestart
);;

 
 Inputs:
    devPtr - pointer to MTD_DEV 
    port - MDIO port address, 0-31

    downShiftThreshold - 0...15, threshold of number of failed training
        attempts before downshifting to next lower enabled speed.

    NOTE: setting threshold to 0 disables downshift

    noEnergyReset - MTD_TRUE to re-enable all speeds when no energy
    is detected on MDI. MTD_FALSE to disable this feature.

    downshiftEnable - MTD_TRUE to enable downshift. MTD_FALSE
        to disable downshift. (setting threshold to 0 overrides this,
        see above)

    anRestart - MTD_TRUE to restart AN after updating register. MTD_FALSE
        to not restart AN (do it later to have changes take effect).
    

 Outputs:
    None

 Returns:
    MTD_OK or MTD_FAIL, if action was successful or not

 Description:
    Sets the control bits for the downshift control register.

    Changes don't take effect until AN is restarted.
    

 Side effects:
    None

 Notes/Warnings:
    None

******************************************************************************/
MTD_STATUS mtdAutonegSetNbaseTDownshiftControl
(
    IN MTD_DEV_PTR devPtr,
    IN MTD_U16 port,
    IN MTD_U16 downshiftThreshold,
    IN MTD_BOOL noEnergyReset,
    IN MTD_BOOL downshiftEnable,
    IN MTD_BOOL anRestart
);

/******************************************************************************
 MTD_STATUS mtdAutonegGetNbaseTDownshiftControl
 (
     IN MTD_DEV_PTR devPtr,
     IN MTD_U16 port,
     IN MTD_U16 *downshiftThreshold,
     IN MTD_BOOL *noEnergyReset,
     IN MTD_BOOL *downshiftEnable
 );

 
 Inputs:
    devPtr - pointer to MTD_DEV 
    port - MDIO port address, 0-31
    
 Outputs:
    downShiftThreshold - 0...15, threshold of number of failed training
        attempts before downshifting to next lower enabled speed.

    noEnergyReset - MTD_TRUE to re-enable all speeds when no energy
        is detected on MDI. MTD_FALSE to disable this feature.

    downshiftEnable - MTD_TRUE to enable downshift. MTD_FALSE
        to disable downshift.

 Returns:
    MTD_OK or MTD_FAIL, if action was successful or not

 Description:
    Reads the control bits for the downshift control register.
    

 Side effects:
    None

 Notes/Warnings:
    None

******************************************************************************/
MTD_STATUS mtdAutonegGetNbaseTDownshiftControl
(
    IN MTD_DEV_PTR devPtr,
    IN MTD_U16 port,
    IN MTD_U16 *downshiftThreshold,
    IN MTD_BOOL *noEnergyReset,
    IN MTD_BOOL *downshiftEnable
);

/******************************************************************************
 MTD_STATUS mtdAutonegGetNbaseTDownshiftStatus
 (
     IN MTD_DEV_PTR devPtr,
     IN MTD_U16 port,
     IN MTD_BOOL *downshiftFrom10G,
     IN MTD_BOOL *downshiftFrom5G,
     IN MTD_BOOL *downshiftFrom2P5G,
     IN MTD_BOOL *downshiftFrom1G,
     IN MTD_U16 *downshiftCount
 );

 
 Inputs:
    devPtr - pointer to MTD_DEV 
    port - MDIO port address, 0-31
    
 Outputs:
    downshiftFrom10G- MTD_TRUE if downshifted to lower speed from 10G
    downshiftFrom5G - MTD_TRUE if downshifted to lower speed from 5G
    downshiftFrom2P5G - MTD_TRUE if downshifted to lower speed from 2.5G
    downshiftFrom1G - MTD_TRUE if downshifted to lower speed from 1G
    downshiftCount - Number of link attempts at the current advertised
        speed.

 Returns:
    MTD_OK or MTD_FAIL, if action was successful or not

 Description:
    Reads the status from the downshift status register.
    

 Side effects:
    None

 Notes/Warnings:
    None

******************************************************************************/
MTD_STATUS mtdAutonegGetNbaseTDownshiftStatus
(
    IN MTD_DEV_PTR devPtr,
    IN MTD_U16 port,
    IN MTD_BOOL *downshiftFrom10G,
    IN MTD_BOOL *downshiftFrom5G,
    IN MTD_BOOL *downshiftFrom2P5G,
    IN MTD_BOOL *downshiftFrom1G,
    IN MTD_U16 *downshiftCount
);


/* Copper Interrupts */
#define MTD_SYMBOL_ERR_INTR           0x02000000
#define MTD_FALSE_CARR_INTR           0x01000000
#define MTD_TX_FIFO_OF_UF_INTR        0x00020000
#define MTD_JABBER_INTR               0x00010000 /* not available on X3540 */
#define MTD_AN_ERR_INTR               0x00008000
#define MTD_SPEED_CHG_INTR            0x00004000 /* Speed change between 2.5G, 5G and 10G will not generate an interrupt */
#define MTD_DX_CHG_INTR               0x00002000
#define MTD_PAGE_RX_INTR              0x00001000
#define MTD_AN_COMPLETED_INTR         0x00000800
#define MTD_LINK_STATUS_CHG_INTR      0x00000400
#define MTD_AN_MDI_CROS_CHG_INTR      0x00000040
#define MTD_DOWNSHIFT_INTR            0x00000020 /* not available on X3540 */
#define MTD_ENG_DETECT_INTR           0x00000010
#define MTD_FLP_XCHG_NO_LINK_INTR     0x00000008
#define MTD_POLARITY_CHG_INTR         0x00000002 /* not available on X3540 */
#define MTD_TUNIT_TEMP_SENSOR_INTR    0x00000001 /* This interrupt is not available on all PHYs */
#define MTD_ALL_TUNIT_INTR           (MTD_SYMBOL_ERR_INTR|MTD_FALSE_CARR_INTR|\
                                      MTD_TX_FIFO_OF_UF_INTR|MTD_JABBER_INTR|\
                                      MTD_AN_ERR_INTR|MTD_SPEED_CHG_INTR|\
                                      MTD_DX_CHG_INTR|MTD_PAGE_RX_INTR|\
                                      MTD_AN_COMPLETED_INTR|MTD_LINK_STATUS_CHG_INTR|\
                                      MTD_AN_MDI_CROS_CHG_INTR|MTD_DOWNSHIFT_INTR|\
                                      MTD_ENG_DETECT_INTR|MTD_FLP_XCHG_NO_LINK_INTR|\
                                      MTD_POLARITY_CHG_INTR|MTD_TUNIT_TEMP_SENSOR_INTR)


/******************************************************************************
MTD_STATUS mtdEnableDisableTunitInterrupts
(
    IN MTD_DEV_PTR devPtr,
    IN MTD_U16  port,
    IN MTD_U32  interrupt_sel,
    IN MTD_BOOL enable
);

 Inputs:
    devPtr - pointer to MTD_DEV initialized by mtdLoadDriver() call
    port   - port number, 0-31
    interrupt_sel - select single or multiple interrupts(OR together) from the list:
                    MTD_SYMBOL_ERR_INTR      
                    MTD_FALSE_CARR_INTR   
                    MTD_TX_FIFO_OV_UV_INTR   
                    MTD_JABBER_INTR             == not available on X3540
                    MTD_AN_ERR_INTR          
                    MTD_SPEED_CHG_INTR       
                    MTD_DX_CHG_INTR      
                    MTD_PAGE_RX_INTR         
                    MTD_AN_COMPLETED_INTR    
                    MTD_LINK_STATUS_CHG_INTR     
                    MTD_AN_MDI_CROS_CHG_INTR 
                    MTD_DOWNSHIFT_INTR          == not available on X3540
                    MTD_ENG_DETECT_INTR      
                    MTD_FLP_XCHG_NO_LINK_INTR
                    MTD_POLARITY_CHG_INTR       == not available on X3540
                    MTD_TUNIT_TEMP_SENSOR_INTR
                    MTD_ALL_TUNIT_INTR          

    enable - MTD_TRUE  = enable all selected interrupts
             MTD_FALSE = disable all selected interrupts

 Outputs:
    None

 Returns:
    MTD_OK or MTD_FAIL

 Description:
    Enable or disable all selected interrupts. 

 Side effects:
    None

 Notes/Warnings:
 Example:
 To enable MTD_SPEED_CHG_INTR and MTD_PAGE_RX_INTR interrupts:
   mtdEnableDisableTunitInterrupts(devPtr,port,(MTD_SPEED_CHG_INTR|MTD_PAGE_RX_INTR),\
                                   MTD_TRUE);
 To disable MTD_SPEED_CHG_INTR and MTD_PAGE_RX_INTR interrupts:
   mtdEnableDisableTunitInterrupts(devPtr,port,(MTD_SPEED_CHG_INTR|MTD_PAGE_RX_INTR),\
                                   MTD_FALSE);
******************************************************************************/
MTD_STATUS mtdEnableDisableTunitInterrupts
(
    IN MTD_DEV_PTR devPtr,
    IN MTD_U16  port,
    IN MTD_U32  interrupt_sel,
    IN MTD_BOOL enable
);

/******************************************************************************
MTD_STATUS mtdGetTunitInterruptsEnabled
(
    IN MTD_DEV_PTR devPtr,
    IN MTD_U16 port,
    OUT MTD_U32 *interruptEnabled
);

 Inputs:
    devPtr - pointer to MTD_DEV initialized by mtdLoadDriver() call
    port   - port number, 0-31

 Outputs:
    interruptEnabled - enabled interrupts:
                MTD_SYMBOL_ERR_INTR      
                MTD_FALSE_CARR_INTR   
                MTD_TX_FIFO_OV_UV_INTR   
                MTD_JABBER_INTR             == not available on X3540
                MTD_AN_ERR_INTR          
                MTD_SPEED_CHG_INTR       
                MTD_DX_CHG_INTR      
                MTD_PAGE_RX_INTR         
                MTD_AN_COMPLETED_INTR    
                MTD_LINK_STATUS_CHG_INTR     
                MTD_AN_MDI_CROS_CHG_INTR 
                MTD_DOWNSHIFT_INTR          == not available on X3540
                MTD_ENG_DETECT_INTR      
                MTD_FLP_XCHG_NO_LINK_INTR
                MTD_POLARITY_CHG_INTR       == not available on X3540
                MTD_TUNIT_TEMP_SENSOR_INTR
                MTD_ALL_TUNIT_INTR          

 Returns:
    MTD_OK or MTD_FAIL

 Description:
    Get Enabled or disabled interrupts. 

 Side effects:
    None

 Notes/Warnings:
******************************************************************************/
MTD_STATUS mtdGetTunitInterruptsEnabled
(
    IN MTD_DEV_PTR devPtr,
    IN MTD_U16  port,
    OUT MTD_U32 *interruptEnabled
);


/******************************************************************************
MTD_STATUS mtdGetTunitInterruptStatus
(
    IN MTD_DEV_PTR devPtr,
    IN MTD_U16 port,
    OUT MTD_U32 *interruptStatus
);

 Inputs:
    devPtr - pointer to MTD_DEV initialized by mtdLoadDriver() call
    port   - port number, 0-31

 Outputs:
    interruptStatus - 
                MTD_SYMBOL_ERR_INTR      
                MTD_FALSE_CARR_INTR      
                MTD_TX_FIFO_OF_UF_INTR   
                MTD_JABBER_INTR             == not available on X3540
                MTD_AN_ERR_INTR          
                MTD_SPEED_CHG_INTR       
                MTD_DX_CHG_INTR          
                MTD_PAGE_RX_INTR         
                MTD_AN_COMPLETED_INTR    
                MTD_LINK_STATUS_CHG_INTR     
                MTD_AN_MDI_CROS_CHG_INTR 
                MTD_DOWNSHIFT_INTR          == not available on X3540
                MTD_ENG_DETECT_INTR      
                MTD_FLP_XCHG_NO_LINK_INTR
                MTD_POLARITY_CHG_INTR       == not available on X3540
                MTD_TUNIT_TEMP_SENSOR_INTR
                
 Returns:
    MTD_OK or MTD_FAIL

 Description:
    Get T-unit interrupt status. 

 Side effects:
    None

 Notes/Warnings:
    The interrupts are cleared after this API call.
******************************************************************************/
MTD_STATUS mtdGetTunitInterruptStatus
(
    IN MTD_DEV_PTR devPtr,
    IN MTD_U16  port,
    OUT MTD_U32 *interruptStatus
);


/* Processor Interrupts (on newer phys) */
#define MTD_PROC_INTR_CTRL            MTD_T_UNIT_PMA_PMD,0xC006
#define MTD_PROC_INTR_STAUS           MTD_T_UNIT_PMA_PMD,0xC007
#define MTD_PROC_INTR_TOP             0x0002 /* Processor overall interrupt enable/disable */
#define MTD_ALL_PROC_INTR             0xFFC0 /* Bit 15:6 for valid interrupt source */

/* Only following chips support processor interrupt */
#define MTD_PROC_INTR_SUPPORT(mTddevPtr) (MTD_IS_X3540_FAMILY(mTddevPtr->deviceId) || \
                                          MTD_IS_X36X0_BASE(mTddevPtr->deviceId) || \
                                          MTD_IS_E2780_FAMILY(mTddevPtr->deviceId))

/*******************************************************************
MTD_STATUS mtdEnableDisableProcInterrupts
(
    IN MTD_DEV_PTR devPtr,
    IN MTD_U16  port,
    IN MTD_U32  interrupt_sel,
    IN MTD_BOOL enable
);

 Inputs:
    devPtr        - pointer to MTD_DEV initialized by mtdLoadDriver() call
    port          - port number, 0-31
    interrupt_sel - interrupt sources
                    Set bit 15:6 for enable/disable individual processor interrupt sources, please refer to specific 
                    device datasheet for detail.


    enable        - MTD_TRUE  = enable all selected interrupts
                    MTD_FALSE = disable all selected interrupts
 
 Outputs:
    None

 Returns:
    MTD_OK or MTD_FAIL

 Description:
    Enable or disable all selected processor interrupts.

 Side effects:
    None

 Notes/Warnings:
    None
 *******************************************************************/
MTD_STATUS mtdEnableDisableProcInterrupts
(
    IN MTD_DEV_PTR devPtr,
    IN MTD_U16  port,
    IN MTD_U16  interrupt_sel,
    IN MTD_BOOL enable
);

/*******************************************************************
MTD_STATUS mtdGetProcInterruptsEnabled
(
    IN MTD_DEV_PTR devPtr,
    IN MTD_U16 port,
    OUT MTD_U16 *interruptEnabled
);


 Inputs:
    devPtr - pointer to MTD_DEV initialized by mtdLoadDriver() call
    port   - port number, 0-31

 Outputs:
    interruptEnabled - enabled interrupts:
                       value of bit 15:6, b'1 = enabled, b'0 = disabled
                       please refer to specific device datasheet for detail interrupt sources definition.

 Returns:
    MTD_OK or MTD_FAIL

 Description:
    Get Enabled or disabled processor interrupts.

 Side effects:
    None

 Notes/Warnings:
    None

 *******************************************************************/
MTD_STATUS mtdGetProcInterruptsEnabled
(
    IN MTD_DEV_PTR devPtr,
    IN MTD_U16 port,
    OUT MTD_U16 *interruptEnabled
);

/*******************************************************************
MTD_STATUS mtdGetProcInterruptStatus
(
    IN MTD_DEV_PTR devPtr,
    IN MTD_U16 port,
    OUT MTD_U16 *interruptStatus
);

 Inputs:
    devPtr - pointer to MTD_DEV initialized by mtdLoadDriver() call
    port   - port number, 0-31

 Outputs:
    interruptStatus - interrupts triggered:
                      value of interrupt status register. b'1 = triggered, b'0 = not triggered
                      please refer to specific device datasheet for detail interrupt sources definition.

 Returns:
    MTD_OK or MTD_FAIL

 Description:
    Get processor interrupt status. 

 Side effects:
    None

 Notes/Warnings:
    This register is read to clear

 *******************************************************************/
MTD_STATUS mtdGetProcInterruptStatus
(
    IN MTD_DEV_PTR devPtr,
    IN MTD_U16 port,
    OUT MTD_U16 *interruptStatus
);


/******************************************************************************
 Functions shared between H unit and X unit for 1000BX/SGMII
******************************************************************************/

/* defines for bit 6,13 in SGMII control register */
#define MTD_SGMII_SPEED_10M   0
#define MTD_SGMII_SPEED_100M  1
#define MTD_SGMII_SPEED_1G    2
#define MTD_SGMII_SPEED_ASIS  3 /* to leave the setting as it is */
/******************************************************************************
 MTD_STATUS mtdSet1000BXSGMIIControl
 (
     IN MTD_DEV_PTR devPtr,
     IN MTD_U16 port,
     IN MTD_U16 HorXunit,
     IN MTD_BOOL loopback,
     IN MTD_U16 speed,
     IN MTD_BOOL an_enable,
     IN MTD_BOOL power_down,
     IN MTD_BOOL restart_an,
     IN MTD_BOOL sw_reset
 );

 Inputs:
    devPtr - pointer to MTD_DEV 
    port   - MDIO port address, 0-31
    HorXunit - MTD_H_UNIT or MTD_X_UNIT, depending on which interface is
                being configured
    loopback - MTD_TRUE to set to loopback, MTD_FALSE to undo loopback
    speed - one of the following (only applies when AN is disabled)
        MTD_SGMII_SPEED_10M
        MTD_SGMII_SPEED_100M  
        MTD_SGMII_SPEED_1G    
    an_enable - MTD_TRUE to enable autonegotiation, MTD_FALSE to disable it
                NOTE: Toggling this setting will cause link to go down
                and restart AN bit 9 automatically will get set to 1
    power_down - MTD_TRUE to power down all PCS/PMA of this interface, MTD_FALSE
                 otherwise
    restart_an - MTD_TRUE to restart the SGMII autonegotiation, MTD_FALSE otherwise,
                 this bit self clears
    sw_reset - MTD_TRUE to do a software reset on this interface, MTD_FALSE otherwise,
               this bit self clears           

 Outputs:
    None

 Returns:
    MTD_OK or MTD_FAIL if some error occurred

 Description:
    This is a multi-purpose control function for controlling the H Unit SGMII
    interface or the X Unit SGMII interface. Most parameters of this function
    should only be used individually. Using this function, you may

        1. Disable autoneg or Enable autoneg (changing this automatically restarts an) (an_enable)
        2. Restart autoneg (restart_an)
        3. Set the SGMII speed (which only takes effect if AN is disabled) (speed)
        4. Put the SGMII into loopback (loopback)
        5. Power down the SGMII interface (power_down)
        6. Perform a software reset on the SGMII MMD

 Side effects:
    None

 Notes/Warnings:
    Call mtdGet1000BXSGMIIControl() to get the current settings. If 
    software reset is set to MTD_TRUE, the software reset will be performed 
    last.

    This call is also used for 2.5G operation.

******************************************************************************/
MTD_STATUS mtdSet1000BXSGMIIControl
(
    IN MTD_DEV_PTR devPtr,
    IN MTD_U16 port,
    IN MTD_U16 HorXunit,
    IN MTD_BOOL loopback,
    IN MTD_U16 speed,
    IN MTD_BOOL an_enable,
    IN MTD_BOOL power_down,
    IN MTD_BOOL restart_an,
    IN MTD_BOOL sw_reset
);


/******************************************************************************
 MTD_STATUS mtdGet1000BXSGMIIControl
 (
     IN MTD_DEV_PTR devPtr,
     IN MTD_U16 port,
     IN MTD_U16 HorXunit,
     OUT MTD_BOOL *loopback,
     OUT MTD_U16 *speed,
     OUT MTD_BOOL *an_enable,
     OUT MTD_BOOL *power_down
 );


 Inputs:
    devPtr - pointer to MTD_DEV 
    port   - MDIO port address, 0-31
    HorXunit - MTD_H_UNIT or MTD_X_UNIT, depending on which interface is
                being queried

 Outputs:
    loopback - MTD_TRUE = loopback enabled, MTD_FALSE to otherwise
    speed - one of the following (only applies when AN is disabled)
        MTD_SGMII_SPEED_10M
        MTD_SGMII_SPEED_100M  
        MTD_SGMII_SPEED_1G    
    an_enable - MTD_TRUE autonegotiation enabled, MTD_FALSE otherwise
    power_down - MTD_TRUE if powered down, MTD_FALSE otherwise

 Returns:
    MTD_OK or MTD_FAIL if some error occurred

 Description:
    This function may be called to query the state of the bits in 3.2000
    (X unit SGMII control register) or 4.2000 (H unit SGMII control register).

 Side effects:
    None

 Notes/Warnings:
    Call mtdSet1000BXSGMIIControl() to change the current settings.
    
    This call is also used for 2.5G operation on 88X33X0/E20X0/E21X0 PHYs.

******************************************************************************/
MTD_STATUS mtdGet1000BXSGMIIControl
(
    IN MTD_DEV_PTR devPtr,
    IN MTD_U16 port,
    IN MTD_U16 HorXunit,
    OUT MTD_BOOL *loopback,
    OUT MTD_U16 *speed,
    OUT MTD_BOOL *an_enable,
    OUT MTD_BOOL *power_down
);

/******************************************************************************
 MTD_STATUS mtdGet1000BXSGMIIStatus
 (
     IN MTD_DEV_PTR devPtr,
     IN MTD_U16 port,
     IN MTD_U16 HorXunit,
     OUT MTD_BOOL *an_complete,
     OUT MTD_BOOL *remote_fault,
     OUT MTD_BOOL *link_status_latched,
     OUT MTD_BOOL *link_status_current
 );

 Inputs:
    devPtr - pointer to MTD_DEV 
    port   - MDIO port address, 0-31
    HorXunit - MTD_H_UNIT or MTD_X_UNIT, depending on which interface is
                being queried    

 Outputs:
    an_complete - MTD_TRUE if autonegotiation has completed (MTD_FALSE
                  indicates link down, autoneg disabled or link up due
                  to autoneg bypassed)
    remote_fault - MTD_TRUE if remote fault condition detected, MTD_FALSE
                   in SGMII mode, or no remote fault condition detected
                   (this parameter only valid in 1000BX mode)
    link_status_latched - MTD_FALSE, indicates link was lost since last read.
                  This bit is a latched low bit. This is first read
                  value.
    link_status_current - MTD_FALSE, indicates link is currently down. 
                  This bit is a latched low bit. This is second read of
                  register.
                  

 Returns:
    MTD_OK or MTD_FAIL

 Description:
    Queries the status register in 3.2001 (X unit 1000BX/SGMII Status)
    or 4.2001 (H unit SGMII Status) and returns the value of bits:
    5: autoneg complete (MTD_TRUE if link up and autoneg has completed)
    4: remote fault (1000BX only mode valid, always MTD_FALSE in SGMII mode)
    2: link status (value of first read of register returned in
       link_status_latched, value of second read returned in link_status_current)
    
    

 Side effects:
    None

 Notes/Warnings:
    The SGMII status is not valid unless the host interface is up at
    a speed of 1G/100M or 10M. Check the media side link up and speed
    before calling this function.

    This means if the media side link is down and the host link down
    speed is set to 10G, the host SGMII link status returned by this function
    is not valid and should be ignored.

    This call is also used for 2.5G operation.   

******************************************************************************/
MTD_STATUS mtdGet1000BXSGMIIStatus
(
    IN MTD_DEV_PTR devPtr,
    IN MTD_U16 port,
    IN MTD_U16 HorXunit,
    OUT MTD_BOOL *an_complete,
    OUT MTD_BOOL *remote_fault,
    OUT MTD_BOOL *link_status_latched,
    OUT MTD_BOOL *link_status_current
);

/******************************************************************************
 Functions shared between H unit and X unit for 10GBASE-R PCS
 XFI
 SFI
 USXGMII
 All use H unit or X unit 10GBASE-R PCS
******************************************************************************/

/******************************************************************************
 MTD_STATUS mtdUSXGMIIAutoNeg
 (
     IN MTD_DEV_PTR devPtr,
     IN MTD_U16 port,
     IN MTD_U16 enableAutoNeg,
     IN MTD_BOOL do10GBRSwReset
 );

 Inputs:
    devPtr - pointer to MTD_DEV initialized by mtdLoadDriver() call
    port   - MDIO port address, 0-31
    enableAutoNeg - MTD_ENABLE or MTD_DISABLE the USXGMII Auto-Neg
    do10GBRSwReset - MTD_TRUE if a 10GBASE-R software reset should be done after change
                     has been made, or MTD_FALSE otherwise. See note below.
 Outputs:
    None

 Returns:
    MTD_OK or MTD_FAIL

 Description:
    USXGMII AutoNeg is enabled by default after link-up.
    This API enables or disables the Auto-Neg for USXGMII Mac type at the
    H-Unit side.

    For X33X0 and E21X0 family, a PCS software reset(4.1000.15) is needed to apply the
    changes immediately. If do10GBRSwReset is set to MTD_FALSE, you need to perform
    a software reset later or re-train the link to make the change become active.

    You don't have to set the do10GBRSwReset to MTD_TRUE for other PHYs.

 Side effects:
    None

 Notes/Warnings:
    Should only call when the device is configured for the USXGMII Mac Type
******************************************************************************/
MTD_STATUS mtdUSXGMIIAutoNeg
(
    IN MTD_DEV_PTR devPtr,
    IN MTD_U16 port,
    IN MTD_U16 enableAutoNeg,
    IN MTD_BOOL do10GBRSwReset
);

/******************************************************************************
 MTD_STATUS mtdSetUSXGMIIControl or
 MTD_STATUS mtdSet10GBRControl 
 (
     IN MTD_DEV_PTR devPtr,
     IN MTD_U16 port,
     IN MTD_U16 HorXunit,
     IN MTD_BOOL loopback,
     IN MTD_BOOL power_down,
     IN MTD_BOOL sw_reset
 );

 Inputs:
    devPtr - pointer to MTD_DEV 
    port   - MDIO port address, 0-31
    HorXunit - MTD_H_UNIT or MTD_X_UNIT, depending on which interface is
                being configured
    loopback - MTD_TRUE to set to loopback, MTD_FALSE to undo loopback
    power_down - MTD_TRUE to power down all PCS/PMA of this interface, MTD_FALSE
                 otherwise
    sw_reset - MTD_TRUE to do a software reset on this interface, MTD_FALSE otherwise,
               this bit self clears           
    

 Outputs:
    None

 Returns:
    MTD_OK or MTD_FAIL

 Description:
    This function is used to control the 10GBASE-R PCS interface on either the
    H unit (MAC interface) or the X unit (fiber interface) on the 88X3240.
    The PCS may be put in loopback, powered down, or a software reset may 
    be performed. The software reset bit is self clearing.

 Side effects:
    None

 Notes/Warnings:
    Use mtdGet10GBRControl() to read the current value of the loopback
    and power_down bits. In registers 3.1000 or 4.1000.

    This call is also used for 5G operation.
    
******************************************************************************/
MTD_STATUS mtdSet10GBRControl
(
    IN MTD_DEV_PTR devPtr,
    IN MTD_U16 port,
    IN MTD_U16 HorXunit,
    IN MTD_BOOL loopback,
    IN MTD_BOOL power_down,
    IN MTD_BOOL sw_reset
);
#define mtdSetUSXGMIIControl mtdSet10GBRControl

/******************************************************************************
 MTD_STATUS mtdGetUSXGMIIControl
 MTD_STATUS mtdGet10GBRControl
 (
     IN MTD_DEV_PTR devPtr,
     IN MTD_U16 port,
     IN MTD_U16 HorXunit,
     OUT MTD_BOOL *loopback,
     OUT MTD_BOOL *power_down,
 );

 Inputs:
    devPtr - pointer to MTD_DEV 
    port   - MDIO port address, 0-31
    HorXunit - MTD_H_UNIT or MTD_X_UNIT, depending on which interface is
                being read
 Outputs:
    loopback - MTD_TRUE if loopback is set, MTD_FALSE if not
    power_down - MTD_TRUE if PCS/PMA is powered down, MTD_FALSE
                 otherwise

 Returns:
    MTD_OK or MTD_FAIL

 Description:
    This function is used to read the values of bits in the 10GBASE-R
    PCS control register of the H or X unit.

 Side effects:
    None

 Notes/Warnings:
    Use mtdSet10GBRControl() to change the settings on the 10GBASE-R PCS
    control register, 3.1000 or 4.1000.

    This call is also used for 5G operation.    

******************************************************************************/
MTD_STATUS mtdGet10GBRControl
(
    IN MTD_DEV_PTR devPtr,
    IN MTD_U16 port,
    IN MTD_U16 HorXunit,
    OUT MTD_BOOL *loopback,
    OUT MTD_BOOL *power_down
);
#define mtdGetUSXGMIIControl mtdGet10GBRControl

/******************************************************************************
 MTD_STATUS mtdGetUSXGMIIStatus1 or
 MTD_STATUS mtdGet10GBRStatus1
 (
     IN MTD_DEV_PTR devPtr,
     IN MTD_U16 port,
     IN MTD_U16 HorXunit,
     OUT MTD_BOOL *tx_lpi_latch,
     OUT MTD_BOOL *rx_lpi_latch,
     OUT MTD_BOOL *tx_lpi_current,
     OUT MTD_BOOL *rx_lpi_current,
     OUT MTD_BOOL *fault,
     OUT MTD_BOOL *link_status_latch,
     OUT MTD_BOOL *link_status_current    
 );

 Inputs:
    devPtr - pointer to MTD_DEV 
    port   - MDIO port address, 0-31
    HorXunit - MTD_H_UNIT or MTD_X_UNIT, depending on which interface is
               being read
 Outputs:
    tx_lpi_latch - value of bit 11, MTD_TRUE if Tx PCS has received LPI
                   since last read of this bit, MTD_FALSE if not
    rx_lpi_latch- value of bit 10, MTD_TRUE if Rx PCS has received LPI
                   since last read of this bit, MTD_FALSE if not
    tx_lpi_current - value of bit 9, MTD_TRUE if Tx PCS is receiving LPI
                     currently, MTD_FALSE if not
    rx_lpi_current - value of bit 8, MTD_TRUE if Rx PCS is receiving LPI
                     currently, MTD_FALSE if not
    fault - value of bit 7, MTD_TRUE if a fault is indicated by 
            bit 10 or 11 in register 0x1008, MTD_FALSE otherwise
    link_status_latch - first read value of bit 2, which is a latched
            low bit. MTD_FALSE if PCS has had a link down since last
            time this register was read, MTD_TRUE if no link down
            has occurred.
    link_status_current - second read of bit 2. MTD_TRUE if link is
            currently up, MTD_FALSE if link is currently down    

 Returns:
    MTD_OK or MTD_FAIL

 Description:
    This function returns the value of status bits in the 10GBASE-R PCS
    STATUS 1 Register in either the H unit or X unit (4.1001 or 3.1001).    

 Side effects:
    None

 Notes/Warnings:
    Latch bits may be cleared when calling this register if the underlying
    condition has changed.

    Clearing fault requires reading register 0x1008.

    This call is also used for 5G operation.   

******************************************************************************/
MTD_STATUS mtdGet10GBRStatus1
(
    IN MTD_DEV_PTR devPtr,
    IN MTD_U16 port,
    IN MTD_U16 HorXunit,
    OUT MTD_BOOL *tx_lpi_latch,
    OUT MTD_BOOL *rx_lpi_latch,
    OUT MTD_BOOL *tx_lpi_current,
    OUT MTD_BOOL *rx_lpi_current,
    OUT MTD_BOOL *fault,
    OUT MTD_BOOL *link_status_latch,
    OUT MTD_BOOL *link_status_current    
);
#define mtdGetUSXGMIIStatus1 mtdGet10GBRStatus1


/******************************************************************************
 MTD_STATUS mtdGetUSXGMIIFault or
 MTD_STATUS mtdGet10GBRFault
 (
     IN MTD_DEV_PTR devPtr,
     IN MTD_U16 port,
     IN MTD_U16 HorXunit,         
     OUT MTD_BOOL *tx_fault_latch,
     OUT MTD_BOOL *rx_fault_latch,
     OUT MTD_BOOL *tx_fault_current,
     OUT MTD_BOOL *rx_fault_current
 );

 Inputs:
    devPtr - pointer to MTD_DEV 
    port   - MDIO port address, 0-31
    HorXunit - MTD_H_UNIT or MTD_X_UNIT, depending on which interface is
               being read
               
 Outputs:
    tx_fault_latch - MTD_TRUE if there was a fault on the tx path since the last
                     call of this function, MTD_FALSE if not (bit 11)
    rx_fault_latch - MTD_TRUE if there was a fault on the rx path since the last
                     call of this function, MTD_FALSE if not (bit 10)
    tx_fault_current - MTD_TRUE if there is currently a fault on the tx path, 
                       MTD_FALSE if not (bit 11)
    rx_fault_current - MTD_TRUE if there is currently a fault on the rx path, 
                       MTD_FALSE if not (bit 10)
                                          
 Returns:
    MTD_OK or MTD_FAIL

 Description:
    Reads the Status 2 register in either the H unit or X unit twice and
    returns the tx/rx fault latched value (first read) in the latch variables
    and the current tx/rx fault value (second read) in the current variables.

    Latched variables indicate if there was a fault since the last read, 
    whereas current variables indicate if there is currently a tx/rx fault.

 Side effects:
    None

 Notes/Warnings:
    Reading this register will clear the fault bit in register 0x1001 if
    the fault condition has cleared.

    This call is also used for 5G operation.  

******************************************************************************/
MTD_STATUS mtdGet10GBRFault
(
    IN MTD_DEV_PTR devPtr,
    IN MTD_U16 port,
    IN MTD_U16 HorXunit,    
    OUT MTD_BOOL *tx_fault_latch,
    OUT MTD_BOOL *rx_fault_latch,
    OUT MTD_BOOL *tx_fault_current,
    OUT MTD_BOOL *rx_fault_current
);
#define mtdGetUSXGMIIFault mtdGet10GBRFault


/******************************************************************************
 MTD_STATUS mtdGetUSXGMIIReceiveStatus or
 MTD_STATUS mtdGet10GBRReceiveStatus
 (
     IN MTD_DEV_PTR devPtr,
     IN MTD_U16 port,
     IN MTD_U16 HorXunit, 
     OUT MTD_BOOL *link_up,
     OUT MTD_BOOL *high_ber,
     OUT MTD_BOOL *block_lock    
 );

 Inputs:
    devPtr - pointer to MTD_DEV 
    port   - MDIO port address, 0-31
    HorXunit - MTD_H_UNIT or MTD_X_UNIT, depending on which interface is
               being read    

 Outputs:
    link_up - MTD_TRUE if receive link is up, MTD_FALSE if down (bit 12)
    high_ber - MTD_TRUE if receive link is reporting high bit error rate,
               MTD_FALSE if not (bit 1)
    block_lock - MTD_TRUE if receive link is locked to received block,
                 MTD_FALSE if not (bit 0)

 Returns:
    MTD_OK or MTD_FAIL

 Description:
    Reads register 0x1020 in either the H unit or X unit and returns the
    flags above reporting the status of the 10GBASE-R PCS Receive path.

 Side effects:
    None

 Notes/Warnings:
    This call is also used for 5G operation.

******************************************************************************/
MTD_STATUS mtdGet10GBRReceiveStatus
(
    IN MTD_DEV_PTR devPtr,
    IN MTD_U16 port,
    IN MTD_U16 HorXunit, 
    OUT MTD_BOOL *link_up,
    OUT MTD_BOOL *high_ber,
    OUT MTD_BOOL *block_lock    
);
#define mtdGetUSXGMIIReceiveStatus mtdGet10GBRReceiveStatus


/******************************************************************************
 MTD_STATUS mtdGetUSXGMIIStatus2 or
 MTD_STATUS mtdGet10GBRStatus2
 (
     IN MTD_DEV_PTR devPtr,
     IN MTD_U16 port,
     IN MTD_U16 HorXunit, 
     OUT MTD_BOOL *has_block_lock,
     OUT MTD_BOOL *reported_high_ber,
     OUT MTD_U8 *ber_counter,
     OUT MTD_U8 *errored_blocks_counter    
 );

 Inputs:
    devPtr - pointer to MTD_DEV 
    port   - MDIO port address, 0-31
    HorXunit - MTD_H_UNIT or MTD_X_UNIT, depending on which interface is
               being read    
   
 Outputs:
    has_block_lock - MTD_FALSE if block lock was lost since the last time
                      this function was called (link encountered rx
                      errors to the point of losing lock), MTD_TRUE
                      if block lock was not lost since last time
                      this function was called.
    reported_high_ber - MTD_TRUE if a high ber condition was reported
                        since the last time this function was called,
                        MTD_FALSE if not (bit14)
    ber_counter - Number of bit errors since last read (0xFF indicates
                  counter saturated) (bits 13:8)
    errored_blocks_counter - Number of errored blocks since last read
                  (0xFF indicates counter saturated) (bits 7:0)

 Returns:
    MTD_OK or MTD_FAIL

 Description:
    This function reads register 0x1021 in either the H unit or X unit
    and returns the above values. Latched bits are cleared if the 
    condition has changed and counters are cleared when read.

    This function is useful to check if there have been errors on
    the rx path since the last time this function was called and how many
    errors may have occurred.

    Call once at the beginning of the period desired to monitor to clear
    the latched bits and counters, and then call again at the end of the
    period to check if any errors occurred during the period.

 Side effects:
    None

 Notes/Warnings:
    Calling this function clears the latched bits in register 0x1021 and
    the counters.

    Current values are not provided in this function. Current values
    for block lock and high ber can be fetched by calling 
    mtdGet10GBRReceiveStatus().

    This call is also used for 5G operation.   

******************************************************************************/
MTD_STATUS mtdGet10GBRStatus2
(
    IN MTD_DEV_PTR devPtr,
    IN MTD_U16 port,
    IN MTD_U16 HorXunit, 
    OUT MTD_BOOL *has_block_lock,
    OUT MTD_BOOL *reported_high_ber,
    OUT MTD_U8 *ber_counter,
    OUT MTD_U8 *errored_blocks_counter    
);
#define mtdGetUSXGMIIStatus2 mtdGet10GBRStatus2

/* H and X unit interrupt. Note: PHYs after X33X0 have no X unit */
/* Interrupts - 5G/10GB-R */
#define MTD_BASER_LOCAL_FAULT_TX_INTR     0x0800
#define MTD_BASER_LOCAL_FAULT_RX_INTR     0x0400
#define MTD_BASER_LINK_STATUS_CHG_INTR    0x0004
#define MTD_BASER_HIGH_BER_CHG_INTR       0x0002
#define MTD_BASER_BLK_LOCK_CHG_INTR       0x0001
#define MTD_ALL_BASER_INTR               (MTD_BASER_LOCAL_FAULT_TX_INTR|MTD_BASER_LOCAL_FAULT_RX_INTR|\
                                          MTD_BASER_LINK_STATUS_CHG_INTR|MTD_BASER_HIGH_BER_CHG_INTR|\
                                          MTD_BASER_BLK_LOCK_CHG_INTR)
/* Realtime status - 5G/10GB-R */
#define MTD_LOCAL_FAULT_TX_STATUS         0x0800
#define MTD_LOCAL_FAULT_RX_STATUS         0x0400
#define MTD_JIT_0_LOCK_STATUS             0x0010
#define MTD_JIT_IF_LOCK_STATUS            0x0008
#define MTD_R_LINK_STATUS                 0x0004
#define MTD_HIGH_BER_STATUS               0x0002
#define MTD_BLK_LOCK_STATUS               0x0001

/******************************************************************************
MTD_STATUS mtdEnableDisableBaseRInterrupts
(
    IN MTD_DEV_PTR devPtr,
    IN MTD_U16  port,
    IN MTD_U16  HorXunit,
    IN MTD_U16  interrupt_sel,
    IN MTD_BOOL enable
);

 Inputs:
    devPtr - pointer to MTD_DEV 
    port   - port number, 0-31
    HorXunit - MTD_H_UNIT or MTD_X_UNIT, depending on which interface is
               being read
    interrupt_sel - select single or multiple interrupts(OR together) from the list:
                    MTD_BASER_LOCAL_FAULT_TX_INTR 
                    MTD_BASER_LOCAL_FAULT_RX_INTR 
                    MTD_BASER_LINK_STATUS_CHG_INTR
                    MTD_BASER_HIGH_BER_CHG_INTR   
                    MTD_BASER_BLK_LOCK_CHG_INTR   
                    MTD_ALL_BASER_INTR            

     enable - MTD_TRUE  = enable all selected interrupts
              MTD_FALSE = disable all selected interrupts

 Outputs:
    None

 Returns:
    MTD_OK or MTD_FAIL

 Description:
    Enable or disable all selected interrupts. 

 Side effects:
    None

 Notes/Warnings:
    No X unit devices after X3310/40.. Set HorXunit to MTD_X_UNIT on these 
    devices will return MTD_FAIL.

 Example:
 To enable MTD_BASER_HIGH_BER_CHG_INTR and MTD_BASER_BLK_LOCK_CHG_INTR interrupts:
   mtdEnableDisableBaseRInterrupts(devPtr,port,(MTD_BASER_HIGH_BER_CHG_INTR|MTD_BASER_BLK_LOCK_CHG_INTR),\
                                   MTD_TRUE);
 To disable MTD_BASER_HIGH_BER_CHG_INTR and MTD_BASER_BLK_LOCK_CHG_INTR interrupts:
   mtdEnableDisableBaseRInterrupts(devPtr,port,(MTD_BASER_HIGH_BER_CHG_INTR|MTD_BASER_BLK_LOCK_CHG_INTR),\
                                   MTD_FALSE);
******************************************************************************/
MTD_STATUS mtdEnableDisableBaseRInterrupts
(
    IN MTD_DEV_PTR devPtr,
    IN MTD_U16  port,
    IN MTD_U16  HorXunit,
    IN MTD_U16  interrupt_sel,
    IN MTD_BOOL enable
);

/******************************************************************************
MTD_STATUS mtdGetBaseRInterruptsEnabled
(
    IN MTD_DEV_PTR devPtr,
    IN MTD_U16   port,
    IN MTD_U16   HorXunit,
    OUT MTD_U16 *interruptEnabled
);

 Inputs:
    devPtr - pointer to MTD_DEV 
    port   - port number, 0-31
    HorXunit - MTD_H_UNIT or MTD_X_UNIT, depending on which interface is
               being read
 Outputs:
    interruptEnabled - enabled interrupts:
                MTD_BASER_LOCAL_FAULT_TX_INTR 
                MTD_BASER_LOCAL_FAULT_RX_INTR 
                MTD_BASER_LINK_STATUS_CHG_INTR
                MTD_BASER_HIGH_BER_CHG_INTR   
                MTD_BASER_BLK_LOCK_CHG_INTR   
                MTD_ALL_BASER_INTR

 Returns:
    MTD_OK or MTD_FAIL

 Description:
    Get enabled 10GBASE-R interrupts. 

 Side effects:
    None

 Notes/Warnings:
    No X unit devices after X3310/40.. Set HorXunit to MTD_X_UNIT on these 
    devices will return MTD_FAIL.
******************************************************************************/
MTD_STATUS mtdGetBaseRInterruptsEnabled
(
    IN MTD_DEV_PTR devPtr,
    IN MTD_U16   port,
    IN MTD_U16   HorXunit,
    OUT MTD_U16 *interruptEnabled
);


/******************************************************************************
MTD_STATUS mtdGetBaseRInterruptStatus
(
    IN MTD_DEV_PTR devPtr,
    IN MTD_U16   port,
    IN MTD_U16   HorXunit,
    OUT MTD_U16 *interruptStatus
);

 Inputs:
    devPtr - pointer to MTD_DEV 
    port   - port number, 0-31
    HorXunit - MTD_H_UNIT or MTD_X_UNIT, depending on which interface is
               being read

 Outputs:
    interruptStatus - 
             MTD_BASER_LOCAL_FAULT_TX_INTR 
             MTD_BASER_LOCAL_FAULT_RX_INTR 
             MTD_BASER_LINK_STATUS_CHG_INTR
             MTD_BASER_HIGH_BER_CHG_INTR   
             MTD_BASER_BLK_LOCK_CHG_INTR   

 Returns:
    MTD_OK or MTD_FAIL

 Description:
    Get 10GBASE-R interrupts status. The interrupts are cleared after this API call.

 Side effects:
    None

 Notes/Warnings:
    The interrupts are cleared after this API call. 
    No X unit devices after X3310/40.. Set HorXunit to MTD_X_UNIT on these 
    devices will return MTD_FAIL.
******************************************************************************/
MTD_STATUS mtdGetBaseRInterruptStatus
(
    IN MTD_DEV_PTR devPtr,
    IN MTD_U16   port,
    IN MTD_U16   HorXunit,
    OUT MTD_U16 *interruptStatus
);

/******************************************************************************
MTD_STATUS mtdGetBaseRrealTimeStatus
(
    IN MTD_DEV_PTR devPtr,
    IN MTD_U16   port,
    IN MTD_U16   HorXunit,
    OUT MTD_U16 *realTimeStatus
);

 Inputs:
    devPtr   - pointer to MTD_DEV 
    port     - port number, 0-31
    HorXunit - MTD_H_UNIT or MTD_X_UNIT, depending on which interface is
               being read
 Outputs:
    realTimeStatus - 
             MTD_LOCAL_FAULT_TX_STATUS
             MTD_LOCAL_FAULT_RX_STATUS
             MTD_JIT_0_LOCK_STATUS    
             MTD_JIT_1_LOCK_STATUS    
             MTD_R_LINK_STATUS          
             MTD_HIGH_BER_STATUS      
             MTD_BLK_LOCK_STATUS                 

 Returns:
    MTD_OK or MTD_FAIL

 Description:
    Get 10GBASE-R realtime status. 

 Side effects:
    None

 Notes/Warnings:
    No X unit devices after X3310/40.. Set HorXunit to MTD_X_UNIT on these 
    devices will return MTD_FAIL.
******************************************************************************/
MTD_STATUS mtdGetBaseRrealTimeStatus
(
    IN MTD_DEV_PTR devPtr,
    IN MTD_U16   port,
    IN MTD_U16   HorXunit,
    OUT MTD_U16 *realTimeStatus
);


/* Interrupts- 1G/2.5GBASE-X for the X-unit*/
/* Interrupts- 1G/2.5GBASE-X/SGMII for the H-unit*/
#define MTD_BASEX_SPEED_CHG_INTR           0x4000
#define MTD_BASEX_DX_CHG_INTR              0x2000
#define MTD_BASEX_PAGE_RX_INTR             0x1000
#define MTD_BASEX_AN_COMPLETED_INTR        0x0800
#define MTD_BASEX_LINK_UP_TO_DN_INTR       0x0400
#define MTD_BASEX_LINK_DN_TO_UP_INTR       0x0200
#define MTD_BASEX_SYMBOL_ERR_INTR          0x0100
#define MTD_BASEX_FALSE_CARR_INTR          0x0080
#define MTD_ALL_BASEX_INTR                (MTD_BASEX_SPEED_CHG_INTR|MTD_BASEX_DX_CHG_INTR|\
                                           MTD_BASEX_PAGE_RX_INTR|MTD_BASEX_AN_COMPLETED_INTR|\
                                           MTD_BASEX_LINK_UP_TO_DN_INTR|MTD_BASEX_LINK_DN_TO_UP_INTR|\
                                           MTD_BASEX_SYMBOL_ERR_INTR|MTD_BASEX_FALSE_CARR_INTR)
/* Realtime Status- 1G/2.5GBASE-X */
#define MTD_RT_SPEED_STATUS                0xC000  /* 0b10: 1000Mbps, 0b01: 100Mbps, 0b00: 10Mbps */
#define MTD_RT_DX_STATUS                   0x2000
#define MTD_RT_PAGE_RX_STATUS              0x1000
#define MTD_RT_SPEED_DX_RESOLVED           0x0800
#define MTD_RT_LINK_STATUS                 0x0400
#define MTD_RT_SYNC_STATUS                 0x0020
#define MTD_RT_ENG_DETECT_STATUS           0x0010
#define MTD_RT_TX_PAUSE_STATUS             0x0008
#define MTD_RT_RX_PAUSE_STATUS             0x0004
                                          
/******************************************************************************
MTD_STATUS mtdEnableDisableBaseXInterrupts
(
    IN MTD_DEV_PTR devPtr,
    IN MTD_U16  port,
    IN MTD_U16  HorXunit,
    IN MTD_U16  interrupt_sel,
    IN MTD_BOOL enable
);

 Inputs:
    devPtr   - pointer to MTD_DEV 
    port     - port number, 0-31
    HorXunit - MTD_H_UNIT or MTD_X_UNIT, depending on which interface is
               being read
    interrupt_sel - select single or multiple interrupts(OR together) from the list:
                    MTD_BASEX_SPEED_CHG_INTR    
                    MTD_BASEX_DX_CHG_INTR       
                    MTD_BASEX_PAGE_RX_INTR      
                    MTD_BASEX_AN_COMPLETED_INTR 
                    MTD_BASEX_LINK_UP_TO_DN_INTR
                    MTD_BASEX_LINK_DN_TO_UP_INTR
                    MTD_BASEX_SYMBOL_ERR_INTR   
                    MTD_BASEX_FALSE_CARR_INTR   
                    MTD_ALL_BASEX_INTR          

     enable - MTD_TRUE  = enable all selected interrupts
              MTD_FALSE = disable all selected interrupts

 Outputs:
    None

 Returns:
    MTD_OK or MTD_FAIL

 Description:
    Enable or disable all selected interrupts. 

 Side effects:
    None

 Notes/Warnings:
    No X unit after X3320/40 devices. Set HorXunit to MTD_X_UNIT on these 
    devices will return MTD_FAIL.

 Example:
 To enable MTD_BASEX_SPEED_CHG_INTR and MTD_BASEX_DX_CHG_INTR interrupts:
   mtdEnableDisableBaseXInterrupts(devPtr,port,(MTD_BASEX_SPEED_CHG_INTR|MTD_BASEX_DX_CHG_INTR),\
                                  MTD_TRUE);
 To disable MTD_BASEX_SPEED_CHG_INTR and MTD_BASEX_DX_CHG_INTR interrupts:
   mtdEnableDisableBaseXInterrupts(devPtr,port,(MTD_BASEX_SPEED_CHG_INTR|MTD_BASEX_DX_CHG_INTR),\
                                  MTD_FALSE);
******************************************************************************/
MTD_STATUS mtdEnableDisableBaseXInterrupts
(
    IN MTD_DEV_PTR devPtr,
    IN MTD_U16  port,
    IN MTD_U16  HorXunit,
    IN MTD_U16  interrupt_sel,
    IN MTD_BOOL enable
);

/******************************************************************************
MTD_STATUS mtdGetBaseXInterruptsEnabled
(
    IN MTD_DEV_PTR devPtr,
    IN MTD_U16   port,
    IN MTD_U16   HorXunit,
    OUT MTD_U16 *interruptEnabled
);

 Inputs:
    devPtr   - pointer to MTD_DEV 
    port     - port number, 0-31
    HorXunit - MTD_H_UNIT or MTD_X_UNIT, depending on which interface is
               being read
 Outputs:
    interruptEnabled - enabled interrupts:
                MTD_BASEX_SPEED_CHG_INTR    
                MTD_BASEX_DX_CHG_INTR       
                MTD_BASEX_PAGE_RX_INTR      
                MTD_BASEX_AN_COMPLETED_INTR 
                MTD_BASEX_LINK_UP_TO_DN_INTR
                MTD_BASEX_LINK_DN_TO_UP_INTR
                MTD_BASEX_SYMBOL_ERR_INTR   
                MTD_BASEX_FALSE_CARR_INTR   
                MTD_ALL_BASEX_INTR          

 Returns:
    MTD_OK or MTD_FAIL

 Description:
    Get Enabled SGMII interrupts. 

 Side effects:
    None

 Notes/Warnings:
    No X unit on devices after X3310/40. Set HorXunit to MTD_X_UNIT on these 
    devices will return MTD_FAIL.
******************************************************************************/
MTD_STATUS mtdGetBaseXInterruptsEnabled
(
    IN MTD_DEV_PTR devPtr,
    IN MTD_U16   port,
    IN MTD_U16   HorXunit,
    OUT MTD_U16 *interruptEnabled
);

/******************************************************************************
MTD_STATUS mtdGetBaseXInterruptStatus
(
    IN MTD_DEV_PTR devPtr,
    IN MTD_U16   port,
    IN MTD_U16   HorXunit,
    OUT MTD_U16 *interruptStatus
);

 Inputs:
    devPtr   - pointer to MTD_DEV 
    port     - port number, 0-31
    HorXunit - MTD_H_UNIT or MTD_X_UNIT, depending on which interface is
               being read
 Outputs:
    interruptStatus - 
              MTD_BASEX_SPEED_CHG_INTR    
              MTD_BASEX_DX_CHG_INTR       
              MTD_BASEX_PAGE_RX_INTR      
              MTD_BASEX_AN_COMPLETED_INTR 
              MTD_BASEX_LINK_UP_TO_DN_INTR
              MTD_BASEX_LINK_DN_TO_UP_INTR
              MTD_BASEX_SYMBOL_ERR_INTR   
              MTD_BASEX_FALSE_CARR_INTR   
         
 Returns:
    MTD_OK or MTD_FAIL

 Description:
    Get SGMII interrupts status. The interrupts are cleared after this API call.

 Side effects:
    None

 Notes/Warnings:
    The interrupts are cleared after this API call.
    No X unit devices after X3310/40. Set HorXunit to MTD_X_UNIT on these 
    devices will return MTD_FAIL.
******************************************************************************/
MTD_STATUS mtdGetBaseXInterruptStatus
(
    IN MTD_DEV_PTR devPtr,
    IN MTD_U16   port,
    IN MTD_U16   HorXunit,
    OUT MTD_U16 *interruptStatus
);

/******************************************************************************
MTD_STATUS mtdGetBaseXrealTimeStatus
(
    IN MTD_DEV_PTR devPtr,
    IN MTD_U16   port,
    IN MTD_U16   HorXunit,
    OUT MTD_U16 *realTimeStatus
);

 Inputs:
    devPtr   - pointer to MTD_DEV 
    port     - port number, 0-31
    HorXunit - MTD_H_UNIT or MTD_X_UNIT, depending on which interface is
               being read
 Outputs:
    realTimeStatus -
             MTD_RT_SPEED_STATUS         
             MTD_RT_DX_STATUS        
             MTD_RT_PAGE_RX_STATUS       
             MTD_RT_SPEED_DX_RESOLVED
             MTD_RT_LINK_STATUS          
             MTD_RT_SYNC_STATUS          
             MTD_RT_ENG_DETECT_STATUS    
             MTD_RT_TX_PAUSE_STATUS                 
             MTD_RT_RX_PAUSE_STATUS      

 Returns:
    MTD_OK or MTD_FAIL

 Description:
    Get SGMII realtime status. 

 Side effects:
    None

 Notes/Warnings:
******************************************************************************/
MTD_STATUS mtdGetBaseXrealTimeStatus
(
    IN MTD_DEV_PTR devPtr,
    IN MTD_U16   port,
    IN MTD_U16   HorXunit,
    OUT MTD_U16 *realTimeStatus
);


/******************************************************************************
 Functions for accessing the temperature sensor
******************************************************************************/

/******************************************************************************
MTD_STATUS mtdGetTempSensorMDIOPort
(
    IN MTD_DEV_PTR devPtr,
    IN  MTD_U16  port,
    OUT MTD_U16 *sensorPort
);

 Inputs:
    devPtr - pointer to MTD_DEV initialized by mtdLoadDriver() call
    port   - MDIO port address, 0-31

 Output:
    sensorPort - MDIO port number of the temperature sensor on the device(0-31)
             
             Temperature sensor mapping on PHY:

             Cunit
             ---------------------------
             X3240  - *P3

             X3310  - *P0
             X3340  - *P0, *P1, *P2, *P3

             Tunit
             ---------------------------
             E2110  - *P0
             E2180  - *P1, *P3

             X3510  - *P0
             X3520  - *P0, *P1

             X3540  - *P0, *P1, *P2, *P3 (there is one temp sensor but it can be
                                          configured or read from any port)

             X3580  - *P0, *P1, *P2, *P3 *P4, *P5, *P6, *P7 
                      (there is one temp sensor per 4 ports but they can be
                       configured or read from any port)

             E2780  - *P0, *P1, *P2, *P3 *P4, *P5, *P6, *P7 (there is one temp sensor but it can be
                                          configured or read from any port)
                                          
             This is a relative port number.
             * indicates the port number where the temperature sensor is located or can be accessed.
 Returns:
    MTD_OK or MTD_FAIL

 Description:
    This function returns the mdio port number of the temperature sensor 
    for the chip type in devPtr->deviceId

 Side effects:
    None

 Notes/Warnings:
    For X33X0 based PHY, every port has a temperature sensor. For E21X0 PHY, 
    use the temperature sensor of the nearest port.
****************************************************************************/
MTD_STATUS mtdGetTempSensorMDIOPort
(
    IN MTD_DEV_PTR devPtr,
    IN  MTD_U16  port,
    OUT MTD_U16 *sensorPort
);

/******************************************************************************
 MTD_STATUS mtdEnableTemperatureSensor
 (
    IN MTD_DEV_PTR devPtr,
    IN MTD_U16 port
 );

 Inputs:
    devPtr - pointer to MTD_DEV initialized by mtdLoadDriver() call
    port   - MDIO port address, 0-31

 Returns:
    MTD_OK or MTD_FAIL

 Description:
    This function enables the temperature sensor for the PHY type 
    in devPtr->deviceId. It configures it for average mode
    2^9 samples with an intersample period of 1.36ms.

    See mtdGetTempSensorMDIOPort() to see which temperature sensor
    is used on PHYs with multiple temperature sensors.

 Side effects:
    None

 Notes/Warnings:
    On E21X0, X35X0 and E27X0, the T unit temperature sensor is shared with the T unit 
    embedded processor.
 
    For E21X0, X35X0 and E27X0, the T Unit temperature sensor is enabled in firmware on newer
    firmware versions. This function checks if it is not enabled, it enables
    the T unit temperature sensor, otherwise it leaves it as is.

    For x36x0 the T Unit temperature sensor is enabled and controlled by the firmware,
    this function checks if the sensor has been enabled by the firmware to return MTD_OK otherwise MTD_FAIL

    Read result is not valid for first 4.5 seconds after sensor is enabled
    or if it was previously disabled.

****************************************************************************/
MTD_STATUS mtdEnableTemperatureSensor
(
    IN MTD_DEV_PTR devPtr,
    IN MTD_U16 port
);

/******************************************************************************
 MTD_STATUS mtdReadTemperature
 (
    IN MTD_DEV_PTR devPtr,
    IN MTD_U16 port,
    OUT MTD_S16 *temperature
 );

 Inputs:
    devPtr - pointer to MTD_DEV initialized by mtdLoadDriver() call
    port   - MDIO port on this device. API will determine the port number 
             of the temperature sensor.
             
 Outputs:
    temperature - Read out average value of the temperature sensor in Celsius.
                  The temperature resolution is 1 degree increment.

                  The temperature sampling is set to average 2^9 samples
                  at a period of approximately 1.36ms per sample.
                  
                  Temperature range: -75C to +180C

 Returns:
    MTD_OK or MTD_FAIL

 Description:
    This function reads the C Unit temperature sensor for X32X0, X33X0, E20X0
    and T Unit temperature sensor for E21X0, X35X0 and E27X0. To get the MDIO port number of 
    the temperature sensor is located, mtdGetTempSensorMDIOPort() is called.

 Side effects:
    None

 Notes/Warnings:
    For X32X0, X33X0, E20X0, the C Unit temperature sensor needs to be enabled 
    by calling mtdEnableTemperatureSensor() before using this function.

    For E21X0, the temperature sensor is shared with the T unit processor.

    Please wait at least 4.5 seconds after the temperature sensor is enabled 
    before calling this API the very first time.
******************************************************************************/
MTD_STATUS mtdReadTemperature
(
    IN MTD_DEV_PTR devPtr,
    IN MTD_U16 port,
    OUT MTD_S16 *temperature
);

/* Some utilities that are used in the APIs for setting/getting bit fields */
MTD_STATUS mtdHwGetPhyRegField
(
    IN  MTD_DEV_PTR devPtr,
    IN  MTD_U16      port, 
    IN  MTD_U16      dev, 
    IN  MTD_U16      regAddr,
    IN  MTD_U8       fieldOffset,
    IN  MTD_U8       fieldLength,
    OUT MTD_U16      *data
);


MTD_STATUS mtdHwSetPhyRegField
(
    IN MTD_DEV_PTR devPtr,
    IN MTD_U16      port, 
    IN MTD_U16      dev, 
    IN MTD_U16      regAddr,
    IN MTD_U8       fieldOffset,
    IN MTD_U8       fieldLength,
    IN MTD_U16      data
);


MTD_STATUS mtdHwGetRegFieldFromWord
(
    IN  MTD_U16      regData,
    IN  MTD_U8       fieldOffset,
    IN  MTD_U8       fieldLength,
    OUT MTD_U16      *data
);


MTD_STATUS mtdHwSetRegFieldToWord
(
    IN  MTD_U16      regData,
    IN  MTD_U16      bitFieldData,
    IN  MTD_U8       fieldOffset,
    IN  MTD_U8       fieldLength,
    OUT MTD_U16      *data
);


#if C_LINKAGE
#if defined __cplusplus
}
#endif
#endif

#endif /* defined MTD_API_H */


