/* 
 Copyright (c) 2014-2024 Marvell International Ltd., All Rights Reserved
 Redistribution and use in source and binary forms, with or without modification, 
 are permitted provided that the following conditions are met:

 1. Redistributions of source code must retain the above copyright notice, this list 
    of conditions and the following disclaimer.
 2. Redistributions in binary form must reproduce the above copyright notice, this list 
    of conditions and the following disclaimer in the documentation and/or other materials 
    provided with the distribution.
    
 THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY EXPRESS 
 OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY 
 AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR 
 CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR 
 CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR 
 SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY 
 THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR 
 OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE 
 POSSIBILITY OF SUCH DAMAGE.
*/

#ifndef MTD_API_DEVICES_H
#define MTD_API_DEVICES_H

#if C_LINKAGE
#if defined __cplusplus
    extern "C" {
#endif
#endif

#define MTD_REV_UNKNOWN 0 

/* MTD_DEVICE_ID format (assumes 32-bit minimum enum ):  */
/* Bits 31:24 Reserved */
/* Bits 23:16 Extended base type, when base type(i.e. ieee model) is 0x3F, this holds Marvell extended model number */
/*            Will be 0 for those parts with other ieee model numbers (not 0x3F) */
/* Bit 15: 1-> It's an X35X0 octal port device */
/* Bit 14: Reserved */
/* Bit 13: 1-> Max speed 2.5G device  */
/* Bit 12: 1-> Max speed 5G device */
/* Bit 11: Reserved */
/* Bit  10: Reserved */
/* Bits 9:4 0x18 -> X32X0 base, 0x1A X33X0 base, 0x1B E21X0 base, 0x3C X35X0 base, 0x3F plus another 8 bits X36X0 base */
/*          These bits may not be 0. Fully 0 extended base/base is not allowed. */
/* Bits 3:0 revision/number of ports indication, see list */

/* Following defines are for building MTD_DEVICE_ID */
#define MTD_X35X0_OCTALDEV (1UL<<15) /* whether it's an X35X0 octal device */

#define MTD_2P5G_DEVICE  (1UL<<13) 
#define MTD_5G_DEVICE    (1UL<<12)  

#define MTD_MACSEC_CAPABLE (1UL<<11) /* whether the device has a Macsec/PTP module */
#define MTD_COPPER_CAPABLE (1UL<<10) /* whether the device has a copper module */
#define MTD_X32X0_BASE (0x18UL<<4)   /* whether the device uses X32X0 firmware base */
#define MTD_X33X0_BASE (0x1AUL<<4)   /* whether the device uses X33X0 firmware base */
#define MTD_E21X0_BASE (0x1BUL<<4)   /* whether the device uses E21X0 firmware base, this is a max 5G part, copper only, no Macsec, basic PTP */
#define MTD_X35X0_BASE (0x3CUL<<4)   /* This is used for X3540/E2540/X3580/E2580 and X3510/E2510/X3520/E2520 */
                                    /* and E2780, need to use rev id and/or number of ports to further identify */

#define MTD_EXTENDED_BASE (0x3FUL<<4)  /* devices X36X0 and later */
#define MTD_X36X0_BASE    (0x00UL<<16 | MTD_EXTENDED_BASE) /* whether device uses X36X0 firmware base */


#define MTD_DEVID_GETBASE(mTdrevID)    (mTdrevID & ((0xFFUL<<16) | (0x3FUL<<4)))  /* to get full ieee model number/base */
                                                                                 /* parts prior to X36X0 it was only 6 bits */
                                                                                 /* starting with X36X0 it is 8 bits and 6 bits */
                                                                                 /* NOTE: On E2780 this will return the identical value as */
                                                                                 /* X35X0 PHYs because they are the same */
#define MTD_DEVID_GETREV(mTdrevID)     (mTdrevID & 0xF) /* To get just the device revision */


/* Following macros are to test MTD_DEVICE_ID for various features */
#define MTD_IS_DEVID_UNK(mTdrevId) ((MTD_BOOL)(mTdrevId == MTD_REV_UNKNOWN))

/* Following macros define valid base */
#define MTD_IS_X32X0_BASE(mTdrevId) ((MTD_BOOL)((MTD_DEVID_GETBASE(mTdrevId) == MTD_X32X0_BASE) && (!MTD_IS_DEVID_UNK(mTdrevId))))
#define MTD_IS_X33X0_BASE(mTdrevId) ((MTD_BOOL)((MTD_DEVID_GETBASE(mTdrevId) == MTD_X33X0_BASE) && (!MTD_IS_DEVID_UNK(mTdrevId))))
#define MTD_IS_X32X0_X33X0_BASE(mTdrevId) (MTD_IS_X32X0_BASE(mTdrevId) || MTD_IS_X33X0_BASE(mTdrevId))
#define MTD_IS_E21X0_BASE(mTdrevId) ((MTD_BOOL)((MTD_DEVID_GETBASE(mTdrevId) == MTD_E21X0_BASE) && (!MTD_IS_DEVID_UNK(mTdrevId))))
#define MTD_IS_X35X0_BASE(mTdrevId) ((MTD_BOOL)((MTD_DEVID_GETBASE(mTdrevId) == MTD_X35X0_BASE) && (!MTD_IS_DEVID_UNK(mTdrevId))))
#define MTD_IS_X36X0_BASE(mTdrevId) ((MTD_BOOL)((MTD_DEVID_GETBASE(mTdrevId) == MTD_X36X0_BASE) && (!MTD_IS_DEVID_UNK(mTdrevId))))

#define MTD_IS_VALID_BASE(mTdrevId) (MTD_IS_X33X0_BASE(mTdrevId) || MTD_IS_E21X0_BASE(mTdrevId) || \
                                     MTD_IS_X35X0_BASE(mTdrevId) || MTD_IS_X36X0_BASE(mTdrevId))

#define MTD_HAS_X_UNIT(mTddevPtr) ((MTD_IS_X32X0_BASE(mTddevPtr->deviceId) || MTD_IS_X33X0_BASE(mTddevPtr->deviceId)) && !(MTD_IS_5G_DEVICE(mTddevPtr->deviceId)))

/* Following macros define families with MTD_IS_X35X0_BASE, currently only this base has families */
#define MTD_IS_X3540_FAMILY(mTdrevId) (MTD_IS_X35X0_BASE(mTdrevId) && (MTD_DEVID_GETREV(mTdrevId) < 8))
#define MTD_IS_E2780_FAMILY(mTdrevId) (MTD_IS_X35X0_BASE(mTdrevId) && (MTD_DEVID_GETREV(mTdrevId) >= 8) && (MTD_DEVID_GETREV(mTdrevId) < 0xC))
#define MTD_IS_X3510_FAMILY(mTdrevId) (MTD_IS_X35X0_BASE(mTdrevId) && (MTD_DEVID_GETREV(mTdrevId) >= 0xC))

#define MTD_IS_5G_DEVICE(mTdrevId) ((MTD_BOOL)((mTdrevId & MTD_5G_DEVICE) && (!MTD_IS_DEVID_UNK(mTdrevId))))
#define MTD_IS_2P5G_DEVICE(mTdrevId) ((MTD_BOOL)((mTdrevId & MTD_2P5G_DEVICE) && (!MTD_IS_DEVID_UNK(mTdrevId))))       

#define MTD_IS_X3510_E2510_DEVICE(mTddevPtr) (MTD_IS_X3510_FAMILY(mTddevPtr->deviceId) && (mTddevPtr->numPortsPerDevice == 1))
#define MTD_IS_X3520_E2520_DEVICE(mTddevPtr) (MTD_IS_X3510_FAMILY(mTddevPtr->deviceId) && (mTddevPtr->numPortsPerDevice == 2))
#define MTD_IS_X3510_DEVICE(mTddevPtr) (MTD_IS_X3510_E2510_DEVICE(mTddevPtr) && !MTD_IS_5G_DEVICE(mTddevPtr->deviceId))
#define MTD_IS_E2510_DEVICE(mTddevPtr) (MTD_IS_X3510_E2510_DEVICE(mTddevPtr) && MTD_IS_5G_DEVICE(mTddevPtr->deviceId))
#define MTD_IS_X3520_DEVICE(mTddevPtr) (MTD_IS_X3520_E2520_DEVICE(mTddevPtr) && !MTD_IS_5G_DEVICE(mTddevPtr->deviceId))
#define MTD_IS_E2520_DEVICE(mTddevPtr) (MTD_IS_X3520_E2520_DEVICE(mTddevPtr) && MTD_IS_5G_DEVICE(mTddevPtr->deviceId))

#define MTD_IS_X3540_E2540_DEVICE(mTddevPtr) (MTD_IS_X3540_FAMILY(mTddevPtr->deviceId) && (mTddevPtr->numPortsPerDevice == 4))
#define MTD_IS_X3580_E2580_DEVICE(mTddevPtr) (MTD_IS_X3540_FAMILY(mTddevPtr->deviceId) && (mTddevPtr->numPortsPerDevice == 8))
#define MTD_IS_X3540_DEVICE(mTddevPtr) (MTD_IS_X3540_E2540_DEVICE(mTddevPtr) && !MTD_IS_5G_DEVICE(mTddevPtr->deviceId))
#define MTD_IS_E2540_DEVICE(mTddevPtr) (MTD_IS_X3540_E2540_DEVICE(mTddevPtr) && MTD_IS_5G_DEVICE(mTddevPtr->deviceId))
#define MTD_IS_X3580_DEVICE(mTddevPtr) (MTD_IS_X3580_E2580_DEVICE(mTddevPtr) && !MTD_IS_5G_DEVICE(mTddevPtr->deviceId))
#define MTD_IS_E2580_DEVICE(mTddevPtr) (MTD_IS_X3580_E2580_DEVICE(mTddevPtr) && MTD_IS_5G_DEVICE(mTddevPtr->deviceId))

#define MTD_IS_X3610_E2610_DEVICE(mTddevPtr) (MTD_IS_X36X0_BASE(mTddevPtr->deviceId) && (mTddevPtr->numPortsPerDevice == 1))
#define MTD_IS_X3680_E2680_DEVICE(mTddevPtr) (MTD_IS_X36X0_BASE(mTddevPtr->deviceId) && (mTddevPtr->numPortsPerDevice == 8))
#define MTD_IS_X3610_DEVICE(mTddevPtr) (MTD_IS_X3610_E2610_DEVICE(mTddevPtr) && !MTD_IS_5G_DEVICE(mTddevPtr->deviceId))
#define MTD_IS_E2610_DEVICE(mTddevPtr) (MTD_IS_X3610_E2610_DEVICE(mTddevPtr) && MTD_IS_5G_DEVICE(mTddevPtr->deviceId))
#define MTD_IS_X3680_DEVICE(mTddevPtr) (MTD_IS_X3680_E2680_DEVICE(mTddevPtr) && !MTD_IS_5G_DEVICE(mTddevPtr->deviceId))
#define MTD_IS_E2680_DEVICE(mTddevPtr) (MTD_IS_X3680_E2680_DEVICE(mTddevPtr) && MTD_IS_5G_DEVICE(mTddevPtr->deviceId))

#define MTD_IS_E2780_DEVICE(mTddevPtr) (MTD_IS_E2780_FAMILY(mTddevPtr->deviceId))

/* Following macros define devices which's serdes are NONE COMPHY, these devices were made before X3540 */
#define MTD_IS_NONE_COMPHY_DEVICE(mTdrevId) (MTD_IS_X33X0_BASE(mTdrevId) || MTD_IS_E21X0_BASE(mTdrevId))

/* Following macros define devices which's serdes are COMPHY, these devices were made starting with X3540 */
#define MTD_IS_COMPHY_DEVICE(mTdrevId) (!MTD_IS_NONE_COMPHY_DEVICE(mTdrevId))

/* Following macros are to test if device contains serdes fw */
#define MTD_HAS_SERDES_FW(mTdrevId) (MTD_IS_COMPHY_DEVICE(mTdrevId) && !MTD_IS_X3510_FAMILY(mTdrevId))

/* Following macros define devices were made after X3540, mac header and EEE buffer feature only apply to these devices */
#define MTD_IS_POST_X3540_DEVICE(mTdrevId) (MTD_IS_COMPHY_DEVICE(mTdrevId) && !MTD_IS_X3540_FAMILY(mTdrevId))

/* Following macros define devices which's code RAM is shared between every 2 ports */
#define MTD_CODE_RAM_SHARED_DEVICE(mTddevPtr) (MTD_IS_X3680_E2680_DEVICE(mTddevPtr) || MTD_IS_E2780_DEVICE(mTddevPtr))

/* Following macro define COMPHY devices with multiple ports sharing one serdes lane */
#define MTD_COMPHY_MULT_PORTS_DEVICE(mTddevPtr) (MTD_IS_X3540_FAMILY(mTddevPtr->deviceId) || MTD_IS_X3680_E2680_DEVICE(mTddevPtr) || MTD_IS_E2780_DEVICE(mTddevPtr))

/* COMPHY multiple port device imply support D/Q/OXGMII mode some of which requires 20G serdes (when bandwidth summation adds to 20G) */
#define MTD_COMPHY_SERDES_20G_DEVICE(mTddevPtr) (MTD_COMPHY_MULT_PORTS_DEVICE(mTddevPtr))

/* Some macros for convenience */
#define MTD_ATTEMPT(xFuncToTry) {if(xFuncToTry != MTD_OK) \
                            {MTD_DBG_ERROR("MTD_ATTEMPT failed at %s:%u\n",__FUNCTION__,__LINE__); \
                            return MTD_FAIL;}}

#define MTD_CONVERT_BOOL_TO_UINT(boolVar,uintVar) {(boolVar) ? (uintVar=1) : (uintVar=0);}
#define MTD_CONVERT_UINT_TO_BOOL(uintVar,boolVar) {(uintVar) ? (boolVar=MTD_TRUE) : (boolVar=MTD_FALSE);}
#define MTD_GET_BOOL_AS_BIT(boolVar) ((boolVar) ? 1 : 0)
#define MTD_GET_BIT_AS_BOOL(uintVar) ((uintVar) ? MTD_TRUE : MTD_FALSE)


/* Revision */
#define MTD_X33X0BASE_SINGLE_PORTA0 0xA /* X3310 A0 */
#define MTD_X33X0BASE_QUAD_PORTA0   0x2 /* X3340 A0 */
#define MTD_X33X0BASE_SINGLE_PORTA1 0xB /* X3310 A1 */
#define MTD_X33X0BASE_QUAD_PORTA1   0x3 /* X3340 A1 */
#define MTD_X33X0BASE_SINGLE_PORTB0 0xE /* X3310 B0 */
#define MTD_X33X0BASE_QUAD_PORTB0   0xC /* X3340 B0 */
#define MTD_X33X0BASE_SINGLE_PORTB1 0xF /* X3310 B1 */
#define MTD_X33X0BASE_QUAD_PORTB1   0xD /* X3340 B1 */

#define MTD_E21X0BASE_SINGLE_PORTA0 0x9 /* E2110 A0 */
#define MTD_E21X0BASE_OCTAL_PORTA0  0x0 /* E2180 A0 */
#define MTD_E21X0BASE_OCTAL_PORTA1  0x1 /* E2180 A1 */

#define MTD_X35X0BASE_QUAD_PORTB0   0x1 /* X3540/E2540 B0 */

#define MTD_X35X0BASE_OCTAL_PORTA0  0x3 /* X3580/E2580 A0 */

#define MTD_X35X0_BASE_SINGLE_PORTA0 0xC /* X3510/E2510 */
#define MTD_X35X0_BASE_DUAL_PORTA0   0xE /* X3520/E2520 */

#define MTD_X35X0BASE_QUADOCTAL_PORTB3 0x7 /* X3540/X3580 same rev id */

#define MTD_E27X0BASE_OCTAL_PORTA0   0x8 /* E2780 */

#define MTD_X36X0BASE_SINGLE_PORTB0  0x4 /* X3610/E2610 */
#define MTD_X36X0BASE_OCTAL_PORTB0   0xC /* X3680/E2680 */


/* Device IDs Supported */
#define MTD_X3310_A0 (MTD_COPPER_CAPABLE | MTD_X33X0_BASE | MTD_X33X0BASE_SINGLE_PORTA0)
#define MTD_X3310_A1 (MTD_COPPER_CAPABLE | MTD_X33X0_BASE | MTD_X33X0BASE_SINGLE_PORTA1)
#define MTD_X3310_B0 (MTD_COPPER_CAPABLE | MTD_X33X0_BASE | MTD_X33X0BASE_SINGLE_PORTB0)
#define MTD_X3310_B1 (MTD_COPPER_CAPABLE | MTD_X33X0_BASE | MTD_X33X0BASE_SINGLE_PORTB1)
#define MTD_X3340_A0 (MTD_COPPER_CAPABLE | MTD_X33X0_BASE | MTD_X33X0BASE_QUAD_PORTA0)
#define MTD_X3340_A1 (MTD_COPPER_CAPABLE | MTD_X33X0_BASE | MTD_X33X0BASE_QUAD_PORTA1)
#define MTD_X3340_B0 (MTD_COPPER_CAPABLE | MTD_X33X0_BASE | MTD_X33X0BASE_QUAD_PORTB0)
#define MTD_X3340_B1 (MTD_COPPER_CAPABLE | MTD_X33X0_BASE | MTD_X33X0BASE_QUAD_PORTB1)

#define MTD_E2110_A0 (MTD_E21X0_BASE | MTD_E21X0BASE_SINGLE_PORTA0)
#define MTD_E2180_A0 (MTD_E21X0_BASE | MTD_E21X0BASE_OCTAL_PORTA0)
#define MTD_E2180_A1 (MTD_E21X0_BASE | MTD_E21X0BASE_OCTAL_PORTA1)

#define MTD_X3510_A0 (MTD_X35X0_BASE | MTD_X35X0_BASE_SINGLE_PORTA0)
#define MTD_X3520_A0 (MTD_X35X0_BASE | MTD_X35X0_BASE_DUAL_PORTA0)
#define MTD_E2510_A0 (MTD_5G_DEVICE | MTD_X35X0_BASE |  MTD_X35X0_BASE_SINGLE_PORTA0)
#define MTD_E2520_A0 (MTD_5G_DEVICE | MTD_X35X0_BASE | MTD_X35X0_BASE_DUAL_PORTA0)

#define MTD_X3540_B0 (MTD_X35X0_BASE | MTD_X35X0BASE_QUAD_PORTB0)
#define MTD_E2540_B0 (MTD_5G_DEVICE | MTD_X35X0_BASE | MTD_X35X0BASE_QUAD_PORTB0)
#define MTD_X3540_B3 (MTD_X35X0_BASE | MTD_X35X0BASE_QUADOCTAL_PORTB3)
#define MTD_E2540_B3 (MTD_5G_DEVICE | MTD_X35X0_BASE | MTD_X35X0BASE_QUADOCTAL_PORTB3)

#define MTD_X3580_A0 (MTD_X35X0_BASE |MTD_X35X0BASE_OCTAL_PORTA0)    
#define MTD_E2580_A0 (MTD_5G_DEVICE |MTD_X35X0_BASE |MTD_X35X0BASE_OCTAL_PORTA0)
#define MTD_X3580_B3 (MTD_X35X0_OCTALDEV | MTD_X35X0_BASE | MTD_X35X0BASE_QUADOCTAL_PORTB3)
#define MTD_E2580_B3 (MTD_X35X0_OCTALDEV | MTD_5G_DEVICE | MTD_X35X0_BASE | MTD_X35X0BASE_QUADOCTAL_PORTB3)

#define MTD_X3610_B0 (MTD_X36X0_BASE | MTD_X36X0BASE_SINGLE_PORTB0)
#define MTD_E2610_B0 (MTD_5G_DEVICE | MTD_X36X0_BASE | MTD_X36X0BASE_SINGLE_PORTB0)    

#define MTD_E2780A_A0 (MTD_2P5G_DEVICE | MTD_X35X0_BASE | MTD_E27X0BASE_OCTAL_PORTA0)


/* Need one of these for each type of device to be controlled by the API. Most systems would need */
/* only one. However, the included sample code has multiple devices so these were all uncommented */
/* so the sample code could be built and run. */

/* Comment out the unused ones to save memory. */

extern MTD_DEV x3310deviceA0;
extern MTD_DEV x3310deviceA1;
extern MTD_DEV x3310deviceB0;
extern MTD_DEV x3310deviceB1;

extern MTD_DEV x3340deviceA0;
extern MTD_DEV x3340deviceA1;
extern MTD_DEV x3340deviceB0;
extern MTD_DEV x3340deviceB1;

extern MTD_DEV e2110deviceA0;
extern MTD_DEV e2180deviceA0;
extern MTD_DEV e2180deviceA1;

extern MTD_DEV x3510deviceA0;
extern MTD_DEV e2510deviceA0;
extern MTD_DEV x3520deviceA0;
extern MTD_DEV e2520deviceA0;

extern MTD_DEV x3540deviceB0;
extern MTD_DEV e2540deviceB0;
extern MTD_DEV x3540deviceB3;
extern MTD_DEV e2540deviceB3;

extern MTD_DEV x3580deviceA0;
extern MTD_DEV e2580deviceA0;
extern MTD_DEV x3580deviceB3;
extern MTD_DEV e2580deviceB3;

extern MTD_DEV x3610deviceB0;
extern MTD_DEV e2610deviceB0;

extern MTD_DEV e2780AdeviceA0;               



#if C_LINKAGE
#if defined __cplusplus
}
#endif
#endif

#endif /* defined MTD_API_DEVICES_H */

