/* 
 Copyright (c) 2014-2024 Marvell International Ltd., All Rights Reserved
 Redistribution and use in source and binary forms, with or without modification, 
 are permitted provided that the following conditions are met:

 1. Redistributions of source code must retain the above copyright notice, this list 
    of conditions and the following disclaimer.
 2. Redistributions in binary form must reproduce the above copyright notice, this list 
    of conditions and the following disclaimer in the documentation and/or other materials 
    provided with the distribution.
    
 THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY EXPRESS 
 OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY 
 AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR 
 CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR 
 CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR 
 SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY 
 THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR 
 OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE 
 POSSIBILITY OF SUCH DAMAGE.
*/


/********************************************************************
This file contains functions prototypes and global defines/data for
higher-level functions using MDIO access to download firmware
into the internal RAM or attached flash memory of the Marvell 88X32X0, 
88X33X0, 88X35X0,  CUX36X0, CUE26X0, 88E20X0, 88E21X0 
and CUE2780 ethernet PHYs..
********************************************************************/
#include "openSourceMtdAPITypes.h"
#include "openSourceMtdUser.h"
#include "openSourceMtdAPI.h"
#include "openSourceMtdAPIDevices.h"
#include "openSourceMtdFwDownload.h"

/*******************************************************************
 Local Macros
 *******************************************************************/
#define MTD_CKSUM_OFFSET 16 
#define MTD_HEADER_SIZE  32

/* Maximum image size calculations */
/* Parts prior to X3540  */
#define MTD_MAX_HEADERS 2
#define MTD_MAX_APP_SIZE (216*1024UL) /* Includes X3510 family devices */
#define MTD_MAX_FLASH_IMAGE_SIZE (MTD_MAX_APP_SIZE + (MTD_MAX_HEADERS*MTD_HEADER_SIZE))

/* X35X0 and also for E2780 parts */
#define MTD_MAX_X35X0_HEADERS 8
#define MTD_MAX_X35X0_APP_SIZE (256*1024UL)
#define MTD_MAX_X35X0_MAX_SERDES_IMG_SZ ((64+4+4)*1024UL)
#define MTD_MAX_X35X0_FLASH_IMAGE_SIZE (MTD_MAX_X35X0_APP_SIZE+ MTD_MAX_X35X0_MAX_SERDES_IMG_SZ + (MTD_MAX_X35X0_HEADERS*MTD_HEADER_SIZE))

/* X36X0 parts */
#define MTD_MAX_X36X0_MAX_SERDES_IMG_SZ (128*1024UL)
#define MTD_MAX_X36X0_FLASH_IMAGE_SIZE (MTD_MAX_X35X0_APP_SIZE+ MTD_MAX_X36X0_MAX_SERDES_IMG_SZ + (MTD_MAX_X35X0_HEADERS*MTD_HEADER_SIZE))

/* X3510/X3520/E2510/E2510 parts, no serdes firmware, smaller memory */
#define MTD_MAX_X3510_FLASH_IMAGE_SIZE (MTD_MAX_APP_SIZE + (MTD_MAX_X35X0_HEADERS*MTD_HEADER_SIZE))

#define MTD_EXECUTABLE_TO (3000) /* max time in ms that intermediate executables may take to execute in multi-stage boot process */

#define MTD_LW_REG 3,0xD0F0 
#define MTD_HW_REG 3,0xD0F1 
#define MTD_RD_REG 3,0xD0F2 
#define MTD_RC_REG 3,0xD0F3 

#define X36X0_E27X0_HDR_IDX 0x0013FE8C 
#define X36X0_E27X0_HDR_TBL 0x0013FEE0 
#define X35X0_E25X0_HDR_IDX 0x28000130 
#define X35X0_E25X0_HDR_TBL 0x28000000 

#define PORT_BIT(port_Num)                              (1 << port_Num)
#define IS_EVEN_PORT(port_Num)                          ((port_Num % 2) == 0)
#define IS_RELEVANT_PORT(Skipped_Port_BitMap, port_Num) ((Skipped_Port_BitMap & PORT_BIT(port_Num)) == 0)

#define PRINT_DL_DEBUG 0
#define MTD_CODE_DOWNLOADED_POS (6)

#define MTD_SERDES_LOADER_DONE 0xBFF 
#define MTD_SERDES_LOADER_SKIPPED 0xBFE 

#define TRY_IO(xFuncToTry) if (xFuncToTry == MTD_FAIL) {*errCode = MTD_IO_ERROR; return MTD_FAIL;}


/*******************************************************************
 Local Data Types
 *******************************************************************/
typedef struct 
{
    MTD_U32 dL;                 
    MTD_U32 dD;       
    MTD_U16 secCh;         
    MTD_U16 d_o; 
    MTD_U16 p_s;
    MTD_U32 nHO;       
} MTD_FILE_HEADER_TYPE;


/*******************************************************************
 Local Function Prototypes
 *******************************************************************/
MTD_STATIC MTD_STATUS mtdMdioRamDownload(IN MTD_DEV_PTR devPtr,IN MTD_U8 data[], IN MTD_U32 size, IN MTD_U16 port, OUT MTD_U16 *errCode); 
MTD_STATIC MTD_STATUS mtdParallelMdioRamDownload(IN MTD_DEV_PTR devPtr,IN MTD_U8 data[],IN MTD_U32 size, IN MTD_U16 ports[],IN MTD_U16 numPorts, OUT MTD_U16 erroredPorts[], OUT MTD_U16 *errCode);

/*******************************************************************/
/*******************************************************************
 RAM Application Code Download
 Parallel RAM Application Code Downloaded (required on newer 
                                           multi-port PHYs)
 *******************************************************************/

MTD_STATUS mtdUpdateRamImage
(
    IN MTD_DEV_PTR devPtr,
    IN MTD_U16 port,
    IN MTD_U8 appData[],
    IN MTD_U32 appSize,
    OUT MTD_U16 *errCode
)
{

    *errCode = 0;

    if (MTD_IS_X3540_FAMILY(devPtr->deviceId))
    {
        MTD_DBG_ERROR("X35X0/E2540 devices must call mtdParallelUpdateRamImage() to download firmware on port pairs.\n");
        *errCode = MTD_NON_GROUPED_RAM_DOWNLOAD;
        return MTD_FAIL;
    }
    
    if (MTD_IS_X3680_E2680_DEVICE(devPtr) || MTD_IS_E2780_DEVICE(devPtr))
    {
        MTD_DBG_ERROR("X3680/E2680/E2780 devices must call mtdParallelUpdateRamImage() to download firmware on all ports.\n");
        *errCode = MTD_NON_GROUPED_RAM_DOWNLOAD;
        return MTD_FAIL;
    }
    
    MTD_ATTEMPT(mtdMdioRamDownload(devPtr,appData,appSize,port,errCode));
    
    return MTD_OK;  /* Ram download is done*/    
}


MTD_STATUS mtdParallelUpdateRamImage
(
    IN MTD_DEV_PTR devPtr,
    IN MTD_U16 ports[],
    IN MTD_U8 appData[],
    IN MTD_U32 appSize, 
    IN MTD_U16 numPorts,
    OUT MTD_U16 erroredPorts[],
    OUT MTD_U16 *errCode
)
{
    MTD_U16 i;

    *errCode = 0;

    if(numPorts > MTD_MAX_PORTS_TO_DOWNLOAD)
    {
        *errCode = MTD_NUM_PORTS_TOO_LARGE;
        return MTD_FAIL;
    }

    for(i = 0; i < numPorts; i++)  
    {
        erroredPorts[i] = 0;
    }

    if (MTD_IS_X3540_FAMILY(devPtr->deviceId))
    {
        MTD_U16 firstPort;
        MTD_BOOL groupFound = MTD_FALSE;
        MTD_BOOL pairFound = MTD_FALSE;
        MTD_BOOL visitedPorts[MTD_MAX_PORTS_TO_DOWNLOAD];

        if (numPorts % 2)
        {
            MTD_DBG_ERROR("X35X0/E2540 device must download firmware on paired ports(P0/P1 or P2/P3, etc.). numPorts must be groups of 2!\n");
            *errCode = MTD_GROUPED_PORTS_NOT_FOUND;
            return MTD_FAIL;        
        }

        for (i = 0; i < MTD_MAX_PORTS_TO_DOWNLOAD; i++)
        {
            visitedPorts[i] = MTD_FALSE;
        }

        for (i = 0; i < numPorts; i++)
        {
            visitedPorts[ports[i]] = MTD_TRUE;
        }

        for (firstPort = 0; firstPort < MTD_MAX_PORTS_TO_DOWNLOAD; firstPort+=2)
        {
            groupFound = (visitedPorts[firstPort] || visitedPorts[firstPort+1]) ? MTD_TRUE : MTD_FALSE;

            if (groupFound == MTD_TRUE)
            {
                pairFound = (visitedPorts[firstPort] && visitedPorts[firstPort+1]) ? MTD_TRUE : MTD_FALSE;
                if (pairFound == MTD_FALSE)
                {
                    MTD_DBG_ERROR("X3540/E2540 device must download firmware on paired ports(P0/P1 or P2/P3). Expected port group is not found in the ports[]\n");
                    *errCode = MTD_GROUPED_PORTS_NOT_FOUND;
                    return MTD_FAIL;
                }
            }
        }
    }

    if (MTD_IS_X3680_E2680_DEVICE(devPtr) || MTD_IS_E2780_DEVICE(devPtr))
    {
        MTD_U16 firstPort;
        MTD_BOOL visitedPorts[MTD_MAX_PORTS_TO_DOWNLOAD];

        if (numPorts % 8)
        {
            MTD_DBG_ERROR("X3580/E2680 device must download firmware on all ports.\n). numPorts must be groups of 8!\n");
            *errCode = MTD_GROUPED_PORTS_NOT_FOUND;
            return MTD_FAIL;        
        }

        for (i = 0; i < MTD_MAX_PORTS_TO_DOWNLOAD; i++)
        {
            visitedPorts[i] = MTD_FALSE;
        }

        for (i = 0; i < numPorts; i++)
        {
            visitedPorts[ports[i]] = MTD_TRUE;
        }        

        for (i = 0; i < MTD_MAX_PORTS_TO_DOWNLOAD; )
        {
            if (visitedPorts[i])
            {
                if ((i % 8) != 0)
                {
                    MTD_DBG_ERROR("X3680/E2680 device must download firmware on all 8 ports. First port is missing in ports[]\n");
                    *errCode = MTD_GROUPED_PORTS_NOT_FOUND;
                    return MTD_FAIL;                
                }
                else
                {
                    for (firstPort = i; firstPort < (i+8); firstPort++)
                    {
                        if (visitedPorts[firstPort] == MTD_FALSE)
                        {
                            MTD_DBG_ERROR("X3680/E2680 device must download firmware on all 8 ports. Some port is missing in ports[]\n");
                            *errCode = MTD_GROUPED_PORTS_NOT_FOUND;
                            return MTD_FAIL;                        
                        }
                    }
                    i += 8; 
                }
            }
            else
            {
                i++;
            }
        }

    }

    MTD_ATTEMPT(mtdParallelMdioRamDownload(devPtr,appData,appSize, ports, \
                                       numPorts,erroredPorts,errCode));    
    
    for(i = 0; i < numPorts; i++)
    {
        if(erroredPorts[i] != 0)
        {
            *errCode = MTD_PAR_DOWNLOAD_FAILED;
            return MTD_FAIL;  
        }
    }

    return MTD_OK;     
}

/*********************************************************************
 Local functions
 *********************************************************************/

MTD_STATIC MTD_PVOID mtdMemSet
(MTD_PVOID start, int symbol, MTD_U32 size)
{
    MTD_U32 i;
    char* buf;
    
    buf = (char*)start;
        
    for(i=0; i<size; i++)
    {
        *buf++ = (char)symbol;
    }

    return start;
}

MTD_STATIC void mtdGetU32fromBuf(MTD_U32 *num, MTD_U8 *buf)
{
    unsigned counter;

    for(*num = 0, counter = 0; counter < 4; counter++)
    {
        *num += (((MTD_U32)(buf[counter]))<<(counter*8));
    }
}

MTD_STATIC void mtdGetU16fromBuf(MTD_U16 *num, MTD_U8 *buf)
{
    unsigned counter;
    
    for(*num = 0, counter = 0; counter < 2; counter++)
    {
        *num += (((MTD_U16)(buf[counter]))<<(counter*8));
    }   
}

MTD_STATIC MTD_BOOL mtdGetHeader(MTD_FILE_HEADER_TYPE *hdr, MTD_U8 *buf)
{
    MTD_U16 calcdChecksum = 0, hdrChecksum;
    MTD_U16 hdrSizeInBytes = MTD_HEADER_SIZE - MTD_CKSUM_OFFSET; /* excludes header checksum */
    MTD_U8 *bufPtr = buf;

    /* First make sure it's a good header */
    while(hdrSizeInBytes--)
    {
        calcdChecksum += *bufPtr++;
    }
    calcdChecksum = (~calcdChecksum);
    mtdGetU16fromBuf(&hdrChecksum, &buf[MTD_CKSUM_OFFSET]);

    if (hdrChecksum != calcdChecksum)
    {
        MTD_DBG_ERROR("mtdGetHeader: checksum mismatched hdrChecksum 0x%X calcdChecksum 0x%X\n", 
                       hdrChecksum, calcdChecksum);
        return MTD_FALSE;
    }

    /* Now fill in the data */
    mtdGetU32fromBuf(&(hdr->dL), buf);      buf += 4;
    mtdGetU32fromBuf(&(hdr->dD), buf); buf += 4;
    mtdGetU16fromBuf(&(hdr->secCh), buf);     buf += 2;
    hdr->d_o = ((*buf++) & 0x40)>>6;
    hdr->p_s = *buf++;
    mtdGetU32fromBuf(&(hdr->nHO), buf);

    return MTD_TRUE;
    
}

MTD_STATIC MTD_STATUS mtdGetBootcodeHeaderIdx(IN MTD_DEV_PTR devPtr, IN MTD_U16 *relevantPorts,
                                              IN MTD_U16 numRelevantPort, OUT MTD_U16 *bootcodeHeaderIdxs)
{
    MTD_U16 i;
    MTD_U32 headeIdxAddr = (MTD_IS_X3540_FAMILY(devPtr->deviceId) || MTD_IS_X3510_FAMILY(devPtr->deviceId))? 
                          (MTD_U32)X35X0_E25X0_HDR_IDX :
                          (MTD_U32)X36X0_E27X0_HDR_IDX;

    for (i = 0; i < numRelevantPort; i++)
    {
        MTD_ATTEMPT(mtdHwXmdioWrite(devPtr, relevantPorts[i], MTD_LW_REG, (MTD_U16)headeIdxAddr));
        MTD_ATTEMPT(mtdHwXmdioWrite(devPtr, relevantPorts[i], MTD_HW_REG, (MTD_U16)(headeIdxAddr >> 16)));
        MTD_ATTEMPT(mtdHwXmdioRead(devPtr, relevantPorts[i], MTD_RD_REG, &bootcodeHeaderIdxs[i]));
    }

    return MTD_OK;
}

MTD_STATIC MTD_STATUS mtdIncreasBootcodeHeaderIdx(IN MTD_DEV_PTR devPtr, IN MTD_U16 *relevantPorts,
                                              IN MTD_U16 numRelevantPort, OUT MTD_U16 *bootcodeHeaderIdxs)
{
    MTD_U16 i;
    MTD_U32 headeIdxAddr = (MTD_IS_X3540_FAMILY(devPtr->deviceId) || MTD_IS_X3510_FAMILY(devPtr->deviceId)) ? 
                          (MTD_U32)X35X0_E25X0_HDR_IDX :
                          (MTD_U32)X36X0_E27X0_HDR_IDX;

    for (i = 0; i < numRelevantPort; i++)
    {
        MTD_ATTEMPT(mtdHwXmdioWrite(devPtr, relevantPorts[i], MTD_LW_REG, (MTD_U16)headeIdxAddr));
        MTD_ATTEMPT(mtdHwXmdioWrite(devPtr, relevantPorts[i], MTD_HW_REG, (MTD_U16)(headeIdxAddr >> 16)));
        MTD_ATTEMPT(mtdHwXmdioWrite(devPtr, relevantPorts[i], MTD_RD_REG, bootcodeHeaderIdxs[i] + 1));
    }

    return MTD_OK;
}

MTD_STATIC MTD_STATUS mtdRamDownloadCalcChecksums(MTD_DEV_PTR devPtr, MTD_U16 port,
                                       MTD_U8 data[], MTD_U32 size, /* size is in bytes, but must be even */
                                       MTD_U32 ramAddress, /* place in RAM to download to */
                                       MTD_U16 *ramChecksumByByte, /* RAM interface or HDR file checksum (inverted) */
                                       MTD_U16 *wordChecksum, /* used by slave bin file protocol for flash download */
                                       MTD_U16 *errCode)
{
    MTD_U32 buffCount;
    MTD_U8  lowByte, highByte;
    MTD_U16 byteChecksum, byteChecksumRead;
    
    TRY_IO(mtdHwXmdioRead(devPtr,port, MTD_RC_REG, &byteChecksum));        

    TRY_IO(mtdHwXmdioWrite(devPtr,port,MTD_LW_REG,(MTD_U16)ramAddress));
    TRY_IO(mtdHwXmdioWrite(devPtr,port,MTD_HW_REG,(MTD_U16)(ramAddress>>16)));

    buffCount=0;
    byteChecksum = 0; 
    *wordChecksum = 0;
    while(buffCount < size)
    {
        lowByte = data[buffCount++];
        highByte = data[buffCount++];
        
        byteChecksum += lowByte; /* HDR file and RAM interface calc checksum this way*/
        byteChecksum += highByte;

        *wordChecksum += ((((MTD_U16)highByte)<<8)|lowByte); /* slave bin file calculates checksum this way */
        TRY_IO(mtdHwXmdioWrite(devPtr,port,MTD_RD_REG,(((MTD_U16)highByte)<<8)|lowByte));        

    }    

    *ramChecksumByByte = byteChecksum;

    TRY_IO(mtdHwXmdioRead(devPtr,port,MTD_RC_REG,&byteChecksumRead));        
    
    if(byteChecksum != byteChecksumRead)
    {
        MTD_DBG_ERROR( "Error downloading code. Expected checksum to be 0x%04X but it was 0x%04X", (int)byteChecksum, (int)byteChecksumRead);
        *errCode = MTD_RAM_HW_CHECKSUM_ERR;
        return MTD_FAIL;
    }    

    return MTD_OK;
}

                                       
MTD_STATIC MTD_STATUS mtdHwWaitForRegFieldValue
(
    IN MTD_DEV_PTR devPtr,
    IN MTD_U16      port, 
    IN MTD_U16      dev, 
    IN MTD_U16      regAddr,
    IN MTD_U8       fieldOffset,
    IN MTD_U8       fieldLength,
    IN MTD_U16      expectedValue,
    IN MTD_U16      timeoutMs
)
{
    MTD_U16 waitCounter, valueRead;
    
    waitCounter = 0;
    MTD_ATTEMPT(mtdHwGetPhyRegField(devPtr,port,dev,regAddr,fieldOffset,fieldLength,&valueRead));
    while((valueRead != expectedValue) && (waitCounter < timeoutMs))
    {
        MTD_ATTEMPT(mtdWait(devPtr,1));
        MTD_ATTEMPT(mtdHwGetPhyRegField(devPtr,port,dev,regAddr,fieldOffset,fieldLength,&valueRead));
        waitCounter++;
    }

    if (valueRead == expectedValue)
    {
        return MTD_OK;
    }
    else
    {
        return MTD_FAIL;
    }
}

MTD_STATIC MTD_STATUS mtdWaitForRegFieldAltValues(IN MTD_DEV_PTR devPtr, IN MTD_U16 port, IN MTD_U16 dev, IN MTD_U16 regAddr,IN MTD_U8 fieldOffset, IN MTD_U8 fieldLength,IN MTD_U16 expectedValue0, IN MTD_U16 expectedValue1, IN MTD_U16 timeoutMs)
{
    MTD_U16 timeOutCounter = 0;
    MTD_STATUS status = MTD_FAIL;;

    while (timeOutCounter++ < timeoutMs)
    {
        if ((mtdHwWaitForRegFieldValue(devPtr, port, dev, regAddr, fieldOffset, fieldLength, expectedValue0, 1) == MTD_OK) ||
            (mtdHwWaitForRegFieldValue(devPtr, port, dev, regAddr, fieldOffset, fieldLength, expectedValue1,1) == MTD_OK))
        {
            status = MTD_OK;
            break;
        }
    }

    return status;
}

MTD_STATIC MTD_STATUS mtdMdioRamDownload(MTD_DEV_PTR devPtr,MTD_U8 data[],
                                         MTD_U32 size, MTD_U16 port, 
                                         MTD_U16 *errCode) 
{
    MTD_U16 word_checksum, byte_checksum, sectionCount = 0, maxSections, basePort, relativePort, bootcodeHeaderIdx;
    MTD_BOOL downloadMode;
    MTD_U32 imageIndex = 0;
    MTD_BOOL appStarted;
    MTD_FILE_HEADER_TYPE fHeader;

    *errCode = 0;
    mtdMemSet(&fHeader, 0, sizeof(MTD_FILE_HEADER_TYPE));
    if (MTD_IS_COMPHY_DEVICE(devPtr->deviceId))
    {
        maxSections = MTD_MAX_X35X0_HEADERS;
    }
    else
    {
        maxSections = MTD_MAX_HEADERS;
    }

    if (size%2)
    {
        *errCode = MTD_SIZE_NOT_EVEN;
        return MTD_FAIL;
    }

    MTD_ATTEMPT(mtdPutPhyInMdioDownloadMode(devPtr,port)); /* will leave errCode as 0 if fails */
    
    MTD_ATTEMPT(mtdWait(devPtr,50));     

    TRY_IO(mtdIsPhyInMdioDownloadMode(devPtr,port,&downloadMode));
        
    if (downloadMode == MTD_FALSE)
    {
        MTD_DBG_ERROR("PHY is not in waiting on download mode.\n");
        MTD_DBG_ERROR("Download failed\n");
        *errCode = MTD_PHY_NOT_IN_DOWNLOAD_MODE;
        return MTD_FAIL;
    }
    else
    {
#if PRINT_DL_DEBUG
        MTD_DBG_INFO("Downloading code to PHY RAM, please wait...\n");
#endif
    }    

    do
    {
        if (imageIndex > size)
        {
            MTD_DBG_ERROR("Image size and image data do not match. Went past end of image.\n");

            *errCode = MTD_ERR_PROCESSING_IMAGE_FILE;
            return MTD_FAIL;                    
        }   
        
        if (mtdGetHeader(&fHeader, &data[imageIndex]) == MTD_FALSE)
        {
            MTD_DBG_ERROR("Header corrupted\n");
            *errCode = MTD_BAD_HEADER;
            return MTD_FAIL;
        }
        else
        {
#if PRINT_DL_DEBUG
            MTD_DBG_INFO("Processing section %d at offset %d\n",(int)sectionCount,(int)imageIndex);
#endif            
        }
        imageIndex += MTD_HEADER_SIZE;

        basePort = ((port/(devPtr->numPortsPerDevice))*(devPtr->numPortsPerDevice));
        relativePort = port - basePort;
        if (!(fHeader.p_s & (1<<relativePort)))
        {
            if (MTD_IS_COMPHY_DEVICE(devPtr->deviceId))
             {
                MTD_U32 headerAddr;
                MTD_ATTEMPT(mtdGetBootcodeHeaderIdx(devPtr, &port, 1, &bootcodeHeaderIdx));
                headerAddr = (MTD_IS_X3540_FAMILY(devPtr->deviceId) || MTD_IS_X3510_FAMILY(devPtr->deviceId)) ?
                             X35X0_E25X0_HDR_TBL + MTD_HEADER_SIZE * bootcodeHeaderIdx :
                             X36X0_E27X0_HDR_TBL + MTD_HEADER_SIZE * bootcodeHeaderIdx;

                MTD_ATTEMPT(mtdRamDownloadCalcChecksums(devPtr, port, &data[imageIndex - MTD_HEADER_SIZE], 
                                                        MTD_HEADER_SIZE, headerAddr, &byte_checksum, &word_checksum, errCode));

                MTD_ATTEMPT(mtdIncreasBootcodeHeaderIdx(devPtr, &port, 1, &bootcodeHeaderIdx));
            }

            MTD_ATTEMPT(mtdRamDownloadCalcChecksums(devPtr, port, &data[imageIndex], fHeader.dL, fHeader.dD, &byte_checksum, &word_checksum, errCode));

            byte_checksum = (~byte_checksum);
            if (byte_checksum != fHeader.secCh)
            {
                MTD_DBG_ERROR( "Error downloading code. Expected RAM HDR checksum to be 0x%04X but it was 0x%04X on section %d", (int)fHeader.secCh, (int)(byte_checksum), (int)sectionCount);
                *errCode = MTD_HDR_RAM_CHECKSUM_BAD;
                return MTD_FAIL;        
            }

            if (!fHeader.d_o)
            {
                TRY_IO(mtdHwSetPhyRegField(devPtr,port,MTD_BOOT_STATUS_REG,MTD_CODE_DOWNLOADED_POS,1,1));
                
                MTD_ATTEMPT(mtdWait(devPtr,200)); /* Give app code time to start */

                /* make sure the app code started */
                TRY_IO(mtdDidPhyAppCodeStart(devPtr,port,&appStarted))
                
                if (appStarted == MTD_FALSE)
                {
                    MTD_DBG_ERROR("App code did not start.\n");
                    MTD_DBG_ERROR("App download failed.\n");
                    *errCode = MTD_APP_CODE_DID_NOT_START;
                    return MTD_FAIL;
                }
                else
                {
#if PRINT_DL_DEBUG
                    MTD_DBG_INFO("App code started section %d on port %d.\n", (int)sectionCount, (int)port);
#endif
                }                

                if ((fHeader.nHO != 0 && fHeader.nHO != 0xFFFFFFFF) && ((sectionCount+1) < maxSections))
                {
                    if (mtdWaitForRegFieldAltValues(devPtr, port, MTD_TUNIT_XG_EXT_STATUS, 0, 12, MTD_SERDES_LOADER_DONE, MTD_SERDES_LOADER_SKIPPED, MTD_EXECUTABLE_TO) == MTD_FAIL)
                    {
                        *errCode = MTD_EXECECUTABLE_TIMED_OUT;
                        return MTD_FAIL;
                    }

#if PRINT_DL_DEBUG
                    MTD_DBG_INFO("App code finished on section %d on port %d.\n", (int)sectionCount, (int)port);
#endif

                    MTD_ATTEMPT(mtdPutPhyInMdioDownloadMode(devPtr,port)); /* will leave errCode as 0 if fails */
                    
                    MTD_ATTEMPT(mtdWait(devPtr,50)); 
                    
                    TRY_IO(mtdIsPhyInMdioDownloadMode(devPtr,port,&downloadMode));
    
                    if (downloadMode == MTD_FALSE)
                    {
                        MTD_DBG_ERROR("PHY is not in waiting on download mode for section %d.\n", (int)(sectionCount+1));
                        MTD_DBG_ERROR("Download failed\n");
                        *errCode = MTD_PHY_NOT_IN_DOWNLOAD_MODE;
                        return MTD_FAIL;
                    }
                    else
                    {
#if PRINT_DL_DEBUG
                        MTD_DBG_INFO("Downloading code to PHY RAM processing section %d, please wait...\n", (int)(sectionCount+1));
#endif
                    }                                    
                }

            }
        }
        else
        {
#if PRINT_DL_DEBUG
            MTD_DBG_INFO("Section %d skipped for port %d\n", (int)sectionCount, (int)port);
#endif
        }

        if (fHeader.nHO != 0 && fHeader.nHO != 0xFFFFFFFF)
        {
            sectionCount++;
            imageIndex = fHeader.nHO; 
        }
    }while((fHeader.nHO != 0 && fHeader.nHO != 0xFFFFFFFF) && (sectionCount < maxSections));

    if ((sectionCount >= maxSections) && (fHeader.nHO != 0 && fHeader.nHO != 0xFFFFFFFF))
    {
        MTD_DBG_ERROR("Max sections reached but more data is coming. Error in input data.\n");
        *errCode = MTD_TOO_MANY_SECTIONS;
        return MTD_FAIL;
    }

#if PRINT_DL_DEBUG
    MTD_DBG_INFO("\nDownload of code to RAM complete.\n");
#endif

    return MTD_OK;
}

MTD_STATIC MTD_STATUS mtdPutPhyInMdioDownloadModeSharedCodeRam
(
    IN MTD_DEV_PTR devPtr,
    IN MTD_U16 port
)
{
    MTD_U16 i;
    MTD_U16 portNum = 2;

    if (!(MTD_CODE_RAM_SHARED_DEVICE(devPtr)))
    {
        MTD_DBG_ERROR("mtdPutPhyInMdioDownloadModeSharedCodeRam does not supported on this device.\n");
        return MTD_FAIL;
    }

    if (port % portNum != 0)
    {
        MTD_DBG_ERROR("mtdPutPhyInMdioDownloadModeSharedCodeRam does not supported on this port.\n");
        return MTD_FAIL;
    }

    for (i = 0; i < portNum; i++)
    {
        MTD_ATTEMPT(mtdHwSetPhyRegField(devPtr, port + i, 31 , 0xF008, 5, 1, 1));
    }

    MTD_ATTEMPT(mtdHwSetPhyRegField(devPtr, port, MTD_CUNIT_PORT_CTRL, 12, 1, 1));

    return MTD_OK;
}

MTD_STATIC MTD_STATUS mtdRemovePhyMdioDownloadModeSharedCodeRam
(
    IN MTD_DEV_PTR devPtr,
    IN MTD_U16 port
)
{
    MTD_U16 i;
    MTD_U16 portNum = 2;

    if (!(MTD_CODE_RAM_SHARED_DEVICE(devPtr)))
    {
        MTD_DBG_ERROR("mtdRemovePhyMdioDownloadModeSharedCodeRam does not supported on this device.\n");
        return MTD_FAIL;
    }

    if (port % portNum != 0)
    {
        MTD_DBG_ERROR("mtdRemovePhyMdioDownloadModeSharedCodeRam does not supported on this port.\n");
        return MTD_FAIL;
    }

    for (i = 0; i < portNum; i++)
    {
        MTD_ATTEMPT(mtdHwSetPhyRegField(devPtr, port + i, 31 , 0xF008, 5, 1, 0));
    }

    MTD_ATTEMPT(mtdHwSetPhyRegField(devPtr, port, MTD_CUNIT_PORT_CTRL, 12, 1, 1));

    return MTD_OK;
}


MTD_STATIC MTD_STATUS mtdPutMultipleDownloadMode(MTD_DEV_PTR devPtr, MTD_U16 ports[], MTD_U16 numPorts, MTD_U16 erroredPorts[], MTD_U16 *errCode)
{
    MTD_U32 i;
    MTD_BOOL downloadMode = MTD_TRUE;
    MTD_STATUS status = MTD_OK;

    if (numPorts > MTD_MAX_PORTS_TO_DOWNLOAD) 
    {
        MTD_DBG_ERROR("mtdPutMultipleDownloadMode failed: numPorts more than MTD_MAX_PORTS_TO_DOWNLOAD\n");
        return MTD_FAIL;
    }

    /* X3680/E2780 device need to operate in a pair of ports that share the code RAM*/
    if (MTD_CODE_RAM_SHARED_DEVICE(devPtr))
    {
        for(i = 0; i < numPorts; i++)
        {
            if (ports[i] % 2 == 0) 
            {
                TRY_IO(mtdPutPhyInMdioDownloadModeSharedCodeRam(devPtr, ports[i]));
            }
        }
    }
    else
    {
        for(i = 0; i < numPorts; i++)
        {
            TRY_IO(mtdPutPhyInMdioDownloadMode(devPtr,ports[i]));
        }
    }

    /* Allow reset to complete */
    MTD_ATTEMPT(mtdWait(devPtr, 50)); /* if fails, returns errCode of 0 */

    /* Make sure we can access the device And it's in the correct mode (waiting for download) */
    for(i = 0; i < numPorts; i++)
    {
        TRY_IO(mtdIsPhyInMdioDownloadMode(devPtr,ports[i],&downloadMode));
        
        if (downloadMode == MTD_FALSE)
        {
            MTD_DBG_ERROR( "T unit in port %d is not in waiting on download mode.\n",(int)ports[i]);
            MTD_DBG_ERROR( "Download failed on MDIO port %d\n",(int)ports[i]);
            erroredPorts[i] = MTD_PHY_NOT_IN_DOWNLOAD_MODE;
            status = MTD_FAIL;
        }
    } 

    if (status == MTD_FAIL)
    {
        *errCode = MTD_PHY_NOT_IN_DOWNLOAD_MODE;
    }

    return status;
}

MTD_STATIC MTD_STATUS mtdStartMultipleAppCode(MTD_DEV_PTR devPtr, MTD_U16 ports[], MTD_U16 numPorts, MTD_U16 portsSkip, MTD_BOOL waitDone, MTD_U16 sectionCount, MTD_U16 erroredPorts[], MTD_U16 *errCode)
{
    MTD_U16 localPorts[MTD_MAX_PORTS_TO_DOWNLOAD],  
            portsIndexes[MTD_MAX_PORTS_TO_DOWNLOAD], 
            localNumPorts; 
    
    MTD_U16 i, basePort, relativePort;

#if !PRINT_DL_DEBUG
    MTD_UNUSED_PARAMETER(sectionCount);
#endif
    
    if (numPorts > MTD_MAX_PORTS_TO_DOWNLOAD) 
    {
        MTD_DBG_ERROR("mtdStartMultipleAppCode failed: numPorts more than MTD_MAX_PORTS_TO_DOWNLOAD\n");
        return MTD_FAIL;
    }

    for (i=0, localNumPorts=0; i < numPorts; i++)
    {
        basePort = ((ports[i]/(devPtr->numPortsPerDevice))*(devPtr->numPortsPerDevice));
        relativePort = ports[i] - basePort;
        if (!(portsSkip & (1<<relativePort)))
        {
            localPorts[localNumPorts] = ports[i]; 
            portsIndexes[localNumPorts] = i; 
            localNumPorts++;
        }
        else
        {
#if PRINT_DL_DEBUG
            MTD_DBG_INFO("Section %d skipped for port %d for starting app code\n", (int)sectionCount, (int)ports[i]);
#endif
            
        }
    }

    for (i=0; i < localNumPorts; i++)
    {
        TRY_IO(mtdHwSetPhyRegField(devPtr,localPorts[i],MTD_BOOT_STATUS_REG,MTD_CODE_DOWNLOADED_POS,1,1));        
    }
    
    MTD_ATTEMPT(mtdWait(devPtr,200)); /* Give app code time to start */

    for (i=0; i < localNumPorts; i++)
    {
        MTD_BOOL appStarted;
        
        TRY_IO(mtdDidPhyAppCodeStart(devPtr,localPorts[i],&appStarted));

        if (appStarted == MTD_FALSE)
        {

            MTD_DBG_ERROR("App code did not start on port %d.\n", (int)localPorts[i]);
            MTD_DBG_ERROR("App download failed.\n");
            
            if (erroredPorts[portsIndexes[i]] == 0)
            {
                erroredPorts[portsIndexes[i]] = MTD_APP_CODE_DID_NOT_START;
            }
            *errCode = MTD_APP_CODE_DID_NOT_START;
            return MTD_FAIL;
        }
        else
        {
#if PRINT_DL_DEBUG
            MTD_DBG_INFO("App code started section %d on port %d.\n", (int)sectionCount, (int)localPorts[i]);
#endif
        }                
    }

    if (waitDone)
    {
        MTD_BOOL timedOut=MTD_FALSE;

        i=0;
        while(!timedOut && i < localNumPorts)
        {
            if (erroredPorts[portsIndexes[i]] == 0)
            {
                if (mtdWaitForRegFieldAltValues(devPtr, localPorts[i], MTD_TUNIT_XG_EXT_STATUS, 0, 12, MTD_SERDES_LOADER_DONE, MTD_SERDES_LOADER_SKIPPED, MTD_EXECUTABLE_TO) == MTD_FAIL)
                {
                    timedOut = MTD_TRUE;
                    erroredPorts[portsIndexes[i]] = MTD_EXECECUTABLE_TIMED_OUT;
                    *errCode = MTD_EXECECUTABLE_TIMED_OUT;
                    return MTD_FAIL;
                }
            }
            
            i++;
        }

#if PRINT_DL_DEBUG
        MTD_DBG_INFO("App code finished on section %d on all ports not skipped.\n", (int)sectionCount);
#endif

        MTD_ATTEMPT(mtdPutMultipleDownloadMode(devPtr, ports, numPorts, erroredPorts, errCode));
    }

    return MTD_OK;
}

MTD_STATIC MTD_STATUS mtdParRamDownloadCalcChecksums(MTD_DEV_PTR devPtr, MTD_U16 ports[],
                                          MTD_U16 numPorts, MTD_U16 portsSkip, 
                                          MTD_U16 sectionCount, /* for RAM download this is the HDR section, for flash download this is the buffer number */
                                          MTD_U16 erroredPorts[],
                                          MTD_U8 data[], MTD_U32 size, /* size is in bytes, but must be even */
                                          MTD_U32 ramAddress, /* place in RAM to download to */
                                          MTD_U16 *ramChecksumByByte, /* RAM interface or HDR file checksum (inverted) */
                                          MTD_U16 *wordChecksum, /* used by slave bin file */
                                          MTD_U16 *errCode)
{
    MTD_U32 buffCount;
    MTD_U8  lowByte, highByte;
    MTD_U16 byteChecksum, byteChecksumRead, tmp;
    MTD_U16 i;
    MTD_U16 localPorts[MTD_MAX_PORTS_TO_DOWNLOAD],  
            portsIndexes[MTD_MAX_PORTS_TO_DOWNLOAD], 
            localNumPorts; 
    
    MTD_U16 basePort, relativePort;

#if !PRINT_DL_DEBUG
    MTD_UNUSED_PARAMETER(sectionCount);
#endif

    if (numPorts > MTD_MAX_PORTS_TO_DOWNLOAD) {
        MTD_DBG_ERROR("mtdParRamDownloadCalcChecksums failed: numPorts more than MTD_MAX_PORTS_TO_DOWNLOAD\n");
        return MTD_FAIL;
    }

    /* Skip any ports in the portsSkip bitmask */
    /* and on X3680/E2780 skip odd ports always quietly */
    localPorts[0] = 0;
    for (i=0, localNumPorts=0; i < numPorts; i++)
    {
        basePort = ((ports[i]/(devPtr->numPortsPerDevice))*(devPtr->numPortsPerDevice));
        relativePort = ports[i] - basePort;
        if (!(portsSkip & (1 << relativePort)))
        {
            if (MTD_CODE_RAM_SHARED_DEVICE(devPtr))
            {
                if ((relativePort % 2) == 0)
                {
                    localPorts[localNumPorts] = ports[i]; 
                    portsIndexes[localNumPorts] = i;      
                    localNumPorts++;
#if PRINT_DL_DEBUG
                    MTD_DBG_INFO("Writing section/buffer %d to port %d in parallel mode.\n", (int)sectionCount, (int)ports[i]);
#endif

                }
            }
            else
            {
                localPorts[localNumPorts] = ports[i]; 
                portsIndexes[localNumPorts] = i; 
                localNumPorts++;
#if PRINT_DL_DEBUG
                MTD_DBG_INFO("Writing section/buffer %d to port %d in parallel mode.\n", (int)sectionCount, (int)ports[i]);
#endif
            }
        }
        else
        {
#if PRINT_DL_DEBUG
            if (!((MTD_CODE_RAM_SHARED_DEVICE(devPtr)) && (relativePort%2)))
            {
                /* Print this on all PHYS, but on X3680 print it only on the even ports that were skipped */
                MTD_DBG_INFO("Section %d skipped for port %d for downloading to ram\n", (int)sectionCount, (int)ports[i]);
            }
            else
            {
                /* Quietly skip all the odd ports on X3680/E2780 devices. It's normal behavior. */
            }
#endif
            
        }
    }
    
    for(i = 0; i < localNumPorts; i++)
    {
        TRY_IO(mtdHwXmdioRead(devPtr,localPorts[i], MTD_RC_REG,&tmp)); 
    }    

    /* Put all PHYs into parallel MDIO mode, meaning they ignore the MDIO port address */

    /* VERY IMPORTANT: If you have a multi-tasking system that accesses this MDIO bus, you must have a semaphore here */
    /* The MDIO bus cannot be accessed for other devices when these devices are in parallel mode! */
    /* After this point, writes on this MDIO bus will go to all of these devices regardless of MDIO address and reads will */
    /* give bad data as multiple devices will respond. */
    
    for(i = 0; i < localNumPorts; i++)
    {
        TRY_IO(mtdPutPhyInParallelWriteMode(devPtr,localPorts[i]));
    }

    /* VERY IMPORTANT: From here until it's removed can have no MDIO reads on this entire MDIO bus! Otherwise bogus data will be returned since */
    /* multiple devices will respond on the read request. */

    TRY_IO(mtdHwXmdioWrite(devPtr,localPorts[0],MTD_LW_REG,(MTD_U16)ramAddress));
    TRY_IO(mtdHwXmdioWrite(devPtr,localPorts[0],MTD_HW_REG,(MTD_U16)(ramAddress>>16)));

    buffCount=0;
    byteChecksum = 0; 
    *wordChecksum = 0;
    while(buffCount < size)
    {
        lowByte = data[buffCount++];
        highByte = data[buffCount++];
        
        byteChecksum += lowByte; 
        byteChecksum += highByte;

        *wordChecksum += ((((MTD_U16)highByte)<<8)|lowByte); 

        TRY_IO(mtdHwXmdioWrite(devPtr,localPorts[0],MTD_RD_REG,(((MTD_U16)highByte)<<8)|lowByte));  
    }    

    *ramChecksumByByte = byteChecksum;

    /* writing to 1 clears all */
    TRY_IO(mtdRemovePhyFromParallelWriteMode(devPtr,localPorts[0])); 

    /* The semaphore could be removed/unlocked here. */

    for(i = 0; i < localNumPorts; i++)
    {
        TRY_IO(mtdHwXmdioRead(devPtr,localPorts[i],MTD_RC_REG,&byteChecksumRead)); 
        
        if(byteChecksum != byteChecksumRead)   
        {
            MTD_DBG_ERROR( "Error downloading code on MDIO port %d. Expected RAM HW checksum to be 0x%04X but it was 0x%04X",(int)localPorts[i],\
                          (int)byteChecksum, (int)byteChecksumRead);
            if (erroredPorts[portsIndexes[i]] == 0)
            {
                /* only overwrite error if there was no earlier error, we're tracking the first error that happened not the last */
                erroredPorts[portsIndexes[i]] = MTD_RAM_HW_CHECKSUM_ERR;
            }
        }
    }

    return MTD_OK;
}

MTD_STATIC MTD_STATUS mtdGetRelevantPorts(IN MTD_DEV_PTR devPtr, IN MTD_U16 *inports, IN MTD_U16 innumPorts,
                                          IN MTD_U16 SkippedPortBitMap, OUT MTD_U16 *outPorts, OUT MTD_U16 *outPortNumer)
{
    MTD_U16  portNum, i;
    MTD_BOOL bIsRelevant;
    *outPortNumer = 0;

    for (i = 0; i < innumPorts; i++)
    {
        portNum     = inports[i] % devPtr->numPortsPerDevice;
        bIsRelevant = (MTD_CODE_RAM_SHARED_DEVICE(devPtr)) ? IS_RELEVANT_PORT(SkippedPortBitMap, portNum) && IS_EVEN_PORT(portNum)
                                                  : IS_RELEVANT_PORT(SkippedPortBitMap, portNum);
        if (bIsRelevant)
        {
            outPorts[(*outPortNumer)++] = inports[i];
            
        }
    }

    return MTD_OK;
}

MTD_STATIC MTD_STATUS mtdParallelMdioRamDownload(MTD_DEV_PTR devPtr,MTD_U8 data[],MTD_U32 size,
                                      MTD_U16 ports[], MTD_U16 numPorts, MTD_U16 erroredPorts[],
                                      MTD_U16 *errCode) 
{
    MTD_U16 word_checksum, byte_checksum, sectionCount = 0, maxSections, numRelevantPort, i;
    MTD_U32 imageIndex = 0, curHeaderAddr;
    MTD_FILE_HEADER_TYPE fHeader;
    MTD_U16 relevantPorts[MTD_MAX_PORTS_TO_DOWNLOAD];
    MTD_U16 bootcodeHeaderIdxs[MTD_MAX_PORTS_TO_DOWNLOAD];

    *errCode = 0;

    if (numPorts > MTD_MAX_PORTS_TO_DOWNLOAD) {
        MTD_DBG_ERROR("mtdParallelMdioRamDownload failed: numPorts more than MTD_MAX_PORTS_TO_DOWNLOAD\n");
        return MTD_FAIL;
    }

    mtdMemSet(&fHeader, 0, sizeof(MTD_FILE_HEADER_TYPE));
    if ((MTD_IS_COMPHY_DEVICE(devPtr->deviceId)))
    {
        maxSections = MTD_MAX_X35X0_HEADERS;
    }
    else
    {
        maxSections = MTD_MAX_HEADERS;
    }

    if (size%2)
    {
        /* it's an error, size must be an even number of bytes */
        *errCode = MTD_SIZE_NOT_EVEN;
        return MTD_FAIL;
    }    

    MTD_ATTEMPT(mtdPutMultipleDownloadMode(devPtr, ports, numPorts, erroredPorts, errCode));

    do
    {
        if (imageIndex > size)
        {
            MTD_DBG_ERROR("Image size and image data do not match. Went past end of image.\n");
            /* went past end of buffer, host passed in wrong size */
            *errCode = MTD_ERR_PROCESSING_IMAGE_FILE;
            return MTD_FAIL;                    
        }        

        /* Now process one or more headers */
        if (mtdGetHeader(&fHeader, &data[imageIndex]) == MTD_FALSE)
        {
            MTD_DBG_ERROR("Header corrupted\n");
            *errCode = MTD_BAD_HEADER;
            return MTD_FAIL;
        }
        else
        {
#if PRINT_DL_DEBUG
            MTD_DBG_INFO("Processing section %d at offset %d\n",(int)sectionCount,(int)imageIndex);
#endif            
        }

        if (MTD_IS_COMPHY_DEVICE(devPtr->deviceId))
        {
            /* clean up temporary buffer */
            mtdMemSet(relevantPorts, 0, sizeof(relevantPorts));
            mtdMemSet(bootcodeHeaderIdxs, 0, sizeof(bootcodeHeaderIdxs));

            MTD_ATTEMPT(mtdGetRelevantPorts(devPtr, ports, numPorts, fHeader.p_s, relevantPorts, &numRelevantPort));
            MTD_ATTEMPT(mtdGetBootcodeHeaderIdx(devPtr, relevantPorts, numRelevantPort, bootcodeHeaderIdxs))

            /* write the header to each port*/
            for (i = 0; i < numRelevantPort; i++)
            {
                curHeaderAddr = (MTD_IS_X3540_FAMILY(devPtr->deviceId) || MTD_IS_X3510_FAMILY(devPtr->deviceId)) ?
                                X35X0_E25X0_HDR_TBL + MTD_HEADER_SIZE * bootcodeHeaderIdxs[i]:
                                X36X0_E27X0_HDR_TBL + MTD_HEADER_SIZE * bootcodeHeaderIdxs[i];

                MTD_ATTEMPT(mtdRamDownloadCalcChecksums(devPtr, relevantPorts[i], &data[imageIndex], MTD_HEADER_SIZE, 
                                                        curHeaderAddr, &byte_checksum, &word_checksum, errCode));

            }

            MTD_ATTEMPT(mtdIncreasBootcodeHeaderIdx(devPtr, relevantPorts, numRelevantPort, bootcodeHeaderIdxs));
        }
        imageIndex += MTD_HEADER_SIZE;

        MTD_ATTEMPT(mtdParRamDownloadCalcChecksums(devPtr, ports, numPorts, fHeader.p_s, sectionCount, erroredPorts, &data[imageIndex], fHeader.dL, fHeader.dD, &byte_checksum, &word_checksum, errCode));

        byte_checksum = (~byte_checksum);
        if (byte_checksum != fHeader.secCh)
        {
            MTD_DBG_ERROR( "Error downloading code. Expected RAM HDR checksum to be 0x%04X but it was 0x%04X on section %d", (int)fHeader.secCh, (int)(byte_checksum), (int)sectionCount);
            *errCode = MTD_HDR_RAM_CHECKSUM_BAD;
            return MTD_FAIL;        
        }

        if (!fHeader.d_o)
        {
            MTD_BOOL moreSectionsComing = (fHeader.nHO != 0 && fHeader.nHO != 0xFFFFFFFF) && ((sectionCount+1) < maxSections);

#if PRINT_DL_DEBUG
            MTD_DBG_INFO("Executing code in section %d on some ports\n", (int)(sectionCount));
#endif
            MTD_ATTEMPT(mtdStartMultipleAppCode(devPtr, ports, numPorts, fHeader.p_s, moreSectionsComing, sectionCount, erroredPorts, errCode));
        }
        
        if (fHeader.nHO != 0 && fHeader.nHO != 0xFFFFFFFF)
        {
            sectionCount++;
            imageIndex = fHeader.nHO; 
        }        
    }while((fHeader.nHO != 0 && fHeader.nHO != 0xFFFFFFFF) && (sectionCount < maxSections));
    

    if ((sectionCount >= maxSections) && (fHeader.nHO != 0 && fHeader.nHO != 0xFFFFFFFF))
    {
        /* there's something wrong with the file, maximum section number reached but more sections are coming */
        MTD_DBG_ERROR("Max sections reached but more data is coming. Error in input data.\n");
        *errCode = MTD_TOO_MANY_SECTIONS;
        return MTD_FAIL;
    }

#if PRINT_DL_DEBUG
    MTD_DBG_INFO("\nDownload of code to RAM complete.\n";
#endif

    return MTD_OK;
}


