/* 
 Copyright (c) 2014-2024 Marvell International Ltd., All Rights Reserved
 Redistribution and use in source and binary forms, with or without modification, 
 are permitted provided that the following conditions are met:

 1. Redistributions of source code must retain the above copyright notice, this list 
    of conditions and the following disclaimer.
 2. Redistributions in binary form must reproduce the above copyright notice, this list 
    of conditions and the following disclaimer in the documentation and/or other materials 
    provided with the distribution.
    
 THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY EXPRESS 
 OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY 
 AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR 
 CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR 
 CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR 
 SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY 
 THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR 
 OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE 
 POSSIBILITY OF SUCH DAMAGE.
*/

/********************************************************************
This file contains functions prototypes and global defines/data for
higher-level functions using MDIO access to download firmware
into the internal RAM or attached flash memory of the Marvell 88X32X0, 
88X33X0, 88X35X0,  CUX36X0, CUE26X0, 88E20X0, 88E21X0 
and CUE2780 ethernet PHYs.
********************************************************************/

#ifndef MTDDL_H
#define MTDDL_H

#if C_LINKAGE
#if defined __cplusplus 
    extern "C" { 
#endif 
#endif

#define MTD_MAX_PORTS_TO_DOWNLOAD 32  /* this is the maximum number of ports that can be downloaded */
                                      /* in parallel ram or parallel flash download functions */
                                      /* which happens to be the same as the maximum number of ports on one MDIO bus */


/* On X3540/E2540, firmware download must be done together on grouped ports(P0-P3) */
/* On X3680/E2680, firmware download must be done together on all 8 ports on the device */
#define MTD_NON_GROUPED_RAM_DOWNLOAD         0xFFD5
/* Firmware download is required on both of the grouped port for X3540/E2540 devices, but grouped not found in the list */
/* Firmware download is required on all of the ports on the X3680/E2680, but some port(s) in the device were not listed */
#define MTD_GROUPED_PORTS_NOT_FOUND           0xFFD6
/* Max sections number reached without end-of-file indication */
#define MTD_TOO_MANY_SECTIONS               0xFFD7
/* Intermediate executable in a multi-stage boot took too long to finish execution (see MTD_EXECUTABLE_TO) */
#define MTD_EXECECUTABLE_TIMED_OUT          0xFFD8
/* Image passed in and size do not match each other, image is probably corrupted or wrong size passed */
#define MTD_ERR_PROCESSING_IMAGE_FILE       0xFFD9
/* Image header is corrupted */
#define MTD_BAD_HEADER                      0xFFDA
/* Slave command timed out */
#define MTD_SLAVE_CMD_TIMEOUT               0xFFDC
/* Header file download to RAM checksum doesn't match expected checksum in header */
#define MTD_HDR_RAM_CHECKSUM_BAD            0xFFDD
/* At least one port during a parallel RAM or flash download failed */
#define MTD_PAR_DOWNLOAD_FAILED             0xFFDE
/* Image larger than max image size to store to flash or the slave code is too large */
#define MTD_IMAGE_TOO_LARGE_TO_DOWNLOAD     0xFFDF
/* Slave code did not start. - Slave code failed to download properly. */
#define MTD_SLAVE_CODE_DID_NOT_START        0xFFE0
/* Flash verified FAILED! Flash probably corrupted */
#define MTD_VERIFY_ERR                      0xFFE1
/* Unknown error, downloading the flash failed! */
#define MTD_UNKNOWN_DOWNLOAD_TO_FLASH_FAIL  0xFFE2
/* App code did not start. - App code failed to download properly in to the RAM. */
#define MTD_APP_CODE_DID_NOT_START          0xFFE3
/* App code failed. - Number of ports to download is greater than maximum ports */
#define MTD_NUM_PORTS_TOO_LARGE             0xFFE4

/* Error reading or writing MDIO register */
#define MTD_IO_ERROR                        0xFFE5
/*size must be an even number of bytes*/
#define MTD_SIZE_NOT_EVEN                   0xFFE6
/* Slave encountered error while erasing flash */
#define MTD_ERR_ERASING_FLASH               0xFFE7
/* unexpected value read back from download code */
#define MTD_ERR_VALUE_READ_BACK             0xFFE8
/* Did not get MTD_SLAVE_OK for writing the data */
#define MTD_ERR_START_WRITE_DATA            0xFFE9
/* Slave failed to get all the data correctly*/
#define MTD_WRITE_VERIFY_DATA                0xFFEA
/* Some kind of error occurred on Slave */
#define MTD_ERR_ON_SLAVE                    0xFFEB
/* Checksum error */
#define MTD_ERR_CHECKSUM                    0xFFEC
/* Slave didn't write enough words to flash. Some kind of error occurred*/
#define MTD_ERR_SLAVE_WRITE_FULL            0xFFED
/* last transfer failed */
#define MTD_ERR_LAST_TRANSFER               0xFFEE
/* RAM checksum register (present only in SFT910X RevB or later) had wrong checksum */
#define MTD_RAM_HW_CHECKSUM_ERR             0xFFEF
/* PHY wasn't waiting in download mode */
#define MTD_PHY_NOT_IN_DOWNLOAD_MODE        0xFFFF

/******************************************************************************
 MTD_STATUS mtdUpdateRamImage(
     IN MTD_DEV_PTR devPtr,
     IN MTD_U16 port,
     IN MTD_U8 appData[],
     IN MTD_U32 appSize
     OUT MTD_U16 *errCode);
                                  
 Inputs:
    devPtr - pointer to MTD_DEV initialized by mtdLoadDriver() call
    port - MDIO port address, 0-31
    appData[] - application code to be downloaded into the RAM
    appSize - application code size in bytes (must be even)

 Outputs:
    errCode - if return status is MTD_FAIL, errCode may contain additional
    information about the error. If it does, it will be one of the following:

         MTD_NON_GROUPED_RAM_DOWNLOAD
         MTD_TOO_MANY_SECTIONS
         MTD_EXECECUTABLE_TIMED_OUT
         MTD_ERR_PROCESSING_IMAGE_FILE
         MTD_BAD_HEADER
         MTD_HDR_RAM_CHECKSUM_BAD
         MTD_APP_CODE_DID_NOT_START
         MTD_IO_ERROR
         MTD_SIZE_NOT_EVEN
         MTD_RAM_HW_CHECKSUM_ERR
         MTD_PHY_NOT_IN_DOWNLOAD_MODE

 Returns:
    MTD_OK if successful, MTD_FAIL if an error was detected.

 Description:
    The function first checks if the app code size is greater than what it 
    should be and returns an error if that was the case (or if the image size
    is not even, and error will also be returned).

    It calls mtdPutPhyInMdioDownloadMode() to put the port in download mode.

    Then it downloads the app code to the RAM and starts it executing. Returns
    MTD_FAIL if any error is encountered, with possibly more information in "errCode".
    
 Side effects:
    None

 Notes/Warnings:
    This function is not applicable for X35X0/E25X0 (4 and 8 port versions), 
    X3680/E2680 and E2780 devices. Call mtdParallelUpdateRamImage() instead. 
    One-port-at-a-time RAM download is not supported on these multi-port devices.

Example:
    FILE *hFile;
    MTD_U8 appData[MTD_MAX_X35X0_FLASH_IMAGE_SIZE];  // Could use MTD_MAX_FLASH_IMAGE_SIZE instead
                                                     // if using PHYs before X35X0
    MTD_U16 error;
    MTD_STATUS retStatus;
    
    fileSize = mtdOpenReadFile(devPtr,port, &hFile, "image_name.hdr", appData,
                                    MTD_MAX_X35X0_FLASH_IMAGE_SIZE); 
                                    // Could use MTD_MAX_FLASH_IMAGE_SIZE instead
                                    // if using PHYs before X35X0
    retStatus = mtdUpdateRamImage(devPtr,port, appData, fileSize, &error);
    if (retStatus == MTD_FAIL)...check "error" for possible error codes    

    fclose(hFile);
******************************************************************************/
MTD_STATUS mtdUpdateRamImage(
    IN MTD_DEV_PTR devPtr,
    IN MTD_U16 port,
    IN MTD_U8 appData[],
    IN MTD_U32 appSize,
    OUT MTD_U16 *errCode);

/******************************************************************************
 MTD_STATUS mtdParallelUpdateRamImage(
     IN MTD_DEV_PTR devPtr,
     IN MTD_U16 ports[],
     IN MTD_U8 appData[],
     IN MTD_U32 appSize, 
     IN MTD_U16 numPorts,
     OUT MTD_U16 erroredPorts[]
     OUT MTD_U16 *errCode);
                                  

 Inputs:
    devPtr - pointer to MTD_DEV initialized by mtdLoadDriver() call
    ports[] - list of port numbers to which the application code is to be downloaded
              On X3540/E2540 devices, both port in the port group(P0-1, P2-3)
              must be passed in.
    appData[] - application code to be downloaded into RAM
    appSize - application code size in bytes (must be even)
    numPorts - number of ports to download (any number from 0-31)

 Outputs:
    errCode - If return status is MTD_FAIL, errCode may contain one of the following:

        MTD_GROUPED_PORTS_NOT_FOUND
        MTD_TOO_MANY_SECTIONS
        MTD_EXECECUTABLE_TIMED_OUT
        MTD_ERR_PROCESSING_IMAGE_FILE
        MTD_BAD_HEADER
        MTD_PAR_DOWNLOAD_FAILED (one or more ports failed)
        MTD_HDR_RAM_CHECKSUM_BAD
        MTD_APP_CODE_DID_NOT_START
        MTD_NUM_PORTS_TOO_LARGE
        MTD_IO_ERROR
        MTD_SIZE_NOT_EVEN
        MTD_PHY_NOT_IN_DOWNLOAD_MODE


    erroredPorts[] - Failure code on ports that had error during download.

        The following are the failure codes stored in the array erroredPorts[]

        MTD_EXECECUTABLE_TIMED_OUT
        MTD_RAM_HW_CHECKSUM_ERR
        MTD_PHY_NOT_IN_DOWNLOAD_MODE


 Returns:
    Download successful on all ports - MTD_OK
    Download not successful on at least one port - MTD_FAIL

    
 Description:
    The function first checks if the number of ports to download is greater
    than maximum ports to download and if true returns an error.
    
    The code then checks if the app code size is greater than what it 
    should be and returns an error if that was the case (or code size is
    not even will also return an error).

    Then it puts all the ports into the download mode by calling
    mtdPutPhyInMdioDownloadMode().
    
    Then it downloads the app code to the RAM in parallel and starts it executing. 

    Makes sure code started.

 Side effects:
    None.

 Notes/Warnings:    
    If this function returns an error in errCode, then it should be assumed
    possibly no ports were downloaded successfully.

    If this function returns MTD_FAIL, the chip(s) should be hardware reset to 
    recover, because one or more ports may be left in parallel download mode.

    If MTD_PAR_DOWNLOAD_FAILED is returned, the erroredPorts[] list will
    contain the ports that had an error (non-zero value) and the ports
    that have a 0 value were loaded successfully.

    On X3540/E2540 devices, the new firmware must be downloaded to paired
    ports first/second, or third/fourth (e.g. P0/P1 or P2/P3) because these
    ports share the same serdes.

    X3580/E2580 have special requirements for RAM download. Please see
    mtdSampleUpdateRamImageX3580() for a description how to do the RAM
    download for X3580/E2580.

    On X3680/E2680/E2780 devices, all 8 ports on a device must be downloaded at
    once, otherwise this function will return an error.

Example:
    FILE *hFile;
    MTD_U8 appData[MTD_MAX_APP_SIZE+MTD_HEADER_SIZE];        // hdr file image
    MTD_U16 error;
    MTD_STATUS retStatus;
    const MTD_U16 ports[4] = {0,1,2,3}; // Will download MDIO ports 0...3 in parallel
    
    fileSize = mtdOpenReadFile(devPtr,port, &hFile, "image_name.hdr", appData,
                                    (MTD_MAX_APP_SIZE+MTD_HEADER_SIZE));
                                    
    retStatus = mtdParallelUpdateRamImage(devPtr,ports, 
                                          appData, fileSize, 4,
                                          erroredPorts, &error);
    if (retStatus == MTD_FAIL)
    {
        ...check error, and/or all ports in erroredPorts for failure codes
    }
    fclose(hFile);

******************************************************************************/
MTD_STATUS mtdParallelUpdateRamImage
(
    IN MTD_DEV_PTR devPtr,
    IN MTD_U16 ports[],
    IN MTD_U8 appData[],
    IN MTD_U32 appSize, 
    IN MTD_U16 numPorts,
    OUT MTD_U16 erroredPorts[],
    OUT MTD_U16 *errCode
);

#if C_LINKAGE
#if defined __cplusplus
}
#endif
#endif

#endif /* defined MTDDL_H */

