/* 
 Copyright (c) 2014-2024 Marvell International Ltd., All Rights Reserved
 Redistribution and use in source and binary forms, with or without modification, 
 are permitted provided that the following conditions are met:

 1. Redistributions of source code must retain the above copyright notice, this list 
    of conditions and the following disclaimer.
 2. Redistributions in binary form must reproduce the above copyright notice, this list 
    of conditions and the following disclaimer in the documentation and/or other materials 
    provided with the distribution.

 THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY EXPRESS 
 OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY 
 AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR 
 CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR 
 CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR 
 SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY 
 THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR 
 OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE 
 POSSIBILITY OF SUCH DAMAGE.
*/

/********************************************************************
 This file contains sample functions showing how to use
 the Open Source MTD API to control and get status from the
 Marvell 88X33X0 and 88E21X0 PHYs.

 Most samples assume a system composed of all 88X3340 A1. If a system
 was composed of different devices, one MTD_DEV for each device
 would need to be declared and a pointer passed to the API function
 to control that device. Sample initialized MTD_DEV structure
 definitions for each supported device are provided in 
 openSourceMtdApi.c.

 There are some samples that are applicable to the E2110/E2180. Those
 samples use those device structures.
********************************************************************/
#include <stdio.h>
#include <stdlib.h>
#include <malloc.h>
#include <string.h>

#include "openSourceMtdAPITypes.h"
#include "openSourceMtdUser.h"
#include "openSourceMtdAPI.h"
#include "openSourceMtdAPIDevices.h"
#include "openSourceMtdFwDownload.h"


typedef struct _MTD_FWIMAGES
{
    MTD_U32 deviceIdentifier;
    char * imageName;
} MTD_FWIMAGES_STRUCT;

#define MTD_NUM_FW_IMAGES 5

const MTD_FWIMAGES_STRUCT fwTestImages[MTD_NUM_FW_IMAGES] =
{
    {MTD_E2780A_A0, "v0E000201_13343_13335_e2780.hdr"},
    {MTD_X3580_B3, "v0A0F0701_13284_12929_x3580.hdr"},
    {MTD_X3540_B0, "v0A0F0701_13284_12929_x3540.hdr"},
    {MTD_E2180_A0, "e21x1fw_8_5_0_1_12934.hdr"},
    {MTD_X3340_A1, "x33x0fw_0_3_11_1_12933.hdr"}
};

MTD_DEV_PTR deviceUnderTest = &x3340deviceA1; /* To run sample code, select a device here.*/

/******************************************************************************
MTD_STATUS mtdSamplePrintFirmwareVersionAllPorts
(
    IN MTD_DEV_PTR devPtr, 
    IN MTD_U16 port, 
    IN const char *functionName
)


 Inputs:
    devPtr - pointer to the device structure
    port - MDIO port address, 0-31 
    functionName - pointer to the name of the calling funtion

 Outputs:
    none

 Returns:
    MTD_OK - this function always returns OK unless one of the API function calls
    fails, then it returns MTD_FAIL

 Description:
    This function goes through all the ports and gets the firmware revision
    and prints it out.

 Side effects:
    None

 Notes/Warnings:
    None                                                                
    
*******************************************************************************/
MTD_STATUS mtdSamplePrintFirmwareVersionAllPorts
(
    IN MTD_DEV_PTR devPtr, 
    IN MTD_U16 port, 
    IN const char *functionName
)
{


    MTD_U8 major, minor, inc, test;
    MTD_U8 portIndex;

    for(portIndex = 0; portIndex < devPtr->numPortsPerDevice; portIndex++)
    {
        MTD_ATTEMPT(mtdGetFirmwareVersion(devPtr, port, &major, &minor, &inc, &test));
        MTD_DBG_INFO("%s: Firmware version: %d.%d.%d.%d\n", functionName, major, minor, inc, test);
    }

    return MTD_OK;
}


/******************************************************************************
MTD_STATUS mtdSampleParallelDownloadFirmware
(
    IN MTD_U16 port
)

 Inputs:
    port - MDIO port address, 0-31

 Outputs:
    None

 Returns:
    Returns MTD_OK or MTD_FAIL if multi-port parallel RAM update failed.
    
 Description:
    This sample test shows how the application code is downloaded to RAM via MDIO
    for multi-port PHYs which require the user to use the parallel RAM download
    function. 

    It shows the operations to open and read the binary file, get the image data size 
    and loads the binary data to be downloaded into the PHY. 

    The mtdParallelUpdateRamImage() is called with the above inputs. After the 
    mtdParallelUpdateRamImage() call is completed, it checks the firmware version.

    Please see the Notes/Warnings section below for more details.
    
 Side effects:
    None

 Notes/Warnings:
    Single port PHYs can be downloaded individually or many in parallel (as long 
    as they are on the same MDIO bus). Newer multi-port PHYs (X3540 and newer)
    require all ports to be downloaded in parallel due to the internal 
    architecture of the PHYs. Multiple chips can be downloaded in parallel as
    well.

    This sample code does not show how to download single ports on older PHYs
    nor single ports of new PHYs. The function mtdUpdateRamImage() may be 
    called directly for these single port downloads.

    The X3580/E2580 has a special download algorithm and the upper 4 ports of
    each device must be downloaded first followed by the lower 4 ports second. 
    For multiple X3580/E2580 the upper 4 ports of all devices should be downloaded
    first in parallel, then the lower 4 ports of all devices should be downloaded
    next if multiple PHYs are to be loaded at once. For multiple devices,
    upper 4 then lower 4 on each device can be done in series if desired. See
    the example below for how to handle the case of the X3580/E2580.

    For X3580/E2580:
    1. Chip reset all upper 4 ports
    2. 10ms delay needed before any MDIO operation after this (IMPORTANT)
    3. Parallel ram download all upper 4 ports
    4. Start code on upper 4 ports
    5. Chip reset applied to lower 4 ports
    6. 10ms delay needed before any MDIO operation after this (IMPORTANT)
    7. Parallel ram download all lower 4 ports
    8. Start code on lower 4 ports    

    NOte that not all error paths free the memory allocated! MTD_ATTEMPT(), if
    failed, does not free memory allocated here. Suggest to fix this. :)

*******************************************************************************/
MTD_STATUS mtdSampleParallelDownloadFirmware
(
    IN MTD_U16 port
)
{    
    MTD_DEV_PTR devPtr = deviceUnderTest;
    MTD_U16 errorCode, filenameIndex=0;
    MTD_U8 *appData = NULL;
    MTD_U32 appSize = 0;
    FILE *file;
    MTD_BOOL filenameFound = MTD_FALSE;
    MTD_U16 aryIndex, portIndex, 
            portList[MTD_MAX_PORTS_TO_DOWNLOAD], errorPortList[MTD_MAX_PORTS_TO_DOWNLOAD];
    

    /* check for a single port PHY and older multi-port which can be downloaded per port */
    /* for these PHYs return MTD_OK without doing anything and print a message that */
    /* mtdUpdateRamImage() can be used directly on those devices and this sample is skipped */
    if (devPtr->numPortsPerDevice == 1)
    {
        MTD_DBG_INFO("%s: Test SKIPPED on single port devices.\n", __FUNCTION__);
        MTD_DBG_INFO("    Run UpdateSinglePortRamImage.py for these phys.\n", __FUNCTION__);
        return MTD_OK;
    }

    if ((port % devPtr->numPortsPerDevice) != 0)
    {
        MTD_DBG_ERROR("%s: Port number must be base port of device\n", __FUNCTION__);
        return MTD_FAIL;
    }    

    /* Find the image file name for this PHY. */
    while(filenameIndex < MTD_NUM_FW_IMAGES && !filenameFound)
    {
        filenameFound = (fwTestImages[filenameIndex].deviceIdentifier == devPtr->deviceId);

        if (!filenameFound)
        {
            filenameIndex++;
        }
    }

    if (!filenameFound)
    {
        MTD_DBG_ERROR("%s: Firmware image file name not found for this PHY\n", __FUNCTION__);
        return MTD_FAIL;
    }
    else
    {
        MTD_DBG_INFO("%s: Downloading firmware image %s\n", __FUNCTION__, fwTestImages[filenameIndex].imageName);
    }

    /* Make sure the file is opened in "rb"(read+binary) mode */ 
    file = fopen(fwTestImages[filenameIndex].imageName, "rb");
    if (!file)
    {
        MTD_DBG_ERROR("%s: Couldn't open file %s.\n", __FUNCTION__, fwTestImages[filenameIndex].imageName);                
        return MTD_FAIL;
    }
    fseek(file, 0, SEEK_END);

    /* get the application code image size needed for the mtdUpdateRamImage() */
    appSize = (MTD_U32)ftell(file);
    rewind(file);

    if( appSize == 0 )
    {
        fclose(file);
        MTD_DBG_ERROR("%s: File is empty\n", __FUNCTION__);                
        return MTD_FAIL;
    }

    appData = (MTD_U8*)malloc(appSize + 1);
    if (appData == NULL)
    {
        MTD_DBG_ERROR("%s: Failed to allocate memory\n", __FUNCTION__);                
        return MTD_FAIL;
    }

    fread(appData, appSize, 1, file);
    fclose(file);
    appData[appSize] = 0; /* Add null termination to image */   

    MTD_DBG_INFO("%s: Firmware version before the download:\n", __FUNCTION__);
    MTD_ATTEMPT(mtdSamplePrintFirmwareVersionAllPorts(devPtr, port, __FUNCTION__));

    /* X3580 needs special handling, downloading lower 4 ports first, start the code, then upper 4 ports */
    /* other PHYs download all ports */
    if (fwTestImages[filenameIndex].deviceIdentifier == MTD_X3580_A0 || 
        fwTestImages[filenameIndex].deviceIdentifier == MTD_X3580_B3)
    {        
        /* chip reset upper ports */
        if (mtdHwXmdioWrite(devPtr,port+((devPtr->numPortsPerDevice))/2,MTD_CUNIT_PORT_CTRL,(1<<14)) == MTD_FAIL) /* If multiple-devices, should do all. This resets upper and lower. */
        {                                                                                                           /* Very important, delay needed after this or will fail */
            MTD_DBG_ERROR("%s: Chip reset on upper ports failed\n", __FUNCTION__);                            
            free(appData);
            return MTD_FAIL;
        }      
        
        MTD_ATTEMPT(mtdWait(devPtr,10)); /* Wait 10ms before any MDIO access, should check status for failure */
        
        /* Download upper 4 ports first */
        for (aryIndex = 0, portIndex = port + (devPtr->numPortsPerDevice)/2; 
             portIndex < (port + devPtr->numPortsPerDevice); 
             aryIndex++,portIndex++)
        {
            portList[aryIndex] = portIndex; /* Build list of ports for upper half of device */
        }
        
        if (mtdParallelUpdateRamImage(devPtr, portList, appData, appSize, (devPtr->numPortsPerDevice)/2, errorPortList, &errorCode) == MTD_OK)
        {
            MTD_DBG_INFO("%s: Download upper ports success\n", __FUNCTION__);        
        }
        else
        {
            MTD_DBG_ERROR("%s: Download upper ports failed, error code was 0x%04X\n", __FUNCTION__, errorCode);
            free(appData);
            return MTD_FAIL;    
        }
        
        
        /* Chip reset lower ports */
        if(mtdHwXmdioWrite(devPtr,port,MTD_CUNIT_PORT_CTRL,(1<<14)) == MTD_FAIL) /* Very important, delay needed after this or will fail */
        {
            MTD_DBG_ERROR("%s: Chip reset on lower ports failed\n", __FUNCTION__);
            free(appData);
            return MTD_FAIL;  
        }
        
        MTD_ATTEMPT(mtdWait(devPtr,10)); /* Wait 10ms before any MDIO access, should check status for failure */    
        
        /* Download lower 4 ports next */
        for (aryIndex = 0, portIndex = port; 
             portIndex < port + (devPtr->numPortsPerDevice)/2; 
             aryIndex++,portIndex++)
        {
            portList[aryIndex] = portIndex; /* Build list of ports for lower half of device */
        }
        
        if (mtdParallelUpdateRamImage(devPtr, portList, appData, appSize, (devPtr->numPortsPerDevice)/2, errorPortList, &errorCode) == MTD_OK)
        {
            MTD_DBG_INFO("%s: Download lower ports success\n", __FUNCTION__);        
        }
        else
        {
            MTD_DBG_ERROR("%s: Download lower ports failed, error code was 0x%04X\n", __FUNCTION__, (int)errorCode);
            free(appData);
            return MTD_FAIL;    
        }        
    }
    else
    {
        /* All other devices, download all ports */
        for (aryIndex = 0, portIndex = port; 
             portIndex < port + devPtr->numPortsPerDevice; 
             aryIndex++,portIndex++)
        {
            portList[aryIndex] = portIndex; /* Build list of all ports on device */
        }
        
        if (mtdParallelUpdateRamImage(devPtr, portList, appData, appSize, devPtr->numPortsPerDevice, errorPortList, &errorCode) == MTD_OK)
        {
            MTD_DBG_INFO("%s: Download %d ports success\n", __FUNCTION__, (int)devPtr->numPortsPerDevice);        
        }
        else
        {
            MTD_DBG_ERROR("%s: Download lower ports failed, error code was 0x%04X\n", __FUNCTION__, (int)errorCode);
            free(appData);
            return MTD_FAIL;    
        }            
    }
    
    MTD_DBG_INFO("%s: Firmware version after the download:\n", __FUNCTION__);
    MTD_ATTEMPT(mtdSamplePrintFirmwareVersionAllPorts(devPtr, port, __FUNCTION__));

    free(appData);
        
    return MTD_OK;
}


/******************************************************************************
 MTD_STATUS mtdSampleEnableSpeed
 ( 
     IN MTD_U16 port
 )


 Inputs:
    port - MDIO port address of the near-end port, 0-31

 Outputs:
    None

 Returns:
    Returns MTD_OK or MTD_FAIL if any call fails or the link doesn't come up
    
 Description:
    This sample test shows how to enable a single advertised speed on the
    copper, restart auto-negotiation and poll for copper link up.

    Alternatively, the link up interrupt could be used to generate an
    interrupt whenever the copper link (or serdes host-side link) changes
    state and then the corresponding function could be called to 
    determine if the link is up or down.

    In order for data to pass through the PHY, both copper side and
    host side serdes link must be up.

    mtdIsBaseTUp() can be used to check the copper side link status and speed

    For the host side, it depends on the host interface configuration and
    speed what function to use to check the host side serdes link status.

    For 2.5G/5G/10G use mtdGet10GBRStatus1() to check the host side serdes
    link status. For 1G and below use mtdGet1000BXSGMIIStatus().
    
 Side effects:
    None

 Notes/Warnings:
    Assumes far end PHY is advertising 1G.

*******************************************************************************/
MTD_STATUS mtdSampleEnableSpeed
( 
    IN MTD_U16 port
)
{    
    MTD_DEV_PTR devPtr = deviceUnderTest;
    MTD_U16 portSpeed;
    MTD_BOOL portLinkUp;
    MTD_U16 speed_to_try = MTD_SPEED_1GIG_FD; /* enable a single speed, "|" multiple together to enable multiple speeds */
    MTD_U16 speeds_enabled;
    MTD_U16 timeCount = 0;
        
    /* enable 1G speed on port with mtdAutonegRestart*/
    MTD_ATTEMPT(mtdEnableSpeeds(devPtr, port, speed_to_try, MTD_TRUE)); 

    MTD_DBG_INFO("mtdSampleEnableSpeed: Testing speed :0x%x\n\n", speed_to_try);

    MTD_ATTEMPT(mtdGetSpeedsEnabled(devPtr, port, &speeds_enabled)); /* read back speed enabled */
    if (speeds_enabled != speed_to_try)
    {
        MTD_DBG_ERROR("Get Enable Speed failed\n");               
        return MTD_FAIL;
    }

    /* Poll mtdIsBaseTUp() for linkup every 0.5 second for up to 10 seconds */
    do
    {
        timeCount++;
        MTD_ATTEMPT(mtdWait(devPtr, 500)); 
        MTD_ATTEMPT(mtdIsBaseTUp(devPtr, port, &portSpeed, &portLinkUp));
    } while ((portLinkUp == MTD_FALSE) && (timeCount < 20));

    if (portSpeed != speed_to_try || portLinkUp != MTD_TRUE)
    {
        MTD_DBG_ERROR("mtdSampleEnableSpeed: Copper link failed to come up at 1G speed.\n");                
        return MTD_FAIL;
    }
    else
    {
        MTD_DBG_INFO("mtdSampleEnableSpeed: Copper link up at 1G.\n");
    }

    MTD_DBG_INFO("mtdSampleEnableSpeed: Enable Speed Test completed successfully\n");
    MTD_DBG_INFO("mtdSampleEnableSpeed: Speed set to: 0x%X\n", portSpeed);
    MTD_DBG_INFO("mtdSampleEnableSpeed: Link up in %d seconds\n", timeCount/2);
        
    return MTD_OK;
}


/******************************************************************************
MTD_STATUS mtdSampleForceSpeed
(
    IN MTD_U16 nearEndPort,
    IN MTD_U16 farEndPort
)

 Inputs:
    nearEndPort - MDIO port address of the near-end port, 0-31
    farEndPort - MDIO port address of the far-end port, 0-31

 Outputs:
    None

 Returns:
    Returns MTD_OK or MTD_FAIL if force speed fails
    
 Description:
    This sample shows how to force the T-unit to one of the supported speeds. The 
    near-end and far-end ports used in this sample helps to show the linkup status 
    between 2 ports after the force speed is configured. 

    This sample forces both the near-end and far-end ports speed to 100BT speed,
    MTD_SPEED_100M_FD_AN_DIS. After the force speed is completed, it shows how to 
    test the link status.  Lastly it undoes the forced speed, re-enables auto-negotiation
    at the highest common speed, and waits for the link to come back up and tests
    the link status.

    "Forced" means that auto-negotiation is disabled and only that speed attempts
    to train. It is only supported for speeds at 100BT and 10BT.
    
 Side effects:
    None

 Notes/Warnings:
    This sample test requires an MDIO connection between the nearEndPort and 
    farEndPort. This sample code assumes both ends are the same device type.

*******************************************************************************/
MTD_STATUS mtdSampleForceSpeed
(
    IN MTD_U16 nearEndPort,
    IN MTD_U16 farEndPort
)
{
    MTD_DEV_PTR devPtr = deviceUnderTest;
    MTD_U16 speed_to_try = MTD_SPEED_100M_FD_AN_DIS; /* may select only one of these speeds */
    MTD_BOOL nearEndPortLinkUp, farEndPortLinkUp;
    MTD_U16 nearEndPortSpeed,farEndPortSpeed;
    MTD_BOOL nearEndPortIsForced,farEndPortIsForced;
    MTD_U16 timeCount = 0;
    MTD_U16 commonSpeeds, highestBit, lowestBit, highestSpeed;    

    /* Force both sides to this speed, and AN disabled */
    MTD_ATTEMPT(mtdForceSpeed(devPtr,nearEndPort,speed_to_try)); 
    MTD_ATTEMPT(mtdForceSpeed(devPtr,farEndPort,speed_to_try));

    /* Make sure both sides are set to that speed */
    MTD_ATTEMPT(mtdGetForcedSpeed(devPtr,nearEndPort,&nearEndPortIsForced,&nearEndPortSpeed));
    MTD_ATTEMPT(mtdGetForcedSpeed(devPtr,farEndPort,&farEndPortIsForced,&farEndPortSpeed));

    if (nearEndPortIsForced == MTD_FALSE || farEndPortIsForced == MTD_FALSE)
    {
        MTD_DBG_ERROR("mtdSampleForceSpeed: Get Force speed status failed\n");               
        return MTD_FAIL;
    }

    if (nearEndPortSpeed != speed_to_try || farEndPortSpeed != speed_to_try)
    {
        MTD_DBG_ERROR("mtdSampleForceSpeed: Get Force speed mode failed\n");                
        return MTD_FAIL;
    }

    /* Poll mtdIsBaseTUp() for linkup every 0.5 second for up to 10 seconds */
    do
    {
        timeCount++;
        MTD_ATTEMPT(mtdWait(devPtr,500)); 
        MTD_ATTEMPT(mtdIsBaseTUp(devPtr,nearEndPort,&nearEndPortSpeed,&nearEndPortLinkUp));
        MTD_ATTEMPT(mtdIsBaseTUp(devPtr,farEndPort,&farEndPortSpeed,&farEndPortLinkUp));
    } while ((nearEndPortLinkUp == MTD_FALSE || farEndPortLinkUp == MTD_FALSE) && timeCount < 20);

    if (nearEndPortSpeed != speed_to_try || farEndPortSpeed != speed_to_try ||
        nearEndPortLinkUp != MTD_TRUE || farEndPortLinkUp != MTD_TRUE)
    {
        MTD_DBG_ERROR("mtdSampleForceSpeed: Force speed failed\n");                
        return MTD_FAIL;
    }

    MTD_DBG_INFO("mtdSampleForceSpeed: Force speed set to: 0x%X\n", nearEndPortSpeed);
    MTD_DBG_INFO("mtdSampleForceSpeed: Force speed link up in %d seconds\n", timeCount/2);

    /* Undo the forced speed */
    MTD_ATTEMPT(mtdUndoForcedSpeed(devPtr,nearEndPort,MTD_FALSE)); /* Calls mtdAutonegEnable() */
    MTD_ATTEMPT(mtdUndoForcedSpeed(devPtr,farEndPort,MTD_FALSE)); /* Calls mtdAutonegEnable() */

    /* use MTD_SPEED_ALL_33X0 to include 5G/2.5G speeds for 88X33X0 parts */
    /* use MTD_SPEED_ALL_21X0_2XX0 to advertise 5G and below only */
    MTD_ATTEMPT(mtdGetSpeedsInCommon(devPtr, nearEndPort, devPtr, farEndPort, &commonSpeeds, &highestBit, &lowestBit, &highestSpeed));

    MTD_ATTEMPT(mtdEnableSpeeds(devPtr,nearEndPort,highestSpeed,MTD_TRUE)); /* Uses mtdAutonegRestart() */
    MTD_ATTEMPT(mtdEnableSpeeds(devPtr,farEndPort,highestSpeed,MTD_TRUE)); /* Uses mtdAutonegRestart() */
    
    /* Poll mtdIsBaseTUp() for linkup every 0.5 second for up to 10 seconds */
    timeCount = 0;
    do
    {
        timeCount++;
        MTD_ATTEMPT(mtdWait(devPtr,500)); 
        MTD_ATTEMPT(mtdIsBaseTUp(devPtr,nearEndPort,&nearEndPortSpeed,&nearEndPortLinkUp));
        MTD_ATTEMPT(mtdIsBaseTUp(devPtr,farEndPort,&farEndPortSpeed,&farEndPortLinkUp));
    } while ((nearEndPortLinkUp == MTD_FALSE || farEndPortLinkUp == MTD_FALSE) && timeCount < 20);

    if (nearEndPortSpeed != highestSpeed || farEndPortSpeed != highestSpeed ||
        nearEndPortLinkUp != MTD_TRUE || farEndPortLinkUp != MTD_TRUE)
    {
        MTD_DBG_ERROR("mtdSampleForceSpeed: Undo Force speed failed\n");                
        return MTD_FAIL;
    }

    MTD_DBG_INFO("mtdSampleForceSpeed: Undo Force speed set to: 0x%X\n", nearEndPortSpeed);
    MTD_DBG_INFO("mtdSampleForceSpeed: Undo Force speed link up in %d seconds\n", timeCount/2);

    MTD_DBG_INFO("mtdSampleForceSpeed: Force speed Test completed successfully\n");
    
    return MTD_OK;
}


/******************************************************************************
MTD_STATUS mtdSampleE21X0EnableLEDs
(
    IN MTD_U16 port
);

 Inputs:
    port - MDIO port address, 0-31 

 Outputs:
    none

 Returns:
    MTD_OK if LEDs are successfully configured
    MTD_FAIL if the device is not supported or other failure

 Description:
    This sample function applies to only 88E2110 and 88E2180 PHY devices.  It shows 
    how to enable the LEDs for various activities, statuses, features, etc...  

    For 88E2180 it enables the serial LEDs. 88E2180 has both serial and parallel LED
    capabilities. See mtdSampleE2180EnableParallelLED() for an example of how
    to enable the 88E2180 parallel LEDs.
    
    The following are some of the LED registers to configure the LEDs. Refer to the 
    datasheet for the complete details on these registers and other LED registers:
        31.0xF03A - LED Control Register is a global register. A write to 31.F03A on 
        any port will update all 8 ports on the 88E2180
        31.0xF020 - LED[0] Control
        31.0xF021 - LED[1] Control
        31.0xF022 - LED[2] Control

    This sample sets the LED1 to turn ON when port is running at 5G speed and 
    LED2 to turn ON when port is running at 2.5G speed.

 Side effects:
    None

 Notes/Warnings:
    On 88E2110 and 88E2180 the LEDs are disabled by default. They must be
    configured prior to use.
******************************************************************************/
MTD_STATUS mtdSampleE21X0EnableLEDs
(
    IN MTD_U16 port
)
{
    MTD_DEV_PTR devPtr = &e2110deviceA0;    

    if (!(MTD_IS_E21X0_BASE(devPtr->deviceId)))
    {
        MTD_DBG_ERROR("mtdSampleE21X0EnableLEDs: Device is not supported.\n");                
        return MTD_FAIL;
    }

    /* if device is not a 1 or 8-port device, it's not supported */
    if (!((devPtr->numPortsPerDevice == 1) || (devPtr->numPortsPerDevice == 8)))
    {
        MTD_DBG_ERROR("mtdSampleE21X0EnableLEDs: Device with port count:%d is not supported.\n", devPtr->numPortsPerDevice);                
        return MTD_FAIL;
    }

    /* enables all 3 LEDs for E2110 and E2180 on 31.F03A; 31.F03A is a global register; 
       A write to 31.F03A to any port will update all 8 ports */
    MTD_ATTEMPT(mtdHwXmdioWrite(devPtr, port, MTD_C_UNIT_GENERAL, CUNIT_SLED_CONTROL, 0x0112));

    if (devPtr->numPortsPerDevice == 1) /* E2110 */
    {
        /* LED1 turns on when linkup on 5G, 31.F021 */
        MTD_ATTEMPT(mtdHwXmdioWrite(devPtr, port, MTD_C_UNIT_GENERAL, CUNIT_LED1_CONTROL, 0x00B0));

        /* LED2 turns on when linkup on 2.5G, 31.F022 */
        MTD_ATTEMPT(mtdHwXmdioWrite(devPtr, port, MTD_C_UNIT_GENERAL, CUNIT_LED2_CONTROL, 0x00A8));

    }
    else /* E2180 */
    {
        MTD_U16 numPortsPerDevice, basePort, portIndex;

        numPortsPerDevice = devPtr->numPortsPerDevice;

        /* calculates the MDIO port number of the 1st(base) port */
        basePort = (port/numPortsPerDevice) * numPortsPerDevice;   
        for (portIndex = basePort; portIndex < basePort + numPortsPerDevice; portIndex++) 
        {
            /* LED1 turns on when linkup on 5G, 31.F021 */
            MTD_ATTEMPT(mtdHwXmdioWrite(devPtr, portIndex, MTD_C_UNIT_GENERAL, CUNIT_LED1_CONTROL, 0x00B0));

            /* LED2 turns on when linkup on 2.5G, 31.F022 */
            MTD_ATTEMPT(mtdHwXmdioWrite(devPtr, portIndex, MTD_C_UNIT_GENERAL, CUNIT_LED2_CONTROL, 0x00A8));
        }
    }
    
    return MTD_OK;
}


/******************************************************************************
MTD_STATUS mtdSampleE2180EnableParallelLED
(
    IN MTD_U16 port
);

 Inputs:
    port - MDIO port address, 0-31 

 Outputs:
    none

 Returns:
    MTD_OK if LEDs are successfully configured
    MTD_FAIL if the device is not supported or other failure

 Description:
    This sample function applies to only 88E2180 PHY device.  It shows 
    how to enable the parallel LEDs on the 88E2180 for various activities, 
    statuses, features, etc...  
    
    The following are some of the LED registers to configure the LEDs. Refer to the 
    datasheet for the complete details on these registers and other LED registers:
        31.0xF015 - GPIO Interrupt Control
        31.0xF020 - LED[0] Control
        31.0xF021 - LED[1] Control

    This sample sets the parallel LEDs where LED0 turns on for copper link and blinks
    on tx/rx activity, and LED1 turns on for 5G speed (off for lower speeds).

 Side effects:
    None

 Notes/Warnings:
    On 88E2110 and 88E2180 the LEDs are disabled by default. They must be
    configured prior to use. 
*******************************************************************************/
MTD_STATUS mtdSampleE2180EnableParallelLED
(
    IN MTD_U16 port
)
{
    MTD_DEV_PTR devPtr = &e2180deviceA0;
    MTD_U16 numPortsPerDevice, basePort, portIndex;    

    if (!(MTD_IS_E21X0_BASE(devPtr->deviceId)))
    {
        MTD_DBG_ERROR("mtdSampleE2180EnableParallelLED: Device is not supported.\n");
        return MTD_FAIL;
    }

    /* if device is not a 8-port device, it's not supported */
    if (!(devPtr->numPortsPerDevice == 8))
    {
        MTD_DBG_ERROR("mtdSampleE2180EnableParallelLED: Device with port count:%d is not supported.\n", devPtr->numPortsPerDevice);    
        return MTD_FAIL;
    }

    numPortsPerDevice = devPtr->numPortsPerDevice;

    /* calculates the MDIO port number of the 1st(base) port */
    basePort = (port/numPortsPerDevice) * numPortsPerDevice; 

    for (portIndex = basePort; portIndex < basePort + numPortsPerDevice; portIndex++) 
    {
        /* enable 2 LED mode control by GPIO, 31.F015 */
        MTD_ATTEMPT(mtdHwXmdioWrite(devPtr, portIndex, MTD_C_UNIT_GENERAL, CUNIT_GPIO_CONTROL, 0x8800));
        
        /* uses default for LED0, on for copper link, blink for tx/rx activity */

        /* LED1 turns on when linkup on 5G, 31.F021 */
        MTD_ATTEMPT(mtdHwXmdioWrite(devPtr, portIndex, MTD_C_UNIT_GENERAL, CUNIT_LED1_CONTROL, 0x00B0));
    }
    
    return MTD_OK;
}

/******************************************************************************
MTD_STATUS mtdSamplePhyLinkUp
(
    IN MTD_U16 port
);

 Inputs:
    port - MDIO port address, 0-31 

 Outputs:
    none

 Returns:
    MTD_OK - this function always returns OK unless one of the API function calls
    fails, then it returns MTD_FAIL

 Description:
    This sample function shows how to check if the PHY link is fully up. For the
    PHY link to be fully up, the host interface serdes side must be up and
    the copper side must also be link up.

    The place to check for host serdes side link up depends on the speed the
    copper link is up at (this function assumes that the host interface is 
    configured to XFI/SGMII, which means for 10G/5G the BASE-R PCS is used
    and for 2.5G and below the SGMII link status is used.)

    The PHY default configuration is to have the host serdes side link down
    whenever the copper side is link down, so this code checks the copper
    side link up first, then uses the speed at which the copper side is up
    to decide where to read the host serdes side link status.

 Side effects:
    None

 Notes/Warnings:
    This function does not check the receive status of the host-side PCS. For
    data to flow from the line to the host, the host-side PCS must also be
    link up. This status will be on the host side PCS link status register. It
    is possible for the egress link to be up on the host interface serdes while
    the ingress link may be down.

    Alternatively, all of the link status could be checked without looking at
    the speed and the result AND together to see if the PHY link is up or down.

    Example:
    MTD_ATTEMPT(mtdGet1000BXSGMIIStatus(devPtr, port, MTD_H_UNIT, &anComplete, &rmtFault, 
                                        &linkStatusLatchSgmii, &linkStatusCurrentSgmii));
    MTD_ATTEMPT(mtdGet10GBRStatus1(devPtr, port, MTD_H_UNIT, &txLpiLatch, &rxLpiLatch, *txLpiCurrent, 
                                   &rxLpiCurrent, &brFault, &linkStatusLatchBaseR, &linkStatusCurrentBaseR);
    MTD_ATTEMPT(mtdIsBaseTUp(devPtr, port, &portSpeed, &portLinkUp));

    if (portLinkUp && (linkStatusCurrentSgmii || linkStatusCurrentBaseR))
    {
        ...PHY is link up on both interfaces, but the interface speed is not determined using this method...
    }                                                                  
    
*******************************************************************************/
MTD_STATUS mtdSamplePhyLinkUp
(
    IN MTD_U16 port
)
{
    MTD_DEV_PTR devPtr = deviceUnderTest;
    MTD_U16 portSpeed;
    MTD_BOOL portLinkUp, anComplete, rmtFault, linkStatusLatch, linkStatusCurrent;   
    MTD_BOOL txLpiLatch, txLpiCurrent, rxLpiLatch, rxLpiCurrent, brFault;    

    MTD_ATTEMPT(mtdIsBaseTUp(devPtr, port, &portSpeed, &portLinkUp));

    if (portLinkUp == MTD_TRUE)
    {
        switch (portSpeed)
        {
            case MTD_SPEED_10M_HD:
            case MTD_SPEED_10M_FD:
            case MTD_SPEED_100M_HD:
            case MTD_SPEED_100M_FD:
            case MTD_SPEED_1GIG_HD:
            case MTD_SPEED_1GIG_FD:    
            case MTD_SPEED_2P5GIG_FD:
                /* Uses SGMII link status */
                MTD_ATTEMPT(mtdGet1000BXSGMIIStatus(devPtr, port, MTD_H_UNIT, &anComplete, &rmtFault, 
                                                    &linkStatusLatch, &linkStatusCurrent));
                break;
            
            case MTD_SPEED_10GIG_FD:
            case MTD_SPEED_5GIG_FD:
                MTD_ATTEMPT(mtdGet10GBRStatus1(devPtr, port, MTD_H_UNIT, &txLpiLatch, &rxLpiLatch, &txLpiCurrent, 
                                               &rxLpiCurrent, &brFault, &linkStatusLatch, &linkStatusCurrent));
                break;

            default:
                MTD_DBG_ERROR("mtdSamplePhyLinkUp: Incorrect speed value\n");
                break;
        }

        if (linkStatusCurrent == MTD_TRUE)
        {
            MTD_DBG_INFO("mtdSamplePhyLinkUp: Copper side and host serdes link both up. PHY is link up.\n");
        }
        else
        {
            MTD_DBG_ERROR("mtdSamplePhyLinkUp: Copper side is link up but host serdes side link is down. PHY is link down.\n");
        }
    }
    else
    {
        MTD_DBG_ERROR("mtdSamplePhyLinkUp: Copper link is not up. Not checking host side serdes link status.\n");
    }

    return MTD_OK;
}

