/* 
 Copyright (c) 2014-2024 Marvell International Ltd., All Rights Reserved
 Redistribution and use in source and binary forms, with or without modification, 
 are permitted provided that the following conditions are met:

 1. Redistributions of source code must retain the above copyright notice, this list 
    of conditions and the following disclaimer.
 2. Redistributions in binary form must reproduce the above copyright notice, this list 
    of conditions and the following disclaimer in the documentation and/or other materials 
    provided with the distribution.
    
 THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY EXPRESS 
 OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY 
 AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR 
 CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR 
 CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR 
 SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY 
 THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR 
 OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE 
 POSSIBILITY OF SUCH DAMAGE.
*/


#ifndef MTD_USER_H
#define MTD_USER_H

#if  MTD_MARVELL_TEST_ENV 
#include "stdio.h"
#include "stdarg.h"
#include "stdlib.h"
#define  MTD_DEBUG_PRINT /* comment out to remove all print code/data */
#endif

#if C_LINKAGE
#if defined __cplusplus
    extern "C" {
#endif
#endif

#ifdef MTD_DEBUG_PRINT
typedef enum 
{
    MTD_DBG_OFF_LVL,
    MTD_DBG_ERR_LVL,
    MTD_DBG_INF_LVL,
    MTD_DBG_ALL_LVL
} MTD_DBG_LEVEL;

//extern void mtdFuncDbgPrint(char* format, ...);
void mtdDbgPrint(FILE *stream, MTD_DBG_LEVEL debug_level, char* format, ...);

#define MTD_DBG_ERROR(...)       mtdDbgPrint(stderr, MTD_DBG_ERR_LVL, __VA_ARGS__) /* macro for error messages */
#define MTD_DBG_INFO(...)        mtdDbgPrint(stdout, MTD_DBG_ALL_LVL, __VA_ARGS__) /* macro for informational messages */
#define MTD_DBG_CRITIC_INFO(...) mtdDbgPrint(stdout, MTD_DBG_INF_LVL, __VA_ARGS__) /* macro for informational messages */


#else /* MTD_DEBUG_PRINT not defined */
#define MTD_DBG_ERROR(...)
#define MTD_DBG_INFO(...)
#define MTD_DBG_CRITIC_INFO(...)
#endif


#ifdef MTD_PY_DEBUG
#include "..\mtdPythonSrc\tstSTypes.h"
#endif

/*******************************************************************
   PORT THESE FUNCTIONS
 *******************************************************************/

/******************************************************************************
MTD_STATUS mtdHwXmdioWrite
(
    IN MTD_DEV_PTR devPtr,
    IN MTD_U16 port, 
    IN MTD_U16 dev, 
    IN MTD_U16 reg, 
    IN MTD_U16 value
);

 Inputs:
    devPtr - pointer to MTD_DEV of the type of device being accessed
    port - MDIO port address, 0-31 
    dev - MMD device address, 0-31
    reg - MMD register address
    value - data to write

 Outputs:
    None

 Returns:
    MTD_OK - wrote successfully
    MTD_FAIL - an error occurred

 Description:
    Writes a 16-bit word to the MDIO
    Address is in format X.Y.Z, where X selects the MDIO port (0-31), Y selects
    the MMD/Device (0-31), and Z selects the register.

 Side effects:
    None

 Notes/Warnings:
    None

******************************************************************************/
MTD_STATUS mtdHwXmdioWrite
( 
    IN MTD_DEV_PTR devPtr,
    IN MTD_U16 port, 
    IN MTD_U16 dev, 
    IN MTD_U16 reg, 
    IN MTD_U16 value
);

/******************************************************************************
 MTD_STATUS mtdHwXmdioRead
 (
     IN MTD_DEV_PTR devPtr,
     IN MTD_U16 port, 
     IN MTD_U16 dev, 
     IN MTD_U16 reg,
     OUT MTD_U16 *data
 );

 Inputs:
    devPtr - pointer to MTD_DEV of the type of device being accessed
    port - MDIO port address, 0-31 
    dev - MMD device address, 0-31
    reg - MMD register address

 Outputs:
    data - Returns 16 bit word from the MDIO

 Returns:
    MTD_OK - read successful
    MTD_FAIL - read was unsuccessful
    
 Description:
    Reads a 16-bit word from the MDIO
    Address is in format X.Y.Z, where X selects the MDIO port (0-31), Y selects the 
    MMD/Device (0-31), and Z selects the register.

 Side effects:
    None

 Notes/Warnings:
    None

******************************************************************************/
MTD_STATUS mtdHwXmdioRead
(
    IN MTD_DEV_PTR devPtr,
    IN MTD_U16 port, 
    IN MTD_U16 dev, 
    IN MTD_U16 reg,
    OUT MTD_U16 *data
);


/******************************************************************************
MTD_STATUS mtdWait
(
    IN MTD_DEV_PTR devPtr,
    IN MTD_UINT x
);

 Inputs:
    devPtr - pointer to MTD_DEV of the type of device being accessed
    x - number of milliseconds to wait

 Outputs:
    None

 Returns:
    MTD_OK if wait was successful, MTD_FAIL otherwise

 Description:
    Waits X milliseconds

 Side effects:
    None

 Notes/Warnings:
    None

******************************************************************************/
MTD_STATUS mtdWait
(
    IN MTD_DEV_PTR devPtr,
    IN MTD_UINT x
);



/*******************************************************************
   END: PORT THESE FUNCTIONS
 *******************************************************************/
#if MTD_MARVELL_TEST_ENV
/* test code uses these functions and needs to include these */

MTD_STATIC MTD_STATUS mtdHwGetPhyRegField
(
    IN  MTD_DEV_PTR devPtr,
    IN  MTD_U16      port,
    IN  MTD_U16      dev,
    IN  MTD_U16      regAddr,
    IN  MTD_U8       fieldOffset,
    IN  MTD_U8       fieldLength,
    OUT MTD_U16      *data
);


MTD_STATIC MTD_STATUS mtdHwSetPhyRegField
(
    IN MTD_DEV_PTR devPtr,
    IN MTD_U16      port,
    IN MTD_U16      dev,
    IN MTD_U16      regAddr,
    IN MTD_U8       fieldOffset,
    IN MTD_U8       fieldLength,
    IN MTD_U16      data
);


MTD_STATIC MTD_STATUS mtdHwGetRegFieldFromWord
(
    IN  MTD_U16      regData,
    IN  MTD_U8       fieldOffset,
    IN  MTD_U8       fieldLength,
    OUT MTD_U16      *data
);


MTD_STATIC MTD_STATUS mtdHwSetRegFieldToWord
(
    IN  MTD_U16      regData,
    IN  MTD_U16      bitFieldData,
    IN  MTD_U8       fieldOffset,
    IN  MTD_U8       fieldLength,
    OUT MTD_U16      *data
);
#endif

#if C_LINKAGE
#if defined __cplusplus
}
#endif
#endif


#endif

